-- SQL-kode for  generere tabellene til AoC3

-- Starter med  fjerne tabellene

DROP TABLE UserText;
DROP TABLE User;
DROP TABLE TeleportRequirement;
DROP TABLE QuestionCategory;
DROP TABLE QuestionGroup;
DROP TABLE Text;
DROP TABLE TextGroup;
DROP TABLE Message;
DROP TABLE TeleportSource;
DROP TABLE TeleportDest;
DROP TABLE Level;

DROP TABLE AutoGenQuestion;         -- added 26.07.2006
DROP TABLE DarkQuestion;            -- added 26.07.2006
DROP TABLE InputQuestion;           -- added 26.07.2006
DROP TABLE MultiChoiceAlternative;  -- added 26.07.2006
DROP TABLE MultiChoiceQuestion;     -- added 26.07.2006
DROP TABLE Question;                -- added 26.07.2006
DROP TABLE Settings;                -- added 26.07.2006
DROP TABLE UserQuestion;            -- added 26.07.2006

-- Selve tabellene

CREATE TABLE Level(
ID int(11) PRIMARY KEY auto_increment,
Name varchar(50) NOT NULL,
Map varchar(50) NOT NULL);

CREATE TABLE TeleportDest(
ID int(11) PRIMARY KEY auto_increment,
Text longtext,
LevelID int(11) NOT NULL,
PosX int(11) NOT NULL,
PosY int(11) NOT NULL,
PosZ int(11) NOT NULL,
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE TeleportSource(
ID int(11) PRIMARY KEY auto_increment,
TeleportDestID int(11) NOT NULL,
Text longtext,
LevelID int(11) NOT NULL,
PosX int(11) NOT NULL,
PosY int(11) NOT NULL,
PosZ int(11) NOT NULL,
FOREIGN KEY (TeleportDestID) REFERENCES TeleportDest(ID),
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE Message(
ID int(11) PRIMARY KEY auto_increment,
Text longtext NOT NULL,
LevelID int(11) NOT NULL,
PosX int(11) NOT NULL,
PosY int(11) NOT NULL,
PosZ int(11) NOT NULL,
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE TextGroup(
ID int(11) PRIMARY KEY auto_increment,
ParentID int(11),
Name varchar(50) NOT NULL,
Text longtext,
Sequence int(11));

CREATE TABLE Text(
ID int(11) PRIMARY KEY auto_increment,
TextGroupID int(11) NOT NULL,
Name varchar(50),
Text longtext not null,
Latex longtext,
Type ENUM('history','fact'),
LevelID int(11) NOT NULL,
PosX int(11) NOT NULL,
PosY int(11) NOT NULL,
PosZ int(11) NOT NULL,
Sequence int(11),
FOREIGN KEY (TextGroupID) REFERENCES TextGroup(ID),
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE QuestionGroup(
ID int(11) PRIMARY KEY auto_increment,
ParentID int(11),
Name varchar(50) NOT NULL,
Description varchar(50));

ALTER TABLE QuestionGroup ADD CONSTRAINT questiongroup_ibfk_1 FOREIGN KEY (ParentID)
    REFERENCES QuestionGroup(ID)
    ON DELETE RESTRICT
    ON UPDATE RESTRICT;

CREATE TABLE QuestionCategory(
ID int(11) PRIMARY KEY auto_increment,
QuestionGroupID int(11) NOT NULL,
Name varchar(50) NOT NULL,
Description varchar(50),
Text longtext,
Latex longtext,
QuestionsRequired int(11) NOT NULL,
LevelID int(11) NOT NULL,
PosX int(11) NOT NULL,
PosY int(11) NOT NULL,
PosZ int(11) NOT NULL,
FOREIGN KEY (QuestionGroupID) REFERENCES QuestionGroup(ID),
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE TeleportRequirement(
TeleportSourceID int(11),
QuestionCategoryID int(11),
PRIMARY KEY (TeleportSourceID, QuestionCategoryID),
FOREIGN KEY (TeleportSourceID) REFERENCES TeleportSource(ID),
FOREIGN KEY (QuestionCategoryID) REFERENCES QuestionCategoryID(ID));

CREATE TABLE User(
ID int(11) PRIMARY KEY auto_increment,
Username varchar(50) NOT NULL,
Password varchar(50) NOT NULL,
Admin int(1) NOT NULL,
Avatar int(11) NOT NULL,
LevelID int(11) NOT NULL DEFAULT '1',
PosX int(11) NOT NULL DEFAULT '1095',
PosY int(11) NOT NULL DEFAULT '633',
PosZ int(11) NOT NULL DEFAULT '0',
FOREIGN KEY (LevelID) REFERENCES Level(ID));

CREATE TABLE UserText(
UserID int(11),
TextID int(11),
PRIMARY KEY (UserID, TextID),
FOREIGN KEY (UserID) REFERENCES User(ID),
FOREIGN KEY (TextID) REFERENCES Text(ID));

CREATE TABLE Question(
ID int(11) PRIMARY KEY auto_increment,
QuestionCategoryID int(11) NOT NULL,
Text longtext NOT NULL,
Latex longtext,
Value int(11) NOT NULL,
Comment longtext,
Type ENUM('input','autogen','dark','multichoice'),
FOREIGN KEY (QuestionCategoryID) REFERENCES QuestionCategory(ID));

CREATE TABLE AutoGenQuestion(
ID int(11) PRIMARY KEY,
Code longtext NOT NULL,
OldCode longtext,
formula varchar(255),
FOREIGN KEY (ID) REFERENCES Question(ID));

CREATE TABLE DarkQuestion(
ID int(11) PRIMARY KEY,
Code longtext NOT NULL,
OldCode longtext,
TestCode longtext,
SuggestedSolution longtext,
FOREIGN KEY (ID) REFERENCES Question(ID));

CREATE TABLE InputQuestion(
ID int(11) PRIMARY KEY,
Answer varchar(50) NOT NULL,
FOREIGN KEY (ID) REFERENCES Question(ID));

CREATE TABLE MultiChoiceQuestion(
ID int(11) PRIMARY KEY,
AnswerID int(11),
FOREIGN KEY (ID) REFERENCES Question(ID));

CREATE TABLE MultiChoiceAlternative(
QuestionID int(11),
AlternativeID int(11),
Text longtext NOT NULL,
LaTeX longtext NOT NULL,
PRIMARY KEY (QuestionID, AlternativeID),
FOREIGN KEY (QuestionID) REFERENCES MultiChoiceQuestion(ID));

ALTER TABLE MultiChoiceQuestion ADD CONSTRAINT FK_multichoicequestion_2 FOREIGN KEY (ID, AnswerID)
    REFERENCES MultiChoiceAlternative (QuestionID, AlternativeID)
    ON DELETE RESTRICT
    ON UPDATE RESTRICT;

CREATE TABLE UserQuestion(
UserID int(11),
QuestionID int(11),
Date datetime NOT NULL,
Correct int(1) NOT NULL,
Params blob,
Answer varchar(255),
TimesAnswered int(4),
PRIMARY KEY (UserID, QuestionID, Date),
FOREIGN KEY (UserID) REFERENCES User(ID),
FOREIGN KEY (QuestionID) REFERENCES Question(ID));

CREATE TABLE Settings (
Name varchar(30) NOT NULL,
Value varchar(30),
PRIMARY KEY(Name));


INSERT INTO Settings values('DoNotShowQuestionGroup', '2');         -- added 26.07.2006
INSERT INTO Settings values('DoNotShowTextGroup', '1026');          -- added 26.07.2006
INSERT INTO Settings values('ExamMode', 'off');                     -- added 26.07.2006
INSERT INTO Settings values('LastQuestionID', '445');               -- added 26.07.2006
INSERT INTO Settings values('QuestionGroupRoot', '146');            -- added 26.07.2006
INSERT INTO Settings values('TextGroupRoot', '1023');               -- added 26.07.2006
INSERT INTO Settings values('StyleSheet', 'style.css');             -- added 27.07.2006


-- The following are required entries in various tables.
-- The required attributes are commented.
-- The others can most likely be changed...

-- ID=QuestionGroupRoot, ParentID=NULL
INSERT INTO QuestionGroup values(146, NULL, 'Alle', '');            -- added 26.07.2006
-- ID=DoNotShowQuestionGroup, ParentID=146
INSERT INTO QuestionGroup values(2, 146, 'Unused', '');             -- added 26.07.2006

-- ID=TextGroupRoot, ParentID=0
INSERT INTO TextGroup values(1023, 0, 'Age of Computers', '', 10);  -- added 26.07.2006
-- ID=1000, ParentID=TextGroupRoot
INSERT INTO TextGroup values(1000, 1023, 'Factbook', '', 10);       -- added 27.07.2006
-- ID=1001, ParentID=TextGroupRoot
INSERT INTO TextGroup values(1001, 1023, 'Historybook', '', 20);    -- added 27.07.2006
-- ID=DoNotShowTextGroup, ParentID=TextGroupRoot
INSERT INTO TextGroup values(1026, 1023, 'Unused', '', 30);         -- added 26.07.2006

-- Map='undefined'
INSERT INTO Level values(100, 'Testlevel', 'undefined');            -- added 26.07.2006

