package aoc3.client;

import aoc3.db.User;
import aoc3.event.*;
import aoc3.net.ClientToServer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.apr.2005
 * Time: 17:50:38
 * To change this template use File | Settings | File Templates.
 */
final class ChatPanel extends JPanel implements ChatEventListener, ActionListener, KeyEventDispatcher
{
  private static final int MAX_HISTORY_SIZE = 3;
  private static final String SETTINGS_PATH = "data/settings/adminCommands.txt";

  private final Calendar calendar = new GregorianCalendar();
  private final ClientToServer connection;
  private final JTextArea taChatbox = new JTextArea();
  private final JTextField tfChatline = new JTextField();
  private final User localUser;

  private final List<String> commandHistory;
  private int currentCommand;

  private final List<String> adminCommands;
  private boolean tabcompAvailable;

  ChatPanel(User localUser, ClientToServer connection)
  {
    super(new BorderLayout(2, 2));
    this.localUser = localUser;
    this.connection = connection;
    connection.addChatListener(this);

    UserListModel userListModel = new UserListModel(localUser);
    connection.addUserListener(userListModel);
    JList userList = new JList(userListModel);
    userList.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
    userList.setCellRenderer(new UserListRenderer());
    JScrollPane scrollUsers = new JScrollPane(userList);
    scrollUsers.setPreferredSize(new Dimension(150, 150));

    taChatbox.setEditable(false);
    taChatbox.setBorder(BorderFactory.createEmptyBorder(1, 3, 1, 3));
    taChatbox.setLineWrap(true);
    taChatbox.setWrapStyleWord(true);
    JScrollPane scrollChat = new JScrollPane(taChatbox);
    scrollChat.setPreferredSize(new Dimension(500, 150));
    tfChatline.addActionListener(this);
    taChatbox.setBackground(Color.BLACK);
    taChatbox.setForeground(Color.WHITE);
    userList.setBackground(Color.BLACK);
    userList.setForeground(Color.WHITE);
    tfChatline.setBackground(Color.BLACK);
    tfChatline.setForeground(Color.WHITE);
    tfChatline.setCaretColor(Color.WHITE);
    setBackground(Color.BLACK);

    add(scrollChat, BorderLayout.CENTER);
    add(tfChatline, BorderLayout.SOUTH);
    add(scrollUsers, BorderLayout.EAST);
    setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));

    KeyboardFocusManager keyboard = KeyboardFocusManager.getCurrentKeyboardFocusManager();
    keyboard.addKeyEventDispatcher(this);

    commandHistory = new ArrayList<String>();
    currentCommand = 0;

    adminCommands = new ArrayList<String>();
    try {
      URL settingsFileURL = new URL("http://" + Client.getServer() + '/' + SETTINGS_PATH);
      BufferedReader reader = new BufferedReader(new InputStreamReader(settingsFileURL.openStream()));
      String temp;
      while ((temp = reader.readLine()) != null)
        adminCommands.add(temp);

      tabcompAvailable = true;
    }
    catch (Exception e) {
      e.printStackTrace();
      tabcompAvailable = false;
    }
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == tfChatline) {
      if (tfChatline.getText().length() > 0) {

//        if(tfChatline.getText().equals("/exception")){
//          try{
//            throw new Exception("Testexception");
//          }
//          catch(Exception e){
//            e.printStackTrace();
//          }
//        }

        ChatEvent chatEvent = new ChatEvent(localUser.getUsername(), new Date(System.currentTimeMillis()),
                                            tfChatline.getText());
        connection.fireRemoteChatEvent(chatEvent);

        // remove old commands from list
        if (commandHistory.size() > MAX_HISTORY_SIZE - 1) {
          while (commandHistory.size() > MAX_HISTORY_SIZE - 1)
            commandHistory.remove(commandHistory.size() - 1);
        }

        currentCommand = 0;
        commandHistory.add(0, tfChatline.getText());

        if (!tfChatline.getText().startsWith("/")) {
          chatLine(chatEvent);
        }

        tfChatline.setText("");
      }
    }
  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface KeyEventDispatcher ---------------------

  public boolean dispatchKeyEvent(KeyEvent e)
  {

    if (e.getSource() == tfChatline && e.getID() == KeyEvent.KEY_RELEASED) {

      if (KeyEvent.getKeyText(e.getKeyCode()).equals("Space") &&
          e.getModifiers() == KeyEvent.CTRL_MASK) {

        if (localUser.isAdmin() && tabcompAvailable && tfChatline.getText().startsWith("/")) {
          List<String> matches = new ArrayList<String>();
          int prefixLength = tfChatline.getText().length();
          for (String command : adminCommands) {
            if (command.length() > prefixLength) {
              String prefix = command.substring(0, prefixLength);
              if (tfChatline.getText().equals(prefix))
                matches.add(command);
            }
          }
          if (matches.size() == 1) {
            tfChatline.setText(matches.get(0));
          }
          else if (matches.size() == 2) {
            int len0 = matches.get(0).length();
            int len1 = matches.get(1).length();

            if (len0 > len1) {
              String prefix = matches.get(0).substring(0, len1);
              if (prefix.equals(matches.get(1)))
                tfChatline.setText(prefix);
            }
            else if (len0 < len1) {
              String prefix = matches.get(1).substring(0, len0);
              if (prefix.equals(matches.get(0)))
                tfChatline.setText(prefix);
            }
          }
        }
        return true;
      }
      else if (KeyEvent.getKeyText(e.getKeyCode()).equals("P") &&
               e.getModifiers() == KeyEvent.CTRL_MASK) {

        if (currentCommand < commandHistory.size()) {
          tfChatline.setText(commandHistory.get(currentCommand));
          currentCommand++;
        }

        return true;
      }
      else if (KeyEvent.getKeyText(e.getKeyCode()).equals("F") &&
               e.getModifiers() == KeyEvent.CTRL_MASK) {

        int caretPos = tfChatline.getCaretPosition();
        if (caretPos < tfChatline.getText().length()) {
          tfChatline.setCaretPosition(caretPos + 1);
        }

        return true;
      }
      else if (KeyEvent.getKeyText(e.getKeyCode()).equals("B") &&
               e.getModifiers() == KeyEvent.CTRL_MASK) {

        int caretPos = tfChatline.getCaretPosition();
        if (caretPos > 0) {
          tfChatline.setCaretPosition(caretPos - 1);
        }

        return true;
      }
    }

    return false;
  }

// --------------------- End Interface KeyEventDispatcher ---------------------

// --------------------- Begin Interface ChatEventListener ---------------------

  public synchronized void chatLine(ChatEvent event) // Blir aksessert b�de av GUI- og nett-tr�d
  {
    calendar.setTime(event.getTime());
    String hour = Integer.toString(calendar.get(Calendar.HOUR_OF_DAY));
    String minute = Integer.toString(calendar.get(Calendar.MINUTE));
    if (hour.length() == 1)
      hour = '0' + hour;
    if (minute.length() == 1)
      minute = '0' + minute;
    String dateString = '[' + hour + ':' + minute + "] ";
    taChatbox.append('\n' + dateString + event.getUsername() + ": " + event.getMessage());
    taChatbox.setCaretPosition(taChatbox.getText().length());
  }

// --------------------- End Interface ChatEventListener ---------------------

  void focusChatline()
  {
    tfChatline.requestFocusInWindow();
  }

// -------------------------- INNER CLASSES --------------------------

  private static final class UserListModel extends AbstractListModel implements UserEventListener
  {
    private final List<User> users = new ArrayList<User>();

    private UserListModel(User thisUser)
    {
      users.add(thisUser);
    }

    public synchronized void userChanged(UserEvent event)
    {
      if (event.getUser() == null)
        return;
      if (event.getType() == UserEvent.USER_CONNECTED) {
        users.add(event.getUser());
        fireIntervalAdded(event, users.size() - 1, users.size());
      }
      else if (event.getType() == UserEvent.USER_DISCONNECTED) {
        int index = users.indexOf(event.getUser());
        users.remove(event.getUser());
        fireIntervalRemoved(event, index, index + 1);
      }
    }

    public synchronized int getSize()
    {
      return users.size();
    }

    public synchronized Object getElementAt(int index)
    {
      if (index >= users.size())
        return null;
      return users.get(index);
    }
  }

  private static final class UserListRenderer extends DefaultListCellRenderer
  {
    public Component getListCellRendererComponent(JList list, Object value, int index,
                                                  boolean isSelected, boolean cellHasFocus)
    {
      super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
      if (value instanceof User && ((User)value).isAdmin())
        setForeground(Color.RED);
      else
        setForeground(Color.WHITE);
      return this;
    }
  }
}

