package aoc3.client;

import aoc3.client.book.QuestionBookPanel;
import aoc3.client.book.TextBookPanel;
import aoc3.db.*;
import aoc3.event.*;
import aoc3.net.ClientConnection;
import aoc3.net.ClientToServer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 15:56:46
 * To change this template use File | Settings | File Templates.
 */
public final class Client extends JFrame implements LoginEventListener, HTMLEventListener, ActionListener,
                                                    QuestionEventListener, AnswerEventListener,
                                                    QuestionBookListener, TextBookListener
{
  public static final String ACTION_BACK = "Back";
  private static final JTextArea consoletext = new JTextArea();
  private static String SERVER;
  private static final String CARD_TEXT = "Text";
  private static final String CARD_QUESTION = "Question";
  private static final String CARD_MAP = "Map";
  private static final String CARD_BOOK_TEXT = "BookText";
  private static final String CARD_BOOK_QUESTION = "BookQuestion";
  private static final String CARD_INNER = "Inner";
  private final CardLayout cards = new CardLayout();
  private final CardLayout cardsInner = new CardLayout();
  private final JPanel mainPanel = new JPanel(cards);
  private final JPanel innerPanel = new JPanel(cardsInner);
  private final DebugConsole debugConsole = new DebugConsole();
  private ChatPanel chatPanel;
  private ClientToServer connection;

  private static final boolean REPORT_EXCEPTIONS = false;

  public static JTextArea getConsoleText()
  {
    return consoletext;
  }

  public static String getServer()
  {
    return SERVER;
  }

  public static void main(String args[])
  {
    SERVER = args[0];
    new Client();
  }

  private Client()
  {
    super("Age of Computers 3");

    // Noen exceptions kommer så ofte at folk ikke får spilt
    // Dette er en hurtigfiks
    if(REPORT_EXCEPTIONS){
      System.setErr(new ConsoleStream(System.err, consoletext));
    }

    try {
      connection = new ClientConnection();
    } catch (IOException e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(null, "F�r ikke kontakt med tjener", "Kritisk feil",
                                    JOptionPane.ERROR_MESSAGE);
      System.exit(20);
    }
    connection.addLoginListener(this);

    LoginPanel loginPanel = new LoginPanel(this, connection);
    setContentPane(loginPanel);
    connection.startListening();

    setSize(1024, 768);
    Dimension screen = getToolkit().getScreenSize();
    int x = (screen.width - getSize().width) / 2;
    int y = (screen.height - getSize().height) / 2;
    setLocation(Math.max(0, x), Math.max(0, y));

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
        connection.logout();
      }
    });

    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setVisible(true);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    if (event.getActionCommand().equals(ACTION_BACK))
      showMap();
  }

// --------------------- End Interface ActionListener ---------------------


// --------------------- Begin Interface AnswerEventListener ---------------------

  public void answerEvaluated(boolean wasCorrect, boolean canRetry, int newScore)
  {
    if (wasCorrect)
      JOptionPane.showMessageDialog(this, "Svaret var riktig!", "", JOptionPane.INFORMATION_MESSAGE);
    else if (!canRetry)
      JOptionPane.showMessageDialog(this, "Svaret var galt!", "", JOptionPane.ERROR_MESSAGE);
    if (wasCorrect || !canRetry)
      showMap();
  }

// --------------------- End Interface AnswerEventListener ---------------------


// --------------------- Begin Interface HTMLEventListener ---------------------

  public void textReceived(Text text)
  {
    cards.show(mainPanel, CARD_TEXT);
  }

// --------------------- End Interface HTMLEventListener ---------------------


// --------------------- Begin Interface LoginEventListener ---------------------

  public void loggedIn(LoginEvent event)
  {
    if (event.getType() == LoginEvent.LOGIN_SUCCESS)
      init(event.getUser(), event.getScore());
    else if (event.getType() == LoginEvent.LOGIN_ALREADY_LOGGED_ON)
      JOptionPane.showMessageDialog(this, "Denne brukeren er allerede p�logget", "Innlogging",
                                    JOptionPane.ERROR_MESSAGE);
    else if (event.getType() == LoginEvent.LOGIN_UNKNOWN_USER)
      JOptionPane.showMessageDialog(this, "Ukjent brukernavn", "Innlogging",
                                    JOptionPane.ERROR_MESSAGE);
    else if (event.getType() == LoginEvent.LOGIN_WRONG_PASSWORD)
      JOptionPane.showMessageDialog(this, "Galt passord", "Innlogging",
                                    JOptionPane.ERROR_MESSAGE);
    else if (event.getType() == LoginEvent.LOGIN_ERROR)
      JOptionPane.showMessageDialog(this, "Brukeren din er p� et mappylag som ikke eksisterer. Ta kontakt med fagstaben!", "Innlogging",
                                    JOptionPane.ERROR_MESSAGE);
  }

  public void loggedOut()
  {
    JOptionPane.showMessageDialog(this, "Mistet kontakten med tjeneren", "Tvungen avslutting",
                                  JOptionPane.ERROR_MESSAGE);
    System.exit(0);
  }

  public void highScoreReceived(List<Highscore> highScore, List<Highscore> highScoreWeek)
  {
  }

// --------------------- End Interface LoginEventListener ---------------------


// --------------------- Begin Interface QuestionBookListener ---------------------

  public void questionsReceived(Map<UserQuestion, Question> completedQuestions,
                                Map<UserQuestion, String> autogenQuestionTexts,
                                Map<UserQuestion, String> autogenFormulas,
                                Map<UserQuestion, String> darkSuggestedSolution)
  {
    cards.show(mainPanel, CARD_INNER);
    cardsInner.show(innerPanel, CARD_BOOK_QUESTION);
  }

  public void categoriesAndGroupsReceived(Map<Integer, QuestionCategory> questionCategories,
                                          Map<Integer, QuestionGroup> questionGroups)
  {
  }

// --------------------- End Interface QuestionBookListener ---------------------


// --------------------- Begin Interface QuestionEventListener ---------------------

  public void questionReceived(QuestionEvent event)
  {
    cards.show(mainPanel, CARD_QUESTION);
  }

// --------------------- End Interface QuestionEventListener ---------------------


// --------------------- Begin Interface TextBookListener ---------------------

  public void textsReceived(int type, List<Text> visitedTexts, Map<Integer, TextGroup> textGroups)
  {
    cards.show(mainPanel, CARD_INNER);
    cardsInner.show(innerPanel, CARD_BOOK_TEXT);
  }

// --------------------- End Interface TextBookListener ---------------------

  private void init(User user, int score)
  {
    connection.addHTMLListener(this);
    connection.addQuestionListener(this);
    connection.addAnswerEventListener(this);
    connection.addQuestionBookListener(this);
    connection.addTextBookListener(this);

    HtmlPanel htmlPanel = new HtmlPanel(this, connection);
    mainPanel.add(htmlPanel, CARD_TEXT);

    QuestionPanel questionPanel = new QuestionPanel(this, connection);
    mainPanel.add(questionPanel, CARD_QUESTION);

    QuestionBookPanel qBookPanel = new QuestionBookPanel(connection);
    innerPanel.add(qBookPanel, CARD_BOOK_QUESTION);

    TextBookPanel tBookPanel = new TextBookPanel(connection);
    innerPanel.add(tBookPanel, CARD_BOOK_TEXT);

    LevelPanel levelPanel = new LevelPanel(connection, user);
    innerPanel.add(levelPanel, CARD_MAP);

    TopPanel topPanel = new TopPanel(this, connection, score);
    JPanel tempPanel = new JPanel(new BorderLayout());
    tempPanel.add(topPanel, BorderLayout.NORTH);
    tempPanel.add(innerPanel, BorderLayout.CENTER);

    mainPanel.add(tempPanel, CARD_INNER);

    cards.show(mainPanel, CARD_INNER);
    cardsInner.show(innerPanel, CARD_MAP);

    chatPanel = new ChatPanel(user, connection);

    JPanel panel = new JPanel(new BorderLayout(2, 2));
    panel.setBackground(Color.BLACK);
    panel.add(mainPanel, BorderLayout.CENTER);
    panel.add(chatPanel, BorderLayout.SOUTH);
    setContentPane(panel);
    panel.revalidate();
    panel.repaint();
    chatPanel.focusChatline();
  }

  void showMap()
  {
    cards.show(mainPanel, CARD_INNER);
    cardsInner.show(innerPanel, CARD_MAP);
    chatPanel.focusChatline();
  }

// -------------------------- INNER CLASSES --------------------------

  private final class ConsoleStream extends PrintStream
  {
    private final JTextArea text;

    private ConsoleStream(OutputStream out, JTextArea text)
    {
      super(out);
      this.text = text;
      text.append("Fint om du kan sende feilmeldingen under til fagstaben\n");
      text.append("P� forh�nd takk!.\n\n");
    }

    public void write(byte buf[], int off, int len)
    {
      super.write(buf, off, len);
      try {
        text.append(new String(buf, off, len));
        debugConsole.setVisible(true);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
}