package aoc3.client;

import aoc3.db.*;
import aoc3.event.*;
import aoc3.net.ClientToServer;
import com.alienfactory.javamappy.*;
import com.alienfactory.javamappy.loader.MapLoader;
import com.alienfactory.javamappy.viewer.LayerViewer;
import com.alienfactory.javamappy.viewer.MapViewer;
import com.alienfactory.javamappy.viewer.render.J2SE14Renderer;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 18.apr.2005
 * Time: 14:10:28
 * To change this template use File | Settings | File Templates.
 */
final class LevelPanel extends JPanel implements UserEventListener, LevelEventListener, PositionChangeListener
{
//    private static final int DATA_STAIRS = 1;
  //    private static final int INDEX_TYPE = 1;
  private static final String MAPPATH = "/data/map/";
  private final ClientToServer connection;
  private final LocalUserSprite localUserSprite;
  private final Map<Integer, StaticAnimation> staticAnimations = new HashMap<Integer, StaticAnimation>();
  private final Map<Integer, UserSprite> otherUserSprites = new HashMap<Integer, UserSprite>();
  private final User localUser;
  private Level currentLevel;
  private Map<Integer, List<LevelItem>> levelItems;
  private MapViewer mapViewer;
  private final Thread paintThread;
  private com.alienfactory.javamappy.Map currentMap;

  private static boolean isSolidBlock(Layer userLayer, int tileX, int tileY)
  {
    Block tile = userLayer.getBlock(tileX, tileY);
    if (tile.getCollisionFlag(Block.LEFT | Block.TOP))
      return true;
    return userLayer.getBlockIndex(tileX, tileY) == 0;  // Blokk 0 er ikke passerbar
  }

  LevelPanel(ClientToServer connection, User localUser)
  {
    this.connection = connection;
    this.localUser = localUser;
    localUserSprite = new LocalUserSprite(this, localUser);
    paintThread = new Thread()
    {
      public void run()
      {
        while (true) {
          repaint();
          try {
            Thread.sleep(150);
          } catch (InterruptedException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
          }
        }
      }
    };
    connection.addUserListener(this);
    connection.addLevelListener(this);
    connection.addPositionChangeListener(this);
    connection.requestLevel(localUser.getLevelid());
  }

// --------------------- Begin Interface LevelEventListener ---------------------

  public synchronized void levelReceived(Level level, Map<Integer, List<LevelItem>> levelItems)
  {
    if (level.getId() == localUser.getLevelid()) {
      currentLevel = level;
      this.levelItems = levelItems;
      for (Integer animNr : levelItems.keySet()) {
        if (!staticAnimations.containsKey(animNr))
          staticAnimations.put(animNr, new StaticAnimation(animNr));
      }
      try {
        URL mapUrl = new URL("http://" + Client.getServer() + MAPPATH + level.getMap());
        BufferedInputStream inputStream = new BufferedInputStream(mapUrl.openStream());
        currentMap = MapLoader.loadMap(inputStream);
        inputStream.close();
      } catch (IOException e) {
        e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      }
      if (!paintThread.isAlive())
        paintThread.start();
    }
    else
      System.err.println("Fikk tilsendt galt brett: " + level.getId() + " (Bruker: " + localUser.getLevelid() + ')');
  }

  public synchronized void levelItemRemoved(LevelItem levelItem)
  {
    for (List<LevelItem> items : levelItems.values())
      items.remove(levelItem);
  }

  public synchronized void levelItemChanged(LevelItem levelItem, int newAnimNr)
  {
    for (Integer animNr : levelItems.keySet()) {
      List<LevelItem> items = levelItems.get(animNr);
      if (animNr.equals(newAnimNr))
        items.add(levelItem);
      else
        items.remove(levelItem);
    }
  }

// --------------------- End Interface LevelEventListener ---------------------

// --------------------- Begin Interface PositionChangeListener ---------------------

  public synchronized void positionChanged(int userID, int posx, int posy)
  {
    User user = otherUserSprites.get(userID).getUser();
    if (user == null)
      System.out.println("Kunne ikke finne bruker: " + userID);
    else
      user.moveTo(posx, posy);
  }

  public synchronized void positionChanged(int userID, int levelID, int posx, int posy, int posz)
  {
    if (userID == localUser.getId()) {
      localUser.moveTo(levelID, posx, posy, posz);
      localUserSprite.reset();
    }
    else {
      User user = otherUserSprites.get(userID).getUser();
      if (user == null)
        System.out.println("Kunne ikke finne bruker: " + userID);
      else
        user.moveTo(levelID, posx, posy, posz);
    }
  }

// --------------------- End Interface PositionChangeListener ---------------------

// --------------------- Begin Interface UserEventListener ---------------------

  public synchronized void userChanged(UserEvent event)
  {
    if (event.getUser() == null)
      System.err.print("userChanged with user = null");
    else if (event.getType() == UserEvent.USER_CONNECTED)
      otherUserSprites.put(event.getUser().getId(), new UserSprite(event.getUser()));
    else if (event.getType() == UserEvent.USER_DISCONNECTED)
      otherUserSprites.remove(event.getUser().getId());
  }

// --------------------- End Interface UserEventListener ---------------------

  protected void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    synchronized (this) {
      if (currentMap == null)
        return;
      if (mapViewer == null ||
          mapViewer.getMap() != currentMap ||
          mapViewer.getViewHeight() != getHeight() ||
          mapViewer.getViewWidth() != getWidth())
        mapViewer = new MapViewer(currentMap, new J2SE14Renderer(currentMap), getWidth(), getHeight());

      Graphics2D g2d = (Graphics2D)g;
      Rectangle clip = g2d.getClipBounds();
      mapViewer.updateAnimBlocks();

      g.setColor(Color.BLACK);
      g.fillRect(0, 0, getWidth(), getHeight());

      MapHeader header = mapViewer.getMap().getMapHeader();
      int nViewX = Math.min(Math.max(0, localUser.getPosx() - clip.width / 2),
                            header.getMapBlockWidth() * header.getBlockWidth() - clip.width);
      int nViewY = Math.min(Math.max(0, localUser.getPosy() - clip.height / 2),
                            header.getMapBlockHeight() * header.getBlockHeight() - clip.height);

      long currentTime = System.currentTimeMillis();
      StaticAnimation.setCurrenTimeMillis(currentTime);
      performCollisionDetection(localUserSprite.getUpdatedPosition(currentTime));

      AffineTransform oIdentity = new AffineTransform();
      oIdentity.setToIdentity();

      AffineTransform oTransform = new AffineTransform();
      oTransform.setToTranslation(-nViewX, -nViewY);

      boolean positionChanged = false;
      int numLayers = mapViewer.getMap().getLayers().length;

      for (int layer = 0; layer < numLayers; layer++) {
        g2d.setTransform(oIdentity);

        LayerViewer layerViewer = mapViewer.getLayerViewers()[layer];
        layerViewer.setPixelX(nViewX);
        layerViewer.setPixelY(nViewY);
        layerViewer.draw(g, true);

        g2d.setTransform(oTransform);

        for (Integer animNr : levelItems.keySet()) {
          StaticAnimation animation = staticAnimations.get(animNr);
          for (LevelItem levelItem : levelItems.get(animNr)) {
            if (levelItem.getPosz() == layer)
              animation.draw(g2d, levelItem);
          }
        }

        for (UserSprite userSprite : otherUserSprites.values()) {
          User user = userSprite.getUser();
          if (user.getLevelid() == currentLevel.getId() && user.getPosz() == layer)
            userSprite.draw(g2d);
        }

        if (localUser.getPosz() == layer)
          positionChanged = localUserSprite.draw(g2d);
      }

      if (positionChanged)
        connection.notifyMove(localUser.getPosx(), localUser.getPosy());
    }
  }

  private synchronized void performCollisionDetection(Point requestedPosition)
  {
    int blockWidth = currentMap.getMapHeader().getBlockWidth();
    int blockHeight = currentMap.getMapHeader().getBlockHeight();
    Layer userLayer = currentMap.getLayers()[localUser.getPosz()];
    int newPosX = (int)requestedPosition.getX();
    int newPosY = (int)requestedPosition.getY();

    if (requestedPosition.getX() < localUser.getPosx()) { // Bevegelse mot venstre
      int fromTileX = localUser.getPosx() / blockWidth;
      int fromTileY = localUser.getPosy() / blockHeight;
      int toTileX = (int)(requestedPosition.getX() / blockWidth);
      if (requestedPosition.getX() < 0)
        newPosX = 0;
      else {
        for (int tileX = fromTileX; tileX >= toTileX; tileX--) {
          if (LevelPanel.isSolidBlock(userLayer, tileX, fromTileY)) {
            newPosX = (tileX + 1) * blockWidth;
            break;
          }
        }
      }
    }
    else if (requestedPosition.getX() > localUser.getPosx()) { // Bevegelse mot høyre
      int spriteWidth = localUserSprite.getFrameSize().width;
      int fromTileX = (localUser.getPosx() + spriteWidth) / blockWidth;
      int fromTileY = localUser.getPosy() / blockHeight;
      int toTileX = (int)((requestedPosition.getX() + spriteWidth) / blockWidth);
      if (requestedPosition.getX() + spriteWidth > userLayer.getWidthInPixels())
        newPosX = userLayer.getWidthInPixels() - spriteWidth;
      else {
        for (int tileX = fromTileX; tileX <= toTileX; tileX++) {
          if (LevelPanel.isSolidBlock(userLayer, tileX, fromTileY)) {
            newPosX = tileX * blockWidth - spriteWidth;
            break;
          }
        }
      }
    }
    localUser.moveTo(newPosX, localUser.getPosy());

    if (requestedPosition.getY() < localUser.getPosy()) { // Bevegelse opp
      int fromTileX = localUser.getPosx() / blockWidth;
      int fromTileY = localUser.getPosy() / blockHeight;
      int toTileY = (int)(requestedPosition.getY() / blockHeight);
      if (requestedPosition.getY() < 0)
        newPosY = 0;
      else {
        for (int tileY = fromTileY; tileY >= toTileY; tileY--) {
          if (LevelPanel.isSolidBlock(userLayer, fromTileX, tileY)) {
            newPosY = (tileY + 1) * blockHeight;
            break;
          }
        }
      }
    }
    else if (requestedPosition.getY() > localUser.getPosy()) { // Bevegelse ned
      int spriteHeight = localUserSprite.getFrameSize().height;
      int fromTileX = localUser.getPosx() / blockWidth;
      int fromTileY = (localUser.getPosy() + spriteHeight) / blockHeight;
      int toTileY = (int)(requestedPosition.getY() + spriteHeight) / blockHeight;
      if (requestedPosition.getY() + spriteHeight > userLayer.getHeightInPixels())
        newPosY = userLayer.getHeightInPixels() - spriteHeight;
      else {
        for (int tileY = fromTileY; tileY <= toTileY; tileY++) {
          if (LevelPanel.isSolidBlock(userLayer, fromTileX, tileY)) {
            newPosY = tileY * blockHeight - spriteHeight;
            break;
          }
        }
      }
    }
    localUser.moveTo(newPosX, newPosY);
  }

// Viste seg å i praksis ikke bli brukt
//    private int findStairLayer(int nBlockX, int nBlockY)
//    {
//      for (int nLayer = 0; nLayer < currentMap.getLayers().length; nLayer++) {
//        Block oBlock = currentMap.getLayers()[nLayer].getBlock(nBlockX, nBlockY);
//        if (oBlock.getUserData(INDEX_TYPE) == DATA_STAIRS)
//          return nLayer;
//      }
//      return -1;
//    }
}

