package aoc3.client;

import aoc3.db.User;
import aoc3.icons.Icons;

import java.awt.*;
import java.awt.event.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 05.mai.2005
 * Time: 15:13:39
 * To change this template use File | Settings | File Templates.
 */
final class LocalUserSprite implements KeyEventDispatcher
{
  private static final float VELOCITY = 0.1f; // piksler per ms
  private static final float HIGH_VELOCITY = 0.3f;
  private final Image image;
  private final LevelPanel panel;
  private final Rectangle frameSize;
  private final User user;
  private float dx;
  private float dy;
  private final int numFramesX;
  private final int numFramesY;
  private long lastTime = System.currentTimeMillis();
  private final Point pointCache = new Point();

  LocalUserSprite(LevelPanel panel, User user)
  {
    this.panel = panel;
    this.user = user;
    image = Icons.getAnimation(user.getAvatar()).getImage();
    frameSize = Icons.getFrameSize(user.getAvatar());
    numFramesX = Math.max(1, image.getWidth(null) / frameSize.width);
    numFramesY = Math.max(1, image.getHeight(null) / frameSize.height);
    KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
    manager.addKeyEventDispatcher(this);
  }

// --------------------- Begin Interface KeyEventDispatcher ---------------------

  public boolean dispatchKeyEvent(KeyEvent e)
  {
    if (panel.isShowing()) {
      if (e.getID() == KeyEvent.KEY_PRESSED) {
        switch (e.getKeyCode()) {
          case KeyEvent.VK_LEFT:
            if(e.getModifiersEx() == InputEvent.SHIFT_DOWN_MASK){
              dx = -HIGH_VELOCITY;
              dy = 0;
            }
            else{
              dx = -VELOCITY;
              dy = 0;
            }
            return true;
          case KeyEvent.VK_RIGHT:
            if(e.getModifiersEx() == InputEvent.SHIFT_DOWN_MASK){
              dx = HIGH_VELOCITY;
              dy = 0;
            }
            else{
              dx = VELOCITY;
              dy = 0;
            }
            return true;
          case KeyEvent.VK_DOWN:
            if(e.getModifiersEx() == InputEvent.SHIFT_DOWN_MASK){
              dx = 0;
              dy = HIGH_VELOCITY;
            }
            else{
              dx = 0;
              dy = VELOCITY;
            }
            return true;
          case KeyEvent.VK_UP:
            if(e.getModifiersEx() == InputEvent.SHIFT_DOWN_MASK){
              dx = 0;
              dy = -HIGH_VELOCITY;
            }
            else{
              dx = 0;
              dy = -VELOCITY;
            }
            return true;
        }
      }
      else if (e.getID() == KeyEvent.KEY_RELEASED) {
        switch (e.getKeyCode()) {
          case KeyEvent.VK_LEFT:
          case KeyEvent.VK_RIGHT:
          case KeyEvent.VK_DOWN:
          case KeyEvent.VK_UP:
            dx = 0;
            dy = 0;
            return true;
        }
      }
    }
    else {
      dx = 0;
      dy = 0;
    }
    return false;
  }

  Point getUpdatedPosition(long currentTime)
  {
    long elapsedTime = currentTime - lastTime;
    lastTime = currentTime;
    pointCache.setLocation(user.getPosx() + dx * elapsedTime, user.getPosy() + dy * elapsedTime);
    return pointCache;
  }

// --------------------- End Interface KeyEventDispatcher ---------------------

  boolean draw(Graphics2D g2d)
  {
    int frameNr = 7;
    if (dx != 0 || dy != 0) {
      int action = 0;
      if (dx == 0 && dy < 0)
        action = 1; // NORTH
      else if (dx == 0 && dy > 0)
        action = 2; // SOUTH
      else if (dx > 0 && dy == 0)
        action = 3; // EAST
      else if (dx < 0 && dy == 0)
        action = 4; // WEST
      frameNr = (int)(System.currentTimeMillis() / 125) % 4 +
                (action - 1) * 5;
    }

    // Modulate frame to sprite anim.
    int frameNumX = frameNr % numFramesX;
    int frameNumY = (frameNr / numFramesX) % numFramesY;

    // Then copy that sprite to viewable pane.
    g2d.drawImage(image,
                  user.getPosx(),
                  user.getPosy(),
                  user.getPosx() + frameSize.width,
                  user.getPosy() + frameSize.height,
                  frameNumX * frameSize.width,
                  frameNumY * frameSize.height,
                  (frameNumX + 1) * frameSize.width,
                  (frameNumY + 1) * frameSize.height,
                  null);
    return dx != 0 || dy != 0;
  }

  void reset()
  {
    dx = 0;
    dy = 0;
  }

  Rectangle getFrameSize()
  {
    return frameSize;
  }
}

