package aoc3.client;

import aoc3.db.Highscore;
import aoc3.event.LoginEvent;
import aoc3.event.LoginEventListener;
import aoc3.net.ClientToServer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.apr.2005
 * Time: 18:05:17
 * To change this template use File | Settings | File Templates.
 */
final class LoginPanel extends JPanel implements ActionListener, LoginEventListener
{
  private final ClientToServer connection;
  private final JButton bLogin = new JButton("Logg inn");
  private final HighScoreTableModel tableModel = new HighScoreTableModel();
  private final HighScoreTableModel tableModelWeek = new HighScoreTableModel();
  private final JTable highScoreTable = new JTable(tableModel);
  private final JTable highScoreWeekTable = new JTable(tableModelWeek);
  private final JTextField tfUsername = new JTextField(15);
  private final JTextField tfPassword = new JPasswordField(15);

  LoginPanel(JFrame parent, ClientToServer connection)
  {
    this.connection = connection;
    connection.addLoginListener(this);
    tfPassword.addActionListener(this);
    bLogin.addActionListener(this);
    highScoreTable.setEnabled(false);
    highScoreTable.getColumnModel().getColumn(0).setPreferredWidth(40);
    highScoreTable.setIntercellSpacing(new Dimension(5, 1));
    highScoreWeekTable.setEnabled(false);
    highScoreWeekTable.getColumnModel().getColumn(0).setPreferredWidth(40);
    highScoreWeekTable.setIntercellSpacing(new Dimension(5, 1));

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    JLabel label0 = new AntiAliasJLabel("Age of Computers 3");
    label0.setFont(label0.getFont().deriveFont(30.0f));
    JLabel label1 = new AntiAliasJLabel("Brukernavn");
    JLabel label2 = new AntiAliasJLabel("Passord");
    JLabel label3 = new AntiAliasJLabel("10 beste poengsummer");
    label3.setFont(label3.getFont().deriveFont(12.0f));
    JLabel label4 = new AntiAliasJLabel("10 beste poengsummer siste 7 dager");
    label4.setFont(label3.getFont().deriveFont(12.0f));

    gbc.insets = new Insets(3, 3, 30, 3);
    gbc.fill = GridBagConstraints.NONE;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(label0, gbc);
    add(label0);

    gbc.insets.bottom = 3;
    gbc.gridwidth = 1;
    gbl.setConstraints(label3, gbc);
    add(label3);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(label4, gbc);
    add(label4);

    JScrollPane scroll = new JScrollPane(highScoreTable);
    scroll.setPreferredSize(new Dimension(highScoreTable.getPreferredSize().width,
                                          highScoreTable.getPreferredSize().height + 19));
    JScrollPane scroll2 = new JScrollPane(highScoreWeekTable);
    scroll2.setPreferredSize(new Dimension(highScoreWeekTable.getPreferredSize().width,
                                           highScoreWeekTable.getPreferredSize().height + 19));
    gbc.insets.bottom = 30;
    gbc.gridwidth = 1;
    gbl.setConstraints(scroll, gbc);
    add(scroll);

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(scroll2, gbc);
    add(scroll2);

    gbc.insets.bottom = 3;
    gbc.gridwidth = 1;
    gbc.anchor = GridBagConstraints.EAST;
    gbl.setConstraints(label1, gbc);
    add(label1);

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.WEST;
    gbl.setConstraints(tfUsername, gbc);
    add(tfUsername);

    gbc.gridwidth = 1;
    gbc.anchor = GridBagConstraints.EAST;
    gbl.setConstraints(label2, gbc);
    add(label2);

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.anchor = GridBagConstraints.WEST;
    gbl.setConstraints(tfPassword, gbc);
    add(tfPassword);

    gbc.anchor = GridBagConstraints.CENTER;
    gbl.setConstraints(bLogin, gbc);
    add(bLogin);
    parent.getRootPane().setDefaultButton(bLogin);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    synchronized (bLogin) {
      if (event.getSource() == bLogin || (event.getSource() == tfPassword && bLogin.isEnabled())) {
        bLogin.setEnabled(false);
        connection.requestLogin(tfUsername.getText(), tfPassword.getText());
      }
    }
  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface LoginEventListener ---------------------

  public void loggedIn(LoginEvent event)
  {
    if (event.getType() != LoginEvent.LOGIN_SUCCESS)
      bLogin.setEnabled(true);
  }

  public void loggedOut()
  {
  }

  public void highScoreReceived(final List<Highscore> highScores, final List<Highscore> highScoresWeek)
  {
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        tableModel.updateHighScore(highScores);
        tableModelWeek.updateHighScore(highScoresWeek);
        tfUsername.requestFocusInWindow();
      }
    });
  }

// --------------------- End Interface LoginEventListener ---------------------

// -------------------------- INNER CLASSES --------------------------

  private static final class HighScoreTableModel implements TableModel
  {
    private final List<Highscore> highScores = new ArrayList<Highscore>();
    private final List<TableModelListener> listeners = new ArrayList<TableModelListener>();

    public void updateHighScore(List<Highscore> highScores)
    {
      this.highScores.clear();
      this.highScores.addAll(highScores);
      TableModelEvent event = new TableModelEvent(this);
      for (TableModelListener listener : listeners)
        listener.tableChanged(event);
    }

    public int getRowCount()
    {
      return 10;
    }

    public int getColumnCount()
    {
      return 3;
    }

    public String getColumnName(int columnIndex)
    {
      if (columnIndex == 0)
        return "Nr";
      else if (columnIndex == 1)
        return "Brukernavn";
      else
        return "Poengsum";
    }

    public Class<?> getColumnClass(int columnIndex)
    {
      if (columnIndex == 0 || columnIndex == 2)
        return Integer.class;
      return Object.class;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
      return false;
    }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
      if (columnIndex == 0)
        return rowIndex + 1;
      if (rowIndex >= highScores.size()) {
        if (columnIndex == 1)
          return "";
        return 0;
      }
      Highscore highScore = highScores.get(rowIndex);
      if (columnIndex == 1)
        return highScore.getUsername();
      return highScore.getScore();
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
    }

    public void addTableModelListener(TableModelListener l)
    {
      listeners.add(l);
    }

    public void removeTableModelListener(TableModelListener l)
    {
      listeners.add(l);
    }
  }
}

