package aoc3.client;

import aoc3.client.question.*;
import aoc3.dark.DarkPanel;
import aoc3.db.*;
import aoc3.event.QuestionEvent;
import aoc3.event.QuestionEventListener;
import aoc3.net.ClientToServer;

import javax.swing.*;
import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.apr.2005
 * Time: 13:41:28
 * To change this template use File | Settings | File Templates.
 */
final class QuestionPanel extends JPanel implements QuestionEventListener
{
  private final ClientToServer connection;
  private final DarkPanel darkPanel;
  private final JButton bBack = new JButton("Tilbake");

  QuestionPanel(Client main, ClientToServer connection)
  {
    super(new BorderLayout());
    this.connection = connection;
    darkPanel = new DarkPanel(main, connection);
    bBack.addActionListener(main);
    bBack.setActionCommand(Client.ACTION_BACK);
    connection.addQuestionListener(this);
  }

// --------------------- Begin Interface QuestionEventListener ---------------------

  public void questionReceived(QuestionEvent event)
  {
    removeAll();

    if (event.getQuestion() instanceof MultiChoiceQuestion)
      add(new MultiChoicePanel(connection, (MultiChoiceQuestion)event.getQuestion(),
                               event.getQuestionCategory()), BorderLayout.CENTER);

    else if (event.getQuestion() instanceof InputQuestion)
      add(new InputPanel(connection, (InputQuestion)event.getQuestion(), event.getQuestionCategory()),
          BorderLayout.CENTER);

    else if (event.getQuestion() instanceof AutoGenQuestion) {
      if (event.getAutoGenQuestionText() == null) {
        add(bBack, BorderLayout.NORTH);
        add(new JLabel("Feil under tolking av AutoGen-kode for spørsmål " + event.getQuestion().getId()),
            BorderLayout.CENTER);
      }
      else
        add(new AutoGenPanel(connection, event.getQuestion(), event.getQuestionCategory(),
                             event.getAutoGenQuestionText()), BorderLayout.CENTER);
    }

    else if (event.getQuestion() instanceof DarkQuestion) {
      if (event.getDarkTestSet() == null) {
        add(bBack, BorderLayout.NORTH);
        add(new JLabel("Feil under tolking av Dark-kode for spørsmål " + event.getQuestion().getId()),
            BorderLayout.CENTER);
      }
      else {
        add(darkPanel, BorderLayout.CENTER);
        darkPanel.setQuestion(event.getQuestion(), event.getDarkTestSet(), event.getDarkTestCode(), event.getQuestionCategory());
      }
    }

    else {
      add(bBack, BorderLayout.NORTH);
      add(new JLabel("Fant ikke spørsmål"), BorderLayout.CENTER);
    }

    revalidate();
    repaint();
  }

// --------------------- End Interface QuestionEventListener ---------------------
}

