package aoc3.client;

import aoc3.db.*;
import aoc3.event.AnswerEventListener;
import aoc3.event.LevelEventListener;
import aoc3.icons.Icons;
import aoc3.net.ClientToServer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 06.mai.2005
 * Time: 14:22:03
 * To change this template use File | Settings | File Templates.
 */
final class TopPanel extends JPanel implements LevelEventListener, ActionListener, AnswerEventListener
{
  private final Client main;
  private final ClientToServer connection;
  private final JToggleButton bFact;
  private final JToggleButton bHistory;
  private final JToggleButton bQuestion;
  private String levelName = "";
  private int score;

  private static JToggleButton makeButton(String name)
  {
    JToggleButton button = new JToggleButton(Icons.getIcon(name + ".png"));
    button.setRolloverIcon(Icons.getIcon(name + "Over.png"));
    button.setRolloverEnabled(true);
    button.setPressedIcon(Icons.getIcon(name + "Press.png"));
    button.setSelectedIcon(Icons.getIcon(name + "Press.png"));
    button.setMargin(new Insets(0, 0, 0, 0));
    button.setFocusPainted(false);
    button.setBorder(BorderFactory.createEmptyBorder());
    return button;
  }

  TopPanel(Client main, ClientToServer connection, int score)
  {
    this.main = main;
    this.connection = connection;
    this.score = score;
    setBackground(Color.BLACK);
    bFact = TopPanel.makeButton("factButton");
    bHistory = TopPanel.makeButton("historyButton");
    bQuestion = TopPanel.makeButton("questionButton");
    bFact.addActionListener(this);
    bHistory.addActionListener(this);
    bQuestion.addActionListener(this);
    JLabel scoreLabel = new ScoreLabel(new ImageIcon(Icons.getIcon("barScore.png").getImage()));
    JLabel fillLabel = new TileLabel(new ImageIcon(Icons.getIcon("barFill.png").getImage()));

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbc.weightx = 0.0;
    gbc.weighty = 0.0;
    gbc.fill = GridBagConstraints.NONE;
    gbl.setConstraints(bFact, gbc);
    add(bFact);

    gbl.setConstraints(bHistory, gbc);
    add(bHistory);

    gbl.setConstraints(bQuestion, gbc);
    add(bQuestion);

    gbc.weightx = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(fillLabel, gbc);
    add(fillLabel);

    gbc.weightx = 0.0;
    gbc.fill = GridBagConstraints.NONE;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(scoreLabel, gbc);
    add(scoreLabel);
    connection.addLevelListener(this);
    connection.addAnswerEventListener(this);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == bFact && bFact.isSelected()) {
      connection.requestBookText(Text.TYPE_FACT);
      bHistory.setSelected(false);
      bQuestion.setSelected(false);
    }
    else if (event.getSource() == bHistory && bHistory.isSelected()) {
      connection.requestBookText(Text.TYPE_HISTORY);
      bFact.setSelected(false);
      bQuestion.setSelected(false);
    }
    else if (event.getSource() == bQuestion && bQuestion.isSelected()) {
      connection.requestBookQuestion();
      bFact.setSelected(false);
      bHistory.setSelected(false);
    }
    else {
      bFact.setSelected(false);
      bHistory.setSelected(false);
      bQuestion.setSelected(false);
      main.showMap();
    }
  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface AnswerEventListener ---------------------

  public void answerEvaluated(boolean wasCorrect, boolean canRetry, int newScore)
  {
    if (wasCorrect) {
      score = newScore;
      repaint();
    }
  }

// --------------------- End Interface AnswerEventListener ---------------------

// --------------------- Begin Interface LevelEventListener ---------------------

  public void levelReceived(Level level, Map<Integer, List<LevelItem>> levelItems)
  {
    levelName = level.getName();
    repaint();
  }

  public void levelItemRemoved(LevelItem levelItem)
  {
  }

  public void levelItemChanged(LevelItem levelItem, int newAnimNr)
  {
  }

// --------------------- End Interface LevelEventListener ---------------------

// -------------------------- INNER CLASSES --------------------------

  private static final class TileLabel extends JLabel
  {
    private TileLabel(Icon image)
    {
      super(image);
    }

    protected void paintComponent(Graphics g)
    {
      Icon icon = getIcon();
      for (int x = 0; x < getWidth(); x++)
        icon.paintIcon(this, g, x, 0);
    }
  }

  private final class ScoreLabel extends JLabel
  {
    private final Font font = Font.decode("Arial-BOLD-12");

    private ScoreLabel(Icon image)
    {
      super(image);
    }

    protected void paintComponent(Graphics g)
    {
      super.paintComponent(g);
      Graphics2D g2d = (Graphics2D)g;
      int barScoreX = getWidth() - getIcon().getIconWidth();
      double levelSize = font.getStringBounds(levelName, 0, levelName.length(), g2d.getFontRenderContext()).getWidth();
      g2d.drawString(levelName, barScoreX + 165 - (int)levelSize, 30);
      String scoreString = String.valueOf(score);
      double scoreSize = font.getStringBounds(scoreString, 0, scoreString.length(), g2d.getFontRenderContext()).getWidth();
      g2d.drawString(scoreString, barScoreX + 345 - (int)scoreSize, 30);
    }
  }
}

