package aoc3.client;

import aoc3.db.User;
import aoc3.icons.Icons;

import java.awt.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 26.apr.2005
 * Time: 16:22:25
 * To change this template use File | Settings | File Templates.
 */
final class UserSprite
{
  private final Image image;
  private final Rectangle frameSize;
  private final int numFramesX;
  private final int numFramesY;
  private final User user;
  private int lastPosX;
  private int lastPosY;

  UserSprite(User user)
  {
    this.user = user;
    image = Icons.getAnimation(user.getAvatar()).getImage();
    frameSize = Icons.getFrameSize(user.getAvatar());
    numFramesX = Math.max(1, image.getWidth(null) / frameSize.width);
    numFramesY = Math.max(1, image.getHeight(null) / frameSize.height);
    lastPosX = user.getPosx();
    lastPosY = user.getPosx();
  }

  void draw(Graphics2D g2d)
  {
    int posX = user.getPosx();
    int posY = user.getPosy();
    int frameNr = 7;
    if (posX != lastPosX || posY != lastPosY) {
      int action = 0;
      if (posX == lastPosX && posY < lastPosY)
        action = 1; // NORTH
      else if (posX == lastPosX && posY > lastPosY)
        action = 2; // SOUTH
      else if (posX > lastPosX && posY == lastPosY)
        action = 3; // EAST
      else if (posX < lastPosX && posY == lastPosY)
        action = 4; // WEST
      frameNr = (int)(System.currentTimeMillis() / 125) % 4 +
                (action - 1) * 5;
    }

    // Modulate frame to sprite anim.
    int frameNumX = frameNr % numFramesX;
    int frameNumY = (frameNr / numFramesX) % numFramesY;

    // Then copy that sprite to viewable pane.
    g2d.drawImage(image,
                  posX,
                  posY,
                  posX + frameSize.width,
                  posY + frameSize.height,
                  frameNumX * frameSize.width,
                  frameNumY * frameSize.height,
                  (frameNumX + 1) * frameSize.width,
                  (frameNumY + 1) * frameSize.height,
                  null);
    lastPosX = posX;
    lastPosY = posY;
  }

  public User getUser()
  {
    return user;
  }
}
