package aoc3.client.book;

import aoc3.db.Text;
import aoc3.db.UserQuestion;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 20.jul.2005
 * Time: 14:50:05
 * To change this template use File | Settings | File Templates.
 */
final class BookTreeNode implements Comparable
{
  private final Comparable payload;
  private final List<BookTreeNode> children = new ArrayList<BookTreeNode>();

  BookTreeNode(Comparable payload)
  {
    this.payload = payload;
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(Object o)
  {
    BookTreeNode btn = (BookTreeNode)o;
    return payload.compareTo(btn.payload);
  }

// --------------------- End Interface Comparable ---------------------

  public String toString()
  {
    return payload.toString();
  }

  void clear()
  {
    children.clear();
  }

  void addChild(BookTreeNode node)
  {
    children.add(node);
  }

  int indexOf(BookTreeNode node)
  {
    return children.indexOf(node);
  }

  int getChildCount()
  {
    return children.size();
  }

  BookTreeNode getChild(int nr)
  {
    return children.get(nr);
  }

  private int countLeafNodes()
  {
    if (children.isEmpty()) {
      if (payload instanceof Text || payload instanceof UserQuestion)
        return 1;
      else
        return 0;
    }
    else {
      int c = 0;
      for (BookTreeNode node : children)
        c += node.countLeafNodes();
      return c;
    }
  }

  Comparable getPayload()
  {
    return payload;
  }

  void purgeEmptyNonLeafNodes()
  {
    for (Iterator<BookTreeNode> iter = children.iterator(); iter.hasNext();) {
      BookTreeNode node = iter.next();
      if (node.countLeafNodes() == 0)
        iter.remove();
      else
        node.purgeEmptyNonLeafNodes();
    }
  }

  void sort()
  {
    Collections.sort(children);
    for (BookTreeNode node : children)
      node.sort();
  }
}

