package aoc3.client.book;

import aoc3.client.Client;
import aoc3.client.ImageURLParser;
import aoc3.client.question.*;
import aoc3.dark.AssemblyEditor;
import aoc3.dark.DarkPanel;
import aoc3.db.*;
import aoc3.event.QuestionBookListener;
import aoc3.net.ClientToServer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 20.jul.2005
 * Time: 15:05:55
 * To change this template use File | Settings | File Templates.
 */
public final class QuestionBookPanel extends JPanel implements TreeSelectionListener, QuestionBookListener,
                                                               ActionListener
{
  private final JButton bAnswer = new JButton("Vis løsning");
  private final JButton bShowFormula = new JButton("Vis formel");
  private final JPanel questionPanel = new JPanel(new BorderLayout());
  private final JTextField tfAnswer = new JTextField(15);
  private final QuestionTreeModel treeModel = new QuestionTreeModel();
  private final JTree tree = new JTree(treeModel);
  private final DarkPanel darkPanel = new DarkPanel();
  private Map<Integer, QuestionCategory> questionCategories;
  private Map<Integer, QuestionGroup> questionGroups;
  private Map<UserQuestion, Question> completedQuestions;
  private Map<UserQuestion, String> autogenQuestionTexts;
  private Map<UserQuestion, String> autogenFormulas;
  private Map<UserQuestion, String> darkSuggestedSolution;
  private UserQuestion userQuestion;

  public QuestionBookPanel(ClientToServer connection)
  {
    super(new BorderLayout());

    JScrollPane scroll = new JScrollPane(tree);
    scroll.setPreferredSize(new Dimension(200, 400));
    bAnswer.addActionListener(this);
    bShowFormula.addActionListener(this);
    tfAnswer.setEditable(false);
    tfAnswer.setPreferredSize(new Dimension(tfAnswer.getPreferredSize().width,
                                            bAnswer.getPreferredSize().height));

    add(scroll, BorderLayout.WEST);
    add(questionPanel, BorderLayout.CENTER);
    tree.addTreeSelectionListener(this);
    tree.setRootVisible(false);
    connection.addQuestionBookListener(this);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    Question question = completedQuestions.get(userQuestion);
    if (question instanceof MultiChoiceQuestion) {

      if (isInvalidInput(userQuestion)) return;

      int nr = 1 + Integer.parseInt(userQuestion.getAnswer().toString());
      tfAnswer.setText("Alternativ nr " + nr);
    }
    else if (question instanceof DarkQuestion) {

      String solutionText = darkSuggestedSolution.get(userQuestion);

      if (solutionText == null) {
        JOptionPane.showMessageDialog(this, "Det er desverre ikke løsningsforslag til denne oppgaven", "Løsningsforslag mangler", JOptionPane.INFORMATION_MESSAGE);
        return;
      }
      else if (solutionText.length() == 0) {
        JOptionPane.showMessageDialog(this, "Det er desverre ikke løsningsforslag til denne oppgaven", "Løsningsforslag mangler", JOptionPane.INFORMATION_MESSAGE);
        return;
      }

      solutionText = DarkPanel.makeJLexSafeCode(solutionText);

      JDialog dialog = new JDialog();

      AssemblyEditor answer;
      if (darkPanel.getCurrentArchitecture() == AssemblyEditor.STACK_EDITOR)
        answer = new AssemblyEditor(AssemblyEditor.STACK_EDITOR, Client.getServer());
      else
        answer = new AssemblyEditor(AssemblyEditor.LOAD_STORE_EDITOR, Client.getServer());

      answer.setText(solutionText);
      answer.setEditable(false);

      JScrollPane scroll = new JScrollPane(answer);

      dialog.getContentPane().add(scroll);
      dialog.pack();
      dialog.setSize(600, 600);
      dialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

      Dimension screen = getToolkit().getScreenSize();
      int x = (screen.width - dialog.getSize().width) / 2;
      int y = (screen.height - dialog.getSize().height) / 2;
      dialog.setLocation(Math.max(0, x), Math.max(0, y));

      dialog.setVisible(true);
    }
    else if (question instanceof AutoGenQuestion) {
      if (event.getActionCommand().equals("Vis formel")) {
        boolean failed = false;
        if (autogenFormulas != null) {
          if (autogenFormulas.containsKey(userQuestion)) {
            String formula = autogenFormulas.get(userQuestion);
            if (formula != null) {
              JDialog dialog = new JDialog();
              JEditorPane formulaPane = new JEditorPane("text/html",
                                                        ImageURLParser.parseHTML("<br><center>" + formula + "</center><br>"));
              formulaPane.setEditable(false);
              formulaPane.setEnabled(false);
              formulaPane.setCaretPosition(0);

              dialog.getContentPane().add(new JScrollPane(formulaPane));
              dialog.pack();
              Dimension screen = getToolkit().getScreenSize();
              int x = (screen.width - dialog.getSize().width) / 2;
              int y = (screen.height - dialog.getSize().height) / 2;
              dialog.setLocation(Math.max(0, x), Math.max(0, y));
              dialog.setTitle("Formel");
              dialog.setVisible(true);
              dialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
            }
            else
              failed = true;
          }
          else {
            System.out.println("ikke riktig spm");
            failed = true;
          }
        }
        else {
          System.out.println("er null");
          failed = true;
        }

        if (failed) {
          JOptionPane.showMessageDialog(this, "Det er ikke registrert noen formel for denne oppgaven. Noter spørsmålsnummeret og kontakt fagstaben.", "Formel", JOptionPane.INFORMATION_MESSAGE);
        }
      }
      else {
        if (isInvalidInput(userQuestion)) return;
        tfAnswer.setText(userQuestion.getAnswer().toString());
      }
    }
    else {
      if (isInvalidInput(userQuestion)) return;
      tfAnswer.setText(userQuestion.getAnswer().toString());
    }
  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface QuestionBookListener ---------------------

  public void questionsReceived(Map<UserQuestion, Question> completedQuestions,
                                Map<UserQuestion, String> autogenQuestionTexts,
                                Map<UserQuestion, String> autogenFormulas,
                                Map<UserQuestion, String> darkSuggestedSolution)
  {
    this.completedQuestions = completedQuestions;
    this.autogenQuestionTexts = autogenQuestionTexts;
    this.autogenFormulas = autogenFormulas;
    this.darkSuggestedSolution = darkSuggestedSolution;
    treeModel.showQuestions();
    treeModel.sort();
    tree.clearSelection();
  }

  public void categoriesAndGroupsReceived(Map<Integer, QuestionCategory> questionCategories,
                                          Map<Integer, QuestionGroup> questionGroups)
  {
    this.questionCategories = questionCategories;
    this.questionGroups = questionGroups;
  }

// --------------------- End Interface QuestionBookListener ---------------------

// --------------------- Begin Interface TreeSelectionListener ---------------------

  public void valueChanged(TreeSelectionEvent event)
  {
    questionPanel.removeAll();
    bAnswer.setEnabled(true);
    tfAnswer.setText("");
    BookTreeNode node = (BookTreeNode)tree.getLastSelectedPathComponent();
    if (node != null && node.getPayload() instanceof UserQuestion) {
      userQuestion = (UserQuestion)node.getPayload();
      Question question = completedQuestions.get(userQuestion);
      QuestionCategory category = questionCategories.get(question.getQuestionCategoryID());

      if (question instanceof MultiChoiceQuestion)
        questionPanel.add(new MultiChoicePanel(null, (MultiChoiceQuestion)question,
                                               category), BorderLayout.CENTER);

      else if (question instanceof InputQuestion)
        questionPanel.add(new InputPanel(null, (InputQuestion)question, category), BorderLayout.CENTER);

      else if (question instanceof AutoGenQuestion) {
        questionPanel.add(new AutoGenPanel(null, question, category, autogenQuestionTexts.get(userQuestion)),
                          BorderLayout.CENTER);
      }

      else if (question instanceof DarkQuestion) {
        questionPanel.add(darkPanel, BorderLayout.CENTER);
        darkPanel.setQuestion(question, userQuestion.getParams()[0].toString(), ((DarkQuestion)question).getTestCode(), category);
        darkPanel.setDarkCode(userQuestion.getAnswer().toString());
      }

      JPanel p = new JPanel(new FlowLayout(FlowLayout.LEFT));
      if (question instanceof AutoGenQuestion) {
        bShowFormula.setEnabled(true);
        p.add(bShowFormula);
      }
      p.add(bAnswer);
      if (!(question instanceof DarkQuestion)) {
        p.add(tfAnswer);
        bAnswer.setText("Vis løsning");
      }
      else {
        bAnswer.setText("Vis løsningsforslag");
      }
      questionPanel.add(p, BorderLayout.SOUTH);
    }

    questionPanel.revalidate();
    questionPanel.repaint();
  }

// --------------------- End Interface TreeSelectionListener ---------------------

  private boolean isInvalidInput(UserQuestion in)
  {
    if (in.getAnswer() == null) {
      JOptionPane.showMessageDialog(this, "Løsning er dessverre ikke tilgjengelig for denne oppgaven", "Løsning mangler", JOptionPane.INFORMATION_MESSAGE);
      return true;
    }
    else if (userQuestion.getAnswer().toString().length() == 0) {
      JOptionPane.showMessageDialog(this, "Løsning er dessverre ikke tilgjengelig for denne oppgaven", "Løsning mangler", JOptionPane.INFORMATION_MESSAGE);
      return true;
    }
    return false;
  }

// -------------------------- INNER CLASSES --------------------------

  private final class QuestionTreeModel implements TreeModel
  {
    private final BookTreeNode root = new BookTreeNode("");
    private final List<TreeModelListener> listeners = new ArrayList<TreeModelListener>();

    private void showQuestions()
    {
      root.clear();
      for (QuestionGroup questionGroup : questionGroups.values()) {
        if (questionGroup.getParentID() == QuestionGroup.NO_PARENT) {
          BookTreeNode node = new BookTreeNode(questionGroup);
          root.addChild(node);
          addChildren(questionGroup, node);
        }
      }
    }

    private void addChildren(QuestionGroup parentGroup, BookTreeNode parentNode)
    {
      // 1: Legg til alle matchende grupper og kall rekursivt
      for (QuestionGroup questionGroup : questionGroups.values()) {
        if (questionGroup.getParentID() == parentGroup.getId()) {
          BookTreeNode node = new BookTreeNode(questionGroup);
          parentNode.addChild(node);
          addChildren(questionGroup, node);
        }
      }
      // 2: Legg til alle matchende kategorier og kall addChildren()
      for (QuestionCategory questionCategory : questionCategories.values()) {
        if (questionCategory.getQuestionGroupid() == parentGroup.getId()) {
          BookTreeNode node = new BookTreeNode(questionCategory);
          parentNode.addChild(node);
          addChildren(questionCategory, node);
        }
      }
    }

    private void addChildren(QuestionCategory questionCategory, BookTreeNode parentNode)
    {
      for (UserQuestion uQuestion : completedQuestions.keySet()) {
        Question question = completedQuestions.get(uQuestion);
        if (question.getQuestionCategoryID() == questionCategory.getId())
          parentNode.addChild(new BookTreeNode(uQuestion));
      }
    }

    void sort()
    {
      root.purgeEmptyNonLeafNodes();
      root.sort();
      TreeModelEvent event = new TreeModelEvent(this, new Object[] {root});
      for (TreeModelListener listener : listeners)
        listener.treeStructureChanged(event);
    }

    public Object getRoot()
    {
      return root;
    }

    public Object getChild(Object parent, int index)
    {
      return ((BookTreeNode)parent).getChild(index);
    }

    public int getChildCount(Object parent)
    {
      return ((BookTreeNode)parent).getChildCount();
    }

    public boolean isLeaf(Object node)
    {
      if (!(node instanceof BookTreeNode)) // Sært - burde ikke kunne skje (se Bugzilla 14)
        return false;
      return ((BookTreeNode)node).getPayload() instanceof UserQuestion;
    }

    public void valueForPathChanged(TreePath path, Object newValue)
    {
    }

    public int getIndexOfChild(Object parent, Object child)
    {
      return ((BookTreeNode)parent).indexOf((BookTreeNode)child);
    }

    public void addTreeModelListener(TreeModelListener l)
    {
      listeners.add(l);
    }

    public void removeTreeModelListener(TreeModelListener l)
    {
      listeners.remove(l);
    }
  }
}

