package aoc3.client.book;

import aoc3.client.ImageURLParser;
import aoc3.db.*;
import aoc3.event.TextBookListener;
import aoc3.net.ClientToServer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Map;

public final class TextBookPanel extends JPanel implements TreeSelectionListener, ActionListener,
                                                           TextBookListener
{
  private final JButton bWindow = new JButton("Åpne i eget vindu");
  private final JEditorPane epHTML = new JEditorPane();
  private final JTree tree = new JTree();

  public TextBookPanel(ClientToServer connection)
  {
    super(new BorderLayout());
    epHTML.setEditable(false);
    epHTML.setContentType("text/html");
    bWindow.addActionListener(this);
    bWindow.setEnabled(false);

    JPanel leftPanel = new JPanel(new BorderLayout());
    leftPanel.add(new JScrollPane(tree), BorderLayout.CENTER);
    leftPanel.add(bWindow, BorderLayout.SOUTH);
    leftPanel.setPreferredSize(new Dimension(200, 400));

    add(leftPanel, BorderLayout.WEST);
    add(new JScrollPane(epHTML), BorderLayout.CENTER);
    tree.addTreeSelectionListener(this);
    connection.addTextBookListener(this);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    BookTreeNode node = (BookTreeNode)tree.getLastSelectedPathComponent();
    if (node != null && node.getPayload() instanceof Text)
      new TextWindow((Text)node.getPayload());
  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface TextBookListener ---------------------

  public void textsReceived(int type, List<Text> visitedTexts, Map<Integer, TextGroup> textGroups)
  {
    int rootID = TextGroup.FACT_ROOT_ID;
    if (type == Text.TYPE_HISTORY)
      rootID = TextGroup.HISTORY_ROOT_ID;
    TextTreeModel treeModel = new TextTreeModel(rootID, visitedTexts, textGroups);
    treeModel.sort();
    tree.clearSelection();
    epHTML.setText("");
    tree.setModel(treeModel);
    tree.setRootVisible(false);
  }

// --------------------- End Interface TextBookListener ---------------------

// --------------------- Begin Interface TreeSelectionListener ---------------------

  public void valueChanged(TreeSelectionEvent event)
  {
    BookTreeNode node = (BookTreeNode)tree.getLastSelectedPathComponent();
    if (node != null && node.getPayload() instanceof Text) {
      bWindow.setEnabled(true);
      Text text = (Text)node.getPayload();
      epHTML.setText(ImageURLParser.parseHTML("<h1>" + text.getName() + "</h1>" + text.getText()));
      epHTML.setCaretPosition(0);
    }
    else {
      bWindow.setEnabled(false);
      epHTML.setText("");
    }
  }

// --------------------- End Interface TreeSelectionListener ---------------------

// -------------------------- INNER CLASSES --------------------------

  private static final class TextTreeModel implements TreeModel
  {
    final BookTreeNode root;

    TextTreeModel(int rootID, List<Text> visitedTexts, Map<Integer, TextGroup> textGroups)
    {
      root = new BookTreeNode(textGroups.get(rootID));
      TextTreeModel.addChildren(textGroups.get(rootID), root, visitedTexts, textGroups);
    }

    static void addChildren(TextGroup parentGroup, BookTreeNode parentNode,
                            List<Text> texts, Map<Integer, TextGroup> textGroups)
    {
      // 1: Legg til grupper med matchende parentID og kjør rekursivt på disse
      for (TextGroup textGroup : textGroups.values()) {
        if (textGroup.getParentID() == parentGroup.getId()) {
          BookTreeNode node = new BookTreeNode(textGroup);
          parentNode.addChild(node);
          addChildren(textGroup, node, texts, textGroups);
        }
      }
      // 2: Legg til alle matchende texts
      for (Text text : texts)
        if (text.getTextGroupID() == parentGroup.getId())
          parentNode.addChild(new BookTreeNode(text));
    }

    void sort()
    {
      root.purgeEmptyNonLeafNodes();
      root.sort();
    }

    public Object getRoot()
    {
      return root;
    }

    public Object getChild(Object parent, int index)
    {
      return ((BookTreeNode)parent).getChild(index);
    }

    public int getChildCount(Object parent)
    {
      return ((BookTreeNode)parent).getChildCount();
    }

    public boolean isLeaf(Object node)
    {
      if (!(node instanceof BookTreeNode)) // Sært - burde ikke kunne skje (se Bugzilla 14)
        return false;
      return ((BookTreeNode)node).getPayload() instanceof Text ||
             ((BookTreeNode)node).getPayload() instanceof Question;
    }

    public void valueForPathChanged(TreePath path, Object newValue)
    {
    }

    public int getIndexOfChild(Object parent, Object child)
    {
      return ((BookTreeNode)parent).indexOf((BookTreeNode)child);
    }

    public void addTreeModelListener(TreeModelListener l)
    {
    }

    public void removeTreeModelListener(TreeModelListener l)
    {
    }
  }

  private static final class TextWindow extends JFrame
  {
    TextWindow(Text text)
    {
      super(text.getName());
      JEditorPane htmlPane = new JEditorPane();
      htmlPane.setEditable(false);
      htmlPane.setContentType("text/html");
      htmlPane.setText(ImageURLParser.parseHTML("<h1>" + text.getName() + "</h1>" + text.getText()));
      htmlPane.setCaretPosition(0);
      JScrollPane scroll = new JScrollPane(htmlPane);
      getContentPane().add(scroll);
      setSize(800, 600);
      Dimension screen = getToolkit().getScreenSize();
      int x = (screen.width - getSize().width) / 2;
      int y = (screen.height - getSize().height) / 2;
      setLocation(Math.max(0, x), Math.max(0, y));
      setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
      setVisible(true);
    }
  }
}

