package aoc3.client.question;

import aoc3.client.ImageURLParser;
import aoc3.db.Question;
import aoc3.db.QuestionCategory;
import aoc3.net.ClientToServer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 19.jul.2005
 * Time: 17:23:17
 * To change this template use File | Settings | File Templates.
 */
abstract class AbstractQuestionPanel extends JPanel implements ActionListener
{
  private final JPanel filler = new JPanel();
  final ClientToServer connection;
  final JButton bAnswer = new JButton("Ok");
  final Question question;
  final boolean readOnly;

  protected AbstractQuestionPanel(ClientToServer connection, Question question,
                                  QuestionCategory questionCategory, String questionText)
  {
    this.connection = connection;
    this.question = question;
    bAnswer.addActionListener(this);
    readOnly = connection == null;
    bAnswer.setEnabled(!readOnly);

    String head = "<head></head><h1>" + questionCategory.getName() + "</h1><br>";
    if (questionCategory.getText() != null && questionCategory.getText().length() > 0)
      head += questionCategory.getText() + "<p>";
    JEditorPane questionPane = new JEditorPane("text/html",
                                               ImageURLParser.parseHTML(head + questionText));
    questionPane.setEditable(false);
    questionPane.setCaretPosition(0);
    JScrollPane scroll = new JScrollPane(questionPane);
    JLabel idLabel = new JLabel("Spørsmål nr " + question.getId());

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbc.insets = new Insets(3, 3, 3, 3);
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbl.setConstraints(scroll, gbc);
    add(scroll);

    if (readOnly) {
      initReadOnlyComponents(question, gbl, gbc);
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbl.setConstraints(filler, gbc);
      add(filler);
    }
    else {
      initAnswerComponents(question, gbl, gbc);
      gbc.fill = GridBagConstraints.NONE;
      gbc.anchor = GridBagConstraints.WEST;
      gbl.setConstraints(bAnswer, gbc);
      add(bAnswer);
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbl.setConstraints(idLabel, gbc);
      add(idLabel);
    }
  }

  abstract void initAnswerComponents(Question question, GridBagLayout gbl, GridBagConstraints gbc);

  abstract void initReadOnlyComponents(Question question, GridBagLayout gbl, GridBagConstraints gbc);
}

