package aoc3.dark;

import aoc3.client.Client;
import aoc3.client.ImageURLParser;
import aoc3.dark.architecture.*;
import aoc3.dark.validation.ValidationException;
import aoc3.dark.validation.Validator;
import aoc3.db.Question;
import aoc3.db.QuestionCategory;
import aoc3.event.AnswerEventListener;
import aoc3.net.ClientToServer;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 28.mai.2005
 * Time: 11:41:25
 * To change this template use File | Settings | File Templates.
 */
public final class DarkPanel extends JPanel implements ActionListener,
                                                       IExecutionListener,
                                                       AnswerEventListener

{
  private static final String CARD_CODE = "Code";
  private static final String CARD_EDIT = "Edit";
  private static final int STATE_EDIT = 0;
  private static final int STATE_RUN = 1;

  private static final int PREV_LINE_INIT = -3;

  private final Architecture architecture = new Architecture();

  private final CardLayout cards = new CardLayout();
  private final ClientToServer connection;
  private final DarkCodeListModel darkCodeListModel = new DarkCodeListModel();
  private final JButton parseButton = new JButton("> Kjør kode");
  private final JButton stepButton = new JButton("Ett steg");
  private final JButton allStepsButton = new JButton("Alle steg");
  private final JButton validateButton = new JButton("Valider");
  private final JButton cancelButton = new JButton("Avbryt");
  private final JButton bBack = new JButton("Tilbake til brett");
  private final JButton insertTestCode = new JButton("Sett inn testkode");
  private final JEditorPane epHTML = new JEditorPane();
  private final JList codeList;
  private final JPanel codePanel = new JPanel(cards);
  private final AssemblyEditor editArea;
  private final TitledBorder variableBorder = BorderFactory.createTitledBorder("Variabler");
  private final TitledBorder codeBorder = BorderFactory.createTitledBorder("Editor");
  private final VariableListModel variableListModel = new VariableListModel();
  private Question question;
  private Validator validator;
  private int state;

  private String darkTestCode = "";
  private int previousLineNumber = PREV_LINE_INIT;

  public DarkPanel()
  {
    this(null, null);
  }

  public DarkPanel(Client main, ClientToServer connection)
  {
    this.connection = connection;
    if (connection != null)
      connection.addAnswerEventListener(this);


    if (architecture.getArchitecture() == Architecture.ARCH_STACK) {
      editArea = new AssemblyEditor(AssemblyEditor.STACK_EDITOR, Client.getServer());

    }
    else {
      editArea = new AssemblyEditor(AssemblyEditor.LOAD_STORE_EDITOR, Client.getServer());
    }
    editArea.setText("main\n\t;Skriv inn din kode her.\nend main");
    JScrollPane editAreaPane = new JScrollPane(editArea);

    codeList = new JList(darkCodeListModel);
    codeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    codeList.setEnabled(false);
    JScrollPane codePane = new JScrollPane(codeList);

    JList variableList = new JList(variableListModel);
    JScrollPane variablePane = new JScrollPane(variableList);
    variablePane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    variablePane.setBorder(variableBorder);
    variablePane.setPreferredSize(new Dimension(400, 250));
    variablePane.setMinimumSize(variablePane.getPreferredSize());

    architecture.addExecutionListener(this);
    architecture.addExecutionListener(variableListModel);

    parseButton.addActionListener(this);
    insertTestCode.addActionListener(this);
    stepButton.addActionListener(this);
    allStepsButton.addActionListener(this);
    cancelButton.addActionListener(this);
    validateButton.addActionListener(this);

    cancelButton.setEnabled(false);
    if (main != null) {
      bBack.addActionListener(main);
      bBack.setActionCommand(Client.ACTION_BACK);
    }

    epHTML.setEditable(false);
    epHTML.setContentType("text/html");
    JScrollPane scrollQuestion = new JScrollPane(epHTML);
    scrollQuestion.setBorder(BorderFactory.createTitledBorder("Oppgave"));

    JPanel codeButtonPanel = new JPanel(new FlowLayout());
    codeButtonPanel.add(stepButton);
    codeButtonPanel.add(allStepsButton);
    codeButtonPanel.add(cancelButton);
    if (connection != null)
      codeButtonPanel.add(validateButton);

    JPanel runCodePanel = new JPanel(new BorderLayout());
    runCodePanel.add(codePane, BorderLayout.CENTER);
    runCodePanel.add(codeButtonPanel, BorderLayout.SOUTH);

    codePanel.setBorder(codeBorder);
    codePanel.add(editAreaPane, CARD_EDIT);
    codePanel.add(runCodePanel, CARD_CODE);
    setCodeState(STATE_EDIT);

    JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    buttonPanel.add(parseButton);
    buttonPanel.add(insertTestCode);
    if (main != null)
      buttonPanel.add(bBack);

    JPanel rightPanel = new JPanel(new BorderLayout());
    rightPanel.add(codePanel, BorderLayout.CENTER);
    rightPanel.add(buttonPanel, BorderLayout.SOUTH);

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbc.insets = new Insets(3, 3, 3, 3);
    gbc.gridwidth = 1;
    gbc.weightx = 0.8; //1.0;
    gbc.weighty = 1.0;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(scrollQuestion, gbc);
    add(scrollQuestion);

    gbc.weightx = 0.2; //0;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.gridheight = GridBagConstraints.REMAINDER;
    gbl.setConstraints(rightPanel, gbc);
    add(rightPanel);

    gbc.gridwidth = 1;
    gbc.gridheight = 1;
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.weighty = 0.0;
    gbc.weightx = 0.8; //1.0;
    gbl.setConstraints(variablePane, gbc);
    add(variablePane);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {

    if (event.getSource() == parseButton) {
      if (state == STATE_EDIT)
        parseCode();
      else
        setCodeState(STATE_EDIT);
    }
    else if (event.getSource() == stepButton) {
      architecture.setShowErrors(true);
      architecture.execute(Architecture.STEPS_ONE);
    }
    else if (event.getSource() == allStepsButton) {
      stepButton.setEnabled(false);
      allStepsButton.setEnabled(false);
      cancelButton.setEnabled(true);
      parseButton.setEnabled(false);
      bBack.setEnabled(false);
      validateButton.setEnabled(false);
      architecture.setShowErrors(true);
      architecture.execute(Architecture.STEPS_ALL);
    }
    else if (event.getSource() == cancelButton)
      architecture.pauseExecution();
    else if (event.getSource() == validateButton) {
      if (parseCode() && validator.isQuestionCorrectlyParsed()) {
        try {
          validateButton.setEnabled(false);
          validator.validate(editArea.getText());
          connection.submitDarkAnswer(question.getId(), editArea.getText());
        } catch (ValidationException e) {
          showError("Feil under validering");
        }
      }
    }
    else if (event.getSource() == insertTestCode) {

      if (darkTestCode.length() == 0) {
        showError("Eksempelkode er dessverre ikke tilgjengelig for dette spørsmålet.");
      }
      else {
        String currentText = editArea.getText();

        int insertPos = editArea.getCaretPosition();
        String prefix = currentText.substring(0, insertPos);
        String suffix = currentText.substring(insertPos, currentText.length());
        editArea.setText(prefix + darkTestCode + suffix);
      }
    }

  }

// --------------------- End Interface ActionListener ---------------------

// --------------------- Begin Interface AnswerEventListener ---------------------

  public void answerEvaluated(boolean wasCorrect, boolean canRetry, int newScore)
  {
    if (!wasCorrect && isShowing())
      showError("Feil under validering");
  }

// --------------------- End Interface AnswerEventListener ---------------------

// --------------------- Begin Interface IExecutionListener ---------------------

  public void executionPaused()
  {

    validateButton.setEnabled(validator != null && validator.isQuestionCorrectlyParsed());
    stepButton.setEnabled(true);
    allStepsButton.setEnabled(true);
    cancelButton.setEnabled(false);
    parseButton.setEnabled(true);
    bBack.setEnabled(true);
  }

  public void lineChanged(int lineNumber)
  {

    if (lineNumber == previousLineNumber) {
      showError("Det er ikke mulig å gå videre i programmet ditt. Vanlige grunner til dette er at testkode mangler eller at programmet er ferdig.");
      return;
    }
    previousLineNumber = lineNumber;

    if (lineNumber != -1)
      codeList.setSelectionInterval(lineNumber - 1, lineNumber - 1);
  }

// --------------------- End Interface IExecutionListener ---------------------


  public void setQuestion(Question question, String testSet, String testCode, QuestionCategory questionCategory)
  {

    this.question = question;
    String head = "<h1>" + questionCategory.getName() + "</h1><br>";
    if (questionCategory.getText() != null && questionCategory.getText().length() > 0)
      head += questionCategory.getText() + "<p>";
    epHTML.setText(ImageURLParser.parseHTML(head + question.getText()));
    validator = new Validator(testSet, architecture);

    if (testCode == null) { // TestCode kan være null
      testCode = "";
    }

    darkTestCode = DarkPanel.makeJLexSafeCode(testCode);

    int architectureNumber = validator.getArchitectureNumber();
    architecture.setArchitecture(architectureNumber);
    switch (architectureNumber) {
      case Architecture.ARCH_STACK:
        variableBorder.setTitle("Stakk");
        try {
          editArea.setArchitecture(AssemblyEditor.STACK_EDITOR);
        }
        catch (Exception e) {
          e.printStackTrace();
        }
        break;
      case Architecture.ARCH_LOAD:
        variableBorder.setTitle("Variabler");
        try {
          editArea.setArchitecture(AssemblyEditor.LOAD_STORE_EDITOR);
        }
        catch (Exception e) {
          e.printStackTrace();
        }
        break;
      default:
        variableBorder.setTitle("Variabler");
        try {
          editArea.setArchitecture(AssemblyEditor.LOAD_STORE_EDITOR);
        }
        catch (Exception e) {
          e.printStackTrace();
        }
    }
  }

  /**
   * Parses the written code.
   */
  private boolean parseCode()
  {
    String source = editArea.getText();
    // Sanity check:
    if (source.length() == 0) {
      showError("Vennligst skriv inn kode.");
      return false;
    }
    try {
      architecture.parse(source);
      // Sets the state ( and updates variable and code list)
      setCodeState(STATE_RUN);
      return true;
    } catch (ParseException parseError) {
      // Denne er egentlig ganske vanlig.
      showError(parseError.getMessage());
      return false;
    } catch (IOException e) {
      e.printStackTrace();
      showError("Fikk en IOException. Vennligst sjekk programmet ditt.");
      return false;
    } catch (NullPointerException e) {
      // Vi viser ikke stacktrace for dette er en veldig vanlig feil.
      // Dette er noe drit. Eneste grunnen er pga udugelig JLex:
      //e.printStackTrace();
      showError("Du har en feil i syntaksen din.\n" + ParserFactory.getInstance().getLastError());
      return false;
    } catch (Exception e) {
      showError("Noe gikk feil under initialiseringen av assembleren.");
      e.printStackTrace();
      return false;
    } catch (Error err) {
      showError("Det har oppstått en fatal feil i Dark, og klienten må avsluttes. Vennligst send stack-trace til fagstaben.");
      err.printStackTrace();
      System.exit(20);
      return false;
    }
  }

  public int getCurrentArchitecture()
  {
    return editArea.getArchitecture();
  }

  public void setDarkCode(String code)
  {
    String safeCode = DarkPanel.makeJLexSafeCode(code);
    editArea.setText(safeCode);
  }

  public static String makeJLexSafeCode(String testCode)
  {
    // JLex er er kjip og må bare få kjente characters
    StringBuilder jlexSafeCode = new StringBuilder();
    for (char x : testCode.toCharArray()) {
      if (Character.isDigit(x) || Character.isLetter(x)) {
        jlexSafeCode.append(x);
      }
      else if (x == ' ') {
        jlexSafeCode.append(' ');
      }
      else if (x == '\n') {
        jlexSafeCode.append('\n');
      }
      else if (x == '\t') {
        jlexSafeCode.append('\t');
      }
      else
      if (x == '$' || x == ';' || x == '+' || x == '-' || x == '=' || x == '*' || x == '/' || x == '\\' || x == ','
          || x == '.' || x == '@' || x == '<' || x == '>' || x == '(' || x == ')' || x == ':' || x == '?') {
        jlexSafeCode.append(x);
      }
    }
    return jlexSafeCode.toString();
  }

  private void setCodeState(int state)
  {
    this.state = state;

    if (state == STATE_EDIT) {
      previousLineNumber = PREV_LINE_INIT;
      parseButton.setText("> Kjør kode");
      variableListModel.setModel(null);

      cards.show(codePanel, CARD_EDIT);
      codeBorder.setTitle("Editor");
      codePanel.repaint();
    }
    else {  // STATE_RUN:
      parseButton.setText("> Editering");
      validateButton.setEnabled(validator != null && validator.isQuestionCorrectlyParsed());
      darkCodeListModel.setSource(editArea.getText());
      codeList.setSelectionInterval(0, 0);
      variableListModel.setModel(architecture);

      cards.show(codePanel, CARD_CODE);
      codeBorder.setTitle("Kjør kode");
      codePanel.repaint();
    }
  }

  private void showError(String error)
  {
    JOptionPane.showMessageDialog(this, error, "Feil", JOptionPane.ERROR_MESSAGE);
  }

}