package aoc3.dark;


import aoc3.dark.architecture.Architecture;
import aoc3.dark.architecture.Mem;

import javax.swing.*;
import javax.swing.event.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Erik Axel Nielsen <erikaxel@stud.ntnu.no>
 * @version 07.jun.2004
 */
final class VariableListModel implements ListModel, IExecutionListener
{
  private final List<ListDataListener> listeners = new ArrayList<ListDataListener>();
  private final List<String> elements = new ArrayList<String>();
  private Architecture model;

  // --------------------- Begin Interface IExecutionListener ---------------------

  public void executionPaused()
  {
    change(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize()));
  }

  public void lineChanged(int lineNumber)
  {
    change(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize()));
  }

// --------------------- End Interface IExecutionListener ---------------------

// --------------------- Begin Interface ListModel ---------------------

  public int getSize()
  {
    return elements.size();
  }

  public Object getElementAt(int index)
  {
    if (model == null)
      return null;
    return elements.get(index);
  }

  public void addListDataListener(ListDataListener l)
  {
    listeners.add(l);
  }

  public void removeListDataListener(ListDataListener l)
  {
    listeners.remove(l);
  }

// --------------------- End Interface ListModel ---------------------

  public void setModel(Architecture model)
  {
    int oldSize = getSize();
    this.model = model;
    change(new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, oldSize));
  }

  /**
   * Notifies all listeners of some event
   *
   * @param event
   */
  private void change(ListDataEvent event)
  {
    updateModel();
    for (ListDataListener listener : listeners)
      listener.contentsChanged(event);
  }

  private void updateModel()
  {
    if (model == null)
      return;

    elements.clear();
    switch (model.getArchitecture()) {
      case Architecture.ARCH_ACCUMULATOR:
        elements.add("Acc = " + model.getRun().a);
        break;
      case Architecture.ARCH_MEMORY:
        break;
      case Architecture.ARCH_INDEX:
        break;
      case Architecture.ARCH_STACK:
        //elements.add("Stakk");
        for (int i = model.getRun().regs[31]; i < model.getMem().memSize; i++)
          elements.add(model.getMem().getValue(i) + " ");
        break;
      case Architecture.ARCH_LOAD:
        elements.add("Registre");
        int[] regs = model.getRun().regs;
        for (int i = 0; i < regs.length; i++)
          // vi vil ikke vise tomme registre. Vi vil heller ikke vise register 31, som bare inneholder en stackpointer.
          if (regs[i] != 0 && i != 31)
            elements.add("    " + i + ": " + regs[i]);
        elements.add("Minne");
        Mem mem = model.getMem();
        for (int i = 0; i < mem.memSize; i++) {
          if (mem.wasChanged(i))
            elements.add("    " + i + ": " + mem.getValue(i));
        }
        break;
    }
  }
}

