package aoc3.dark.architecture;

//
// Classes used for building the data memory
//=========================================================


public final class Mem
{
  private final boolean[] wasChanged;
  public final int memSize;
  /**
   * Vector, where the memPos are stored.
   */
  private final int[] memPos;
  /**
   * We would like to be able to reset the state of the memory to a
   * previous state (i.e. resetting the memory to its original state
   * after code parsing). memBackup keeps track of this state.
   */
  private final int[] memBackup;

  public Mem()
  {
    memSize = 1024;

    memPos = new int[memSize];
    wasChanged = new boolean[memSize];
    memBackup = new int[memSize];
  }

  /**
   * Save a backup of the state of the memory.
   */
  public void backupState()
  {
    System.arraycopy(memPos, 0, memBackup, 0, memSize);
  }

  /**
   * dump the data memory
   */
  public void dump()
  {
    for (int i = 0; i < memSize; i++) {
      if (memPos[i] != 0)
        System.out.println(i + ":\t" + memPos[i]);
    }
  }

  /**
   * Remove everything from data memory
   */
  public void empty()
  {
    // memPos = new int[memSize];
    for (int i = 0; i < memSize; i++) {
      memPos[i] = 0;
      wasChanged[i] = false;
    }
  }

  /**
   * Get a symbol from the data memory
   *
   * @param i
   */
  public int getValue(int i)
  {
    return memPos[i];
  }

  /**
   * Reset the memory to the backup state.
   */
  public void resetState()
  {
    for (int i = 0; i < memSize; i++) {
      memPos[i] = memBackup[i];
      wasChanged[i] = false;
    }
  }

  /**
   * Add an identifier to data memory
   *
   * @param data
   * @param addr
   */
  public void setValue(int addr, int data)
  {
    memPos[addr] = data;
    wasChanged[addr] = true;
  }

  public boolean wasChanged(int i)
  {
    return wasChanged[i];
  }
}

