/*
 * The entries in the symbol table
 */
package aoc3.dark.architecture;

import aoc3.dark.ExecuteException;


/**
 * A symbol - entry in symbol-table
 */
public final class Run
{
  public final InstructionMemory instructionMemory;
  public boolean stop;
  /**
   * Internal registers
   */
  public int a;
  private int b;
  private int c;
  public final int[] regs;
  public int pc;
  public static final int exts = 0;
  private int ints;

  private final Mem mem;
  private final Retst retst;

  Run(Mem mem)
  {
    this.mem = mem;
    regs = new int[32];
    retst = new Retst();
    instructionMemory = new InstructionMemory();
    clear();
  }

  public void clear()
  {
    for (int i = 0; i < regs.length; i++)
      regs[i] = 0;
    regs[31] = mem.memSize;
    a = b = c = ints = 0;

    stop = false;
  }

  public void dump()
  {
    System.out.println("PC = " + pc);
    System.out.println("a: " + a + "\tb: " + b + "\tc: " + c + "\tRegs:");
    for (int i = 0; i < 32; i++)
      System.out.print(regs[i] + "\t");
    System.out.println(" ");
    System.out.println("Memory:");
    mem.dump();
  }

  public boolean executeOne(boolean ignorebreak) throws ExecuteException
  {
    if (pc >= instructionMemory.next()) {// Reached end of exection
      stop = true;
      return false;
    }
    //dump();
    Instruction i = instructionMemory.getInstr(pc);
    if (!ignorebreak && i.stop)
      return false;
    //System.out.println(Inte.names[i.opCode] + "\t" + i.data);
    switch (i.opCode) {
      case Inte.LDAI:
        a = i.data;
        break;
      case Inte.LDAM:
        a = mem.getValue(i.data);
        break;
      case Inte.LDAR:
        a = regs[i.data];
        break;
      case Inte.LDAP:
        try {
          a = mem.getValue(regs[i.data]);
          break;
        } catch (ArrayIndexOutOfBoundsException e) { // Dette skyldes vanligvis stack underflow
          stop = true;
          throw new ExecuteException("Stakk underflyt");
        }
      case Inte.STAM:
        mem.setValue(i.data, a);
        break;
      case Inte.STAR:
        if (i.data > 0)
          regs[i.data] = a;
        break;
      case Inte.STAP:
        try {
          mem.setValue(regs[i.data], a);
          break;
        } catch (ArrayIndexOutOfBoundsException e) { // Dette skyldes kun stack overflow
          stop = true;
          throw new ExecuteException("Stakk overflyt");
        }
      case Inte.INCA:
        a++;
        break;
      case Inte.DECA:
        a--;
        break;
      case Inte.LDBI:
        b = i.data;
        break;
      case Inte.LDBM:
        b = mem.getValue(i.data);
        break;
      case Inte.LDBR:
        b = regs[i.data];
        break;
      case Inte.LDBP:
        b = mem.getValue(regs[i.data]);
        break;
      case Inte.STBM:
        mem.setValue(i.data, b);
        break;
      case Inte.STBR:
        if (i.data > 0)
          regs[i.data] = b;
        break;
      case Inte.STBP:
        mem.setValue(regs[i.data], b);
        break;
      case Inte.INCB:
        b++;
        break;
      case Inte.DECB:
        b--;
        break;
      case Inte.LDCI:
        c = i.data;
        break;
      case Inte.LDCM:
        c = mem.getValue(i.data);
        break;
      case Inte.LDCR:
        c = regs[i.data];
        break;
      case Inte.LDCP:
        c = mem.getValue(regs[i.data]);
        break;
      case Inte.STCM:
        mem.setValue(i.data, c);
        break;
      case Inte.STCR:
        if (i.data > 0)
          regs[i.data] = c;
        break;
      case Inte.STCP:
        mem.setValue(regs[i.data], c);
        break;
      case Inte.INCC:
        c++;
        break;
      case Inte.DECC:
        c--;
        break;
      case Inte.INCR:
        if (i.data > 0)
          regs[i.data]++;
        break;
      case Inte.DECR:
        if (i.data > 0)
          regs[i.data]--;
        break;
      case Inte.INCM:
        mem.setValue(i.data, mem.getValue(i.data) + 1);
        break;
      case Inte.DECM:
        mem.setValue(i.data, mem.getValue(i.data) - 1);
        break;
      case Inte.ADD:
        c = a + b;
        break;
      case Inte.SUB:
        c = a - b;
        break;
      case Inte.MUL:
        c = a * b;
        break;
      case Inte.DIV:
        c = a / b;
        break;
      case Inte.MOD:
        c = a % b;
        break;
      case Inte.AND:
        c = (a != 0 && b != 0) ? 1 : 0;
        break;
      case Inte.OR:
        c = (a != 0 || b != 0) ? 1 : 0;
        break;
      case Inte.XOR:
        c = ((a != 0 && b == 0) || (a == 0 && b != 0)) ? 1 : 0;
        break;
      case Inte.NOT:
        c = (a == 0) ? 1 : 0;
        break;
      case Inte.BAND:
        c = a & b;
        break;
      case Inte.BOR:
        c = a | b;
        break;
      case Inte.BXOR:
        c = a ^ b;
        break;
      case Inte.BNOT:
        c = ~a;
        break;
      case Inte.MVCA:
        a = c;
        break;
      case Inte.JMP:
        pc = i.data - 1;
        break;
      case Inte.JEQ:
        if (a == b)
          pc = i.data - 1;
        break;
      case Inte.JNE:
        if (a != b)
          pc = i.data - 1;
        break;
      case Inte.JLT:
        if (a < b)
          pc = i.data - 1;
        break;
      case Inte.JLE:
        if (a <= b)
          pc = i.data - 1;
        break;
      case Inte.JGT:
        if (a > b)
          pc = i.data - 1;
        break;
      case Inte.JGE:
        if (a >= b)
          pc = i.data - 1;
        break;
      case Inte.CALL:
        retst.push(pc);
        pc = i.data - 1;
        break;
      case Inte.RET:
        pc = retst.pop();
        break;
      case Inte.STOP:
        // ints++;
        stop = true;
        return false;
    }
    pc++;
    ints++;
    return true;
  }

  /**
   * Will get the linenumber
   *
   * @return The linenumber, or -1 if we have reached the end of exectution
   */
  public int getLineNo()
  {
    if (pc >= instructionMemory.next() || pc < 0) {
      stop = true;
      return -1;
    }
    return instructionMemory.getInstr(pc).line;
  }

  public int reSetBreakFromLineNo(int n)
  {
    for (int j = 0; j < instructionMemory.next(); j++) {
      try {
        Instruction i = instructionMemory.getInstr(j);
        if (i.line >= n) {
          i.stop = !i.stop;
          return i.line;
        }
      } catch (Exception e) {
        return -1;
      }
    }
    return -1;
  }
}

