package aoc3.dark.architecture;

import java.util.ArrayList;
import java.util.List;

/**
 * Classes used for building symbol tables.
 * First made by Ole Ågren.
 * Later refactored and refined by Erik Axel Nielsen <erikaxel@stud.ntnu.no>
 */

public final class SymbolTable
{
  private static SymbolTable instance;

  /**
   * List where the symbols are stored.
   */
  private final List<Symbol> symbols = new ArrayList<Symbol>();
  /**
   * Check all symbols for OK status
   */
  private List<String> unresolved = new ArrayList<String>();

  public static SymbolTable getInstance()
  {
    if (instance == null)
      instance = new SymbolTable();
    return instance;
  }

  private SymbolTable()
  {
  }

  /**
   * Add an identifier to symbol table
   *
   * @param s
   */
  public void addSymbol(Symbol s)
  {
    symbols.add(s);
  }

  /**
   * Dump the symbol table
   */
  public void dump()
  {
    for (Symbol symbol : symbols) {
      symbol.dump();
    }
  }

  /**
   * Remove everything from symbol table
   */
  public void empty()
  {
    symbols.clear();
  }

  /**
   * Get a symbol from the symbol table
   *
   * @param i
   */
  public Symbol getSymbol(int i)
  {
    if (i >= 0 && i < symbols.size())
      return symbols.get(i);
    return null;
  }

  /**
   * Get a symbol from the symbol table
   *
   * @param identifier
   */
  public Symbol getSymbol(String identifier)
  {
    for (Symbol symbol : symbols) {
      if (identifier.equalsIgnoreCase(symbol.identifier))
        return symbol;
    }
    return new Symbol("ERROR", 0, 0);
  }

  public int getSymbolCount()
  {
    return symbols.size();
  }

  public List<String> getUnresolved()
  {
    return unresolved;
  }

  public boolean isOK()
  {
    boolean res = true;
    unresolved = new ArrayList<String>();
    for (Symbol symbol : symbols) {
      if (!symbol.updates.isEmpty()) {
        unresolved.add(symbol.identifier);
        res = false;
      }
    }
    return res;
  }
}

