package aoc3.dark.architecture.index;


import aoc3.dark.architecture.*;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

public final class Parser extends AbstractParser
{
  private static boolean OK = true;

  private static int memPos;

  private static final int X = 1;
  private static final int Y = 2;
  private static final int TMP = 3;
  private static final int TMP2 = 4;
  private final Yylex y;

  /**
   * constructor for string parsing
   *
   * @param mem
   * @param source
   */
  public Parser(String source, Mem mem)
  {
    this(new Yylex(new StringReader(source)), mem);
  }

  /**
   * constructor
   *
   * @param lex
   * @param mem
   */
  private Parser(Yylex lex, Mem mem)
  {
    super(mem);
    memPos = 0;
    OK = true;

    y = lex;

    st.empty();
    st.addSymbol(new Symbol("load", Sym.ATTR1, Sym.LOAD));
    st.addSymbol(new Symbol("store", Sym.ATTR1, Sym.STORE));
    st.addSymbol(new Symbol("add", Sym.EXE, Sym.ADD));
    st.addSymbol(new Symbol("sub", Sym.EXE, Sym.SUB));
    st.addSymbol(new Symbol("mul", Sym.EXE, Sym.MUL));
    st.addSymbol(new Symbol("div", Sym.EXE, Sym.DIV));
    st.addSymbol(new Symbol("mod", Sym.EXE, Sym.MOD));
    st.addSymbol(new Symbol("and", Sym.EXE, Sym.AND));
    st.addSymbol(new Symbol("or", Sym.EXE, Sym.OR));
    st.addSymbol(new Symbol("xor", Sym.EXE, Sym.XOR));
    st.addSymbol(new Symbol("not", Sym.ATTR0, Sym.NOT));
    st.addSymbol(new Symbol("band", Sym.EXE, Sym.BAND));
    st.addSymbol(new Symbol("bor", Sym.EXE, Sym.BOR));
    st.addSymbol(new Symbol("bxor", Sym.EXE, Sym.BXOR));
    st.addSymbol(new Symbol("bnot", Sym.ATTR0, Sym.BNOT));
    st.addSymbol(new Symbol("cmp", Sym.ATTR1, Sym.CMP));
    st.addSymbol(new Symbol("ret", Sym.ATTR0, Sym.RET));
    st.addSymbol(new Symbol("data", Sym.DATA, Sym.DATA));
    st.addSymbol(new Symbol("stop", Sym.ATTR0, Sym.STOP));
    st.addSymbol(new Symbol("end", Sym.ATTR1, Sym.END));
    st.addSymbol(new Symbol("jmp", Sym.JUMPS, Sym.JMP));
    st.addSymbol(new Symbol("jeq", Sym.JUMPS, Sym.JEQ));
    st.addSymbol(new Symbol("jne", Sym.JUMPS, Sym.JNE));
    st.addSymbol(new Symbol("jlt", Sym.JUMPS, Sym.JLT));
    st.addSymbol(new Symbol("jle", Sym.JUMPS, Sym.JLE));
    st.addSymbol(new Symbol("jgt", Sym.JUMPS, Sym.JGT));
    st.addSymbol(new Symbol("jge", Sym.JUMPS, Sym.JGE));
    st.addSymbol(new Symbol("call", Sym.JUMPS, Sym.CALL));
    st.addSymbol(new Symbol("loadx", Sym.ATTR1, Sym.LOADX));
    st.addSymbol(new Symbol("loady", Sym.ATTR1, Sym.LOADY));
    st.addSymbol(new Symbol("loadax",
                            Sym.ATTR1, Sym.LOADAX));
    st.addSymbol(new Symbol("loaday",
                            Sym.ATTR1, Sym.LOADAY));
    st.addSymbol(new Symbol("loadix",
                            Sym.ATTR1, Sym.LOADIX));
    st.addSymbol(new Symbol("loadiy",
                            Sym.ATTR1, Sym.LOADIY));
    st.addSymbol(new Symbol("loadixy",
                            Sym.ATTR1, Sym.LOADIXY));
    st.addSymbol(new Symbol("storex",
                            Sym.ATTR1, Sym.STOREX));
    st.addSymbol(new Symbol("storey",
                            Sym.ATTR1, Sym.STOREY));
    st.addSymbol(new Symbol("storeix",
                            Sym.ATTR1, Sym.STOREIX));
    st.addSymbol(new Symbol("storeiy",
                            Sym.ATTR1, Sym.STOREIY));
    st.addSymbol(new Symbol("storeixy",
                            Sym.ATTR1, Sym.STOREIXY));
    st.addSymbol(new Symbol("tax", Sym.ATTR0, Sym.TAX));
    st.addSymbol(new Symbol("tay", Sym.ATTR0, Sym.TAY));
    st.addSymbol(new Symbol("txa", Sym.ATTR0, Sym.TXA));
    st.addSymbol(new Symbol("txy", Sym.ATTR0, Sym.TXY));
    st.addSymbol(new Symbol("tya", Sym.ATTR0, Sym.TYA));
    st.addSymbol(new Symbol("tyx", Sym.ATTR0, Sym.TYX));
    st.addSymbol(new Symbol("incx", Sym.ATTR0, Sym.INCX));
    st.addSymbol(new Symbol("incy", Sym.ATTR0, Sym.INCY));
    st.addSymbol(new Symbol("decx", Sym.ATTR0, Sym.DECX));
    st.addSymbol(new Symbol("decy", Sym.ATTR0, Sym.DECY));
  }

  public boolean parse() throws ParseException, IOException
  {
    Yytoken tok;
    boolean done = false;

    if (!OK)
      return false;

    while (!done && (tok = y.yylex()).m_index != 0) {
      Symbol s;
      Yytoken tok2;
      switch (tok.m_index) {
        case Sym.DATA:
          tok2 = y.yylex();
          if (tok2.m_index == Sym.NO) {
            Yytoken tok3 = y.yylex();
            if (tok3.m_index == Sym.SYMBOL) {
              s = st.getSymbol(tok3.m_text);
              s.kind = Inte.LABEL;
              s.where = memPos++;
              mem.setValue(s.where, Integer.parseInt(tok2.m_text));
            }
            else {
              AbstractParser.parse_error("Bad data initialization at line " + tok3.m_line);
              return false;
            }
          }
          else if (tok2.m_index == Sym.SYMBOL) {
            s = st.getSymbol(tok2.m_text);
            s.kind = Inte.LABEL;
            s.where = memPos++;
          }
          else {
            AbstractParser.parse_error("Bad data initialization at line " + tok2.m_line);
            return false;
          }
          break;
        case Sym.ATTR0:
          switch (tok.m_subtype) {
            case Sym.NOT:
              instructionMemory.add(new Instruction(Inte.NOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.MVCA, 0, tok.m_line));
              break;
            case Sym.BNOT:
              instructionMemory.add(new Instruction(Inte.BNOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.MVCA, 0, tok.m_line));
              break;
            case Sym.STOP:
              instructionMemory.add(new Instruction(Inte.STOP, 0, tok.m_line));
              break;
            case Sym.RET:
              instructionMemory.add(new Instruction(Inte.RET, 0, tok.m_line));
              break;
            case Sym.TAX:
              instructionMemory.add(new Instruction(Inte.STAR, X, tok.m_line));
              break;
            case Sym.TAY:
              instructionMemory.add(new Instruction(Inte.STAR, Y, tok.m_line));
              break;
            case Sym.TXA:
              instructionMemory.add(new Instruction(Inte.LDAR, X, tok.m_line));
              break;
            case Sym.TYA:
              instructionMemory.add(new Instruction(Inte.LDAR, Y, tok.m_line));
              break;
            case Sym.TXY:
              instructionMemory.add(new Instruction(Inte.LDCR, X, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCR, Y, tok.m_line));
              break;
            case Sym.TYX:
              instructionMemory.add(new Instruction(Inte.LDCR, Y, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCR, X, tok.m_line));
              break;
            case Sym.INCX:
              instructionMemory.add(new Instruction(Inte.INCR, X, tok.m_line));
              break;
            case Sym.INCY:
              instructionMemory.add(new Instruction(Inte.INCR, Y, tok.m_line));
              break;
            case Sym.DECX:
              instructionMemory.add(new Instruction(Inte.DECR, X, tok.m_line));
              break;
            case Sym.DECY:
              instructionMemory.add(new Instruction(Inte.DECR, Y, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.ATTR1:
          tok2 = y.yylex();
          s = st.getSymbol(tok2.m_text);
          switch (tok.m_subtype) {
            case Sym.LOAD:
              if (tok2.m_index == Sym.NO) {
                instructionMemory.add(new Instruction(Inte.LDAI,
                                                      Integer.parseInt(tok2.m_text), tok.m_line));
              }
              else if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for load, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STORE:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.STAM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for store, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.CMP:
              if (tok2.m_index == Sym.NO) {
                instructionMemory.add(new Instruction(Inte.LDBI,
                                                      Integer.parseInt(tok2.m_text), tok.m_line));
              }
              else if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDBM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for load, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.END:
              if (tok2.m_index == Sym.LABELP) {
                run.pc = s.where;
              }
              done = true;
              break;
            case Sym.LOADX:
              if (tok2.m_index == Sym.NO) {
                instructionMemory.add(new Instruction(Inte.LDCI,
                                                      Integer.parseInt(tok2.m_text), tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, X, tok.m_line));
              }
              else if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDCM, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, X, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loadx, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADY:
              if (tok2.m_index == Sym.NO) {
                instructionMemory.add(new Instruction(Inte.LDCI,
                                                      Integer.parseInt(tok2.m_text), tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, Y, tok.m_line));
              }
              else if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDCM, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, Y, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loady, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADAX:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDCI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, X, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loadax, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADAY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDCI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, Y, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loaday, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADIX:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, X, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loadix, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADIY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, Y, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loadiy, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.LOADIXY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, X, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAP, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, Y, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for loadixy, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STOREX:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDBR, X, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STBM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for storex, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STOREY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDBR, Y, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STBM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for storex, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STOREIX:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.STAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, X, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for storeix, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STOREIY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.STAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, Y, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for storeiy, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.STOREIXY:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.STAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, X, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAP, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBR, Y, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, TMP, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDAR, TMP2, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STAP, TMP, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for storeixy, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              break;
          }
          break;
        case Sym.EXE:
          tok2 = y.yylex();
          s = st.getSymbol(tok2.m_text);
          switch (tok2.m_index) {
            case Sym.NO:
              instructionMemory.add(new Instruction(Inte.LDBI,
                                                    Integer.parseInt(tok2.m_text), tok.m_line));
              break;
            case Sym.LABEL:
              instructionMemory.add(new Instruction(Inte.LDBM, s.where, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Erroneous data type for instruction, " +
                                         tok2.m_text + " at line " + tok2.m_line);
              return false;
          }
          switch (tok.m_subtype) {
            case Sym.ADD:
              instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
              break;
            case Sym.SUB:
              instructionMemory.add(new Instruction(Inte.SUB, 0, tok.m_line));
              break;
            case Sym.MUL:
              instructionMemory.add(new Instruction(Inte.MUL, 0, tok.m_line));
              break;
            case Sym.DIV:
              instructionMemory.add(new Instruction(Inte.DIV, 0, tok.m_line));
              break;
            case Sym.MOD:
              instructionMemory.add(new Instruction(Inte.MOD, 0, tok.m_line));
              break;
            case Sym.AND:
              instructionMemory.add(new Instruction(Inte.AND, 0, tok.m_line));
              break;
            case Sym.OR:
              instructionMemory.add(new Instruction(Inte.OR, 0, tok.m_line));
              break;
            case Sym.XOR:
              instructionMemory.add(new Instruction(Inte.XOR, 0, tok.m_line));
              break;
            case Sym.BAND:
              instructionMemory.add(new Instruction(Inte.BAND, 0, tok.m_line));
              break;
            case Sym.BOR:
              instructionMemory.add(new Instruction(Inte.BOR, 0, tok.m_line));
              break;
            case Sym.BXOR:
              instructionMemory.add(new Instruction(Inte.BXOR, 0, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown instruction, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          instructionMemory.add(new Instruction(Inte.MVCA, 0, tok.m_line));
          break;
        case Sym.JUMPS:
          tok2 = y.yylex();
          s = st.getSymbol(tok2.m_text);
          switch (tok2.m_index) {
            case Sym.LABELP:
              break;
            case Sym.SYMBOL:
              s.updates.add(instructionMemory.next());
              break;
            default:
              AbstractParser.parse_error("Unknown jump label, " + tok2.m_text +
                                         " at line " + tok2.m_line);
              return false;
          }
          switch (tok.m_subtype) {
            case Sym.JMP:
              instructionMemory.add(new Instruction(Inte.JMP, s.where, tok.m_line));
              break;
            case Sym.JEQ:
              instructionMemory.add(new Instruction(Inte.JEQ, s.where, tok.m_line));
              break;
            case Sym.JNE:
              instructionMemory.add(new Instruction(Inte.JNE, s.where, tok.m_line));
              break;
            case Sym.JLT:
              instructionMemory.add(new Instruction(Inte.JLT, s.where, tok.m_line));
              break;
            case Sym.JLE:
              instructionMemory.add(new Instruction(Inte.JLE, s.where, tok.m_line));
              break;
            case Sym.JGT:
              instructionMemory.add(new Instruction(Inte.JGT, s.where, tok.m_line));
              break;
            case Sym.JGE:
              instructionMemory.add(new Instruction(Inte.JGE, s.where, tok.m_line));
              break;
            case Sym.CALL:
              instructionMemory.add(new Instruction(Inte.CALL, s.where, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown jump instruction, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.SYMBOL:
          s = st.getSymbol(tok.m_text);
          s.kind = Sym.LABELP;
          s.where = instructionMemory.next();
          for (int i = 0; i < s.updates.size(); i++) {
            int next = s.updates.get(i);
            try {
              instructionMemory.getInstr(next).data = s.where;
            } catch (Exception e) {
              AbstractParser.parse_error("Failed to find instruction " + i +
                                         " to update at line " + tok.m_line);
              return false;
            }
          }
          s.updates = new ArrayList<Integer>();
          break;
        case Sym.NO:
          AbstractParser.parse_error("Unused number " + tok.m_text +
                                     " at line " + tok.m_line);
          return false;
        case Sym.STR:
          AbstractParser.parse_error("Unused string '" + tok.m_text +
                                     "' at line " + tok.m_line);
          return false;
        default:
          break;
      }
    }
    return OK;
  }
}

