package aoc3.dark.architecture.load;


import aoc3.dark.architecture.*;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

public final class Parser extends AbstractParser
{
  private static boolean OK = true;

  private static int memPos;
  private final Yylex y;

  /**
   * Constructor for string parsing.
   *
   * @param mem
   * @param source
   */
  public Parser(String source, Mem mem)
  {
    this(new Yylex(new StringReader(source)), mem);
  }

  /**
   * constructor
   *
   * @param mem
   * @param lex
   */
  private Parser(Yylex lex, Mem mem)
  {
    super(mem);
    memPos = 0;
    OK = true;

    y = lex;

    st.empty();
    st.addSymbol(new Symbol("load", Sym.LOSTO, Sym.LOAD));
    st.addSymbol(new Symbol("store", Sym.LOSTO, Sym.STORE));
    st.addSymbol(new Symbol("inc", Sym.ATTR1, Sym.INC));
    st.addSymbol(new Symbol("dec", Sym.ATTR1, Sym.DEC));
    st.addSymbol(new Symbol("mv", Sym.ATTR2, Sym.MOV));
    st.addSymbol(new Symbol("mov", Sym.ATTR2, Sym.MOV));
    st.addSymbol(new Symbol("not", Sym.ATTR2, Sym.NOT));
    st.addSymbol(new Symbol("bnot", Sym.ATTR2, Sym.BNOT));
    st.addSymbol(new Symbol("add", Sym.ATTR3, Sym.ADD));
    st.addSymbol(new Symbol("sub", Sym.ATTR3, Sym.SUB));
    st.addSymbol(new Symbol("mul", Sym.ATTR3, Sym.MUL));
    st.addSymbol(new Symbol("div", Sym.ATTR3, Sym.DIV));
    st.addSymbol(new Symbol("mod", Sym.ATTR3, Sym.MOD));
    st.addSymbol(new Symbol("and", Sym.ATTR3, Sym.AND));
    st.addSymbol(new Symbol("or", Sym.ATTR3, Sym.OR));
    st.addSymbol(new Symbol("xor", Sym.ATTR3, Sym.XOR));
    st.addSymbol(new Symbol("band", Sym.ATTR3, Sym.BAND));
    st.addSymbol(new Symbol("bor", Sym.ATTR3, Sym.BOR));
    st.addSymbol(new Symbol("bxor", Sym.ATTR3, Sym.BXOR));
    st.addSymbol(new Symbol("ret", Sym.ATTR0, Sym.RET));
    st.addSymbol(new Symbol("jeq", Sym.JUMP1, Sym.JEQ));
    st.addSymbol(new Symbol("jne", Sym.JUMP1, Sym.JNE));
    st.addSymbol(new Symbol("jlt", Sym.JUMP1, Sym.JLT));
    st.addSymbol(new Symbol("jle", Sym.JUMP1, Sym.JLE));
    st.addSymbol(new Symbol("jgt", Sym.JUMP1, Sym.JGT));
    st.addSymbol(new Symbol("jge", Sym.JUMP1, Sym.JGE));
    st.addSymbol(new Symbol("data", Sym.DATA, Sym.DATA));
    st.addSymbol(new Symbol("stop", Sym.ATTR0, Sym.STOP));
    st.addSymbol(new Symbol("end", Sym.ATTR1, Sym.END));
    st.addSymbol(new Symbol("jmp", Sym.JUMP0, Sym.JMP));
    st.addSymbol(new Symbol("call", Sym.JUMP0, Sym.CALL));
    st.addSymbol(new Symbol("sp", Sym.REG, 31));
    st.addSymbol(new Symbol("zero", Sym.REG, 0));
  }

  public boolean parse() throws ParseException, IOException
  {
    Yytoken tok;
    boolean done = false;

    if (!OK)
      return false;

    while (!done && (tok = y.yylex()).m_index != 0) {
      Symbol s;

      Yytoken tok2;
      Yytoken tok3;
      Yytoken tok4;
      switch (tok.m_index) {
        case Sym.DATA:
          tok2 = y.yylex();
          if (tok2.m_index == Sym.NO) {
            tok3 = y.yylex();
            if (tok3.m_index == Sym.SYMBOL) {
              s = st.getSymbol(tok3.m_text);
              s.kind = Inte.LABEL;
              s.where = memPos++;
              mem.setValue(s.where, Integer.parseInt(tok2.m_text));
            }
            else {
              AbstractParser.parse_error("Bad data initialization at line " + tok3.m_line);
              return false;
            }
          }
          else if (tok2.m_index == Sym.SYMBOL) {
            s = st.getSymbol(tok2.m_text);
            s.kind = Inte.LABEL;
            s.where = memPos++;
          }
          else {
            AbstractParser.parse_error("Bad data initialization at line " + tok2.m_line);
            return false;
          }
          break;
        case Sym.ATTR0:
          switch (tok.m_subtype) {
            case Sym.STOP:
              instructionMemory.add(new Instruction(Inte.STOP, 0, tok.m_line));
              break;
            case Sym.RET:
              instructionMemory.add(new Instruction(Inte.RET, 0, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.ATTR1:
          tok2 = y.yylex();
          s = st.getSymbol(tok2.m_text);
          switch (tok.m_subtype) {
            case Sym.INC:
              if (tok2.m_index == Sym.REG) {
                instructionMemory.add(new Instruction(Inte.INCR, tok2.m_subtype, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Only a register allowed for increase, not " +
                                           tok2.m_text + " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.DEC:
              if (tok2.m_index == Sym.REG) {
                instructionMemory.add(new Instruction(Inte.DECR, tok2.m_subtype, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Only a register allowed for increase, not " +
                                           tok2.m_text + " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.END:
              if (tok2.m_index == Sym.LABELP) {
                run.pc = s.where;
              }
              done = true;
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.ATTR2:
          tok2 = y.yylex();
          if (tok2.m_index != Sym.REG) {
            AbstractParser.parse_error("Erroneous name given for register, " + tok2.m_text +
                                       " at line " + tok2.m_line);
            return false;
          }
          tok3 = y.yylex();
          if (tok3.m_index == Sym.REG) {
            instructionMemory.add(new Instruction(Inte.LDAR, tok3.m_subtype, tok.m_line));
          }
          else if (tok3.m_index == Sym.NO) {
            instructionMemory.add(new Instruction(Inte.LDAI,
                                                  Integer.parseInt(tok3.m_text), tok.m_line));
          }
          else {
            AbstractParser.parse_error("Erroneous data given, " + tok3.m_text +
                                       " at line " + tok3.m_line);
            return false;
          }
          switch (tok.m_subtype) {
            case Sym.MOV:
              instructionMemory.add(new Instruction(Inte.STAR, tok2.m_subtype, tok.m_line));
              break;
            case Sym.NOT:
              instructionMemory.add(new Instruction(Inte.NOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCR, tok2.m_subtype, tok.m_line));
              break;
            case Sym.BNOT:
              instructionMemory.add(new Instruction(Inte.BNOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCR, tok2.m_subtype, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.ATTR3:
          tok2 = y.yylex();
          if (tok2.m_index != Sym.REG) {
            AbstractParser.parse_error("Erroneous name given for register, " + tok2.m_text +
                                       " at line " + tok2.m_line);
            return false;
          }
          tok3 = y.yylex();
          if (tok3.m_index != Sym.REG) {
            AbstractParser.parse_error("Erroneous name given for register, " + tok3.m_text +
                                       " at line " + tok3.m_line);
            return false;
          }
          instructionMemory.add(new Instruction(Inte.LDAR, tok3.m_subtype, tok.m_line));
          tok4 = y.yylex();
          if (tok4.m_index == Sym.REG) {
            instructionMemory.add(new Instruction(Inte.LDBR, tok4.m_subtype, tok.m_line));
          }
          else if (tok4.m_index == Sym.NO) {
            instructionMemory.add(new Instruction(Inte.LDBI,
                                                  Integer.parseInt(tok4.m_text), tok.m_line));
          }
          else {
            AbstractParser.parse_error("Erroneous data given, " + tok4.m_text +
                                       " at line " + tok4.m_line);
            return false;
          }
          switch (tok.m_subtype) {
            case Sym.ADD:
              instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
              break;
            case Sym.SUB:
              instructionMemory.add(new Instruction(Inte.SUB, 0, tok.m_line));
              break;
            case Sym.MUL:
              instructionMemory.add(new Instruction(Inte.MUL, 0, tok.m_line));
              break;
            case Sym.DIV:
              instructionMemory.add(new Instruction(Inte.DIV, 0, tok.m_line));
              break;
            case Sym.MOD:
              instructionMemory.add(new Instruction(Inte.MOD, 0, tok.m_line));
              break;
            case Sym.AND:
              instructionMemory.add(new Instruction(Inte.AND, 0, tok.m_line));
              break;
            case Sym.OR:
              instructionMemory.add(new Instruction(Inte.OR, 0, tok.m_line));
              break;
            case Sym.XOR:
              instructionMemory.add(new Instruction(Inte.XOR, 0, tok.m_line));
              break;
            case Sym.BAND:
              instructionMemory.add(new Instruction(Inte.BAND, 0, tok.m_line));
              break;
            case Sym.BOR:
              instructionMemory.add(new Instruction(Inte.BOR, 0, tok.m_line));
              break;
            case Sym.BXOR:
              instructionMemory.add(new Instruction(Inte.BXOR, 0, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Erroneous instruction " +
                                         tok.m_text + " at line " + tok.m_line);
              return false;
          }
          instructionMemory.add(new Instruction(Inte.STCR, tok2.m_subtype, tok.m_line));
          break;
        case Sym.LOSTO:
          tok2 = y.yylex();
          if (tok2.m_index != Sym.REG) {
            AbstractParser.parse_error("Erroneous name given for register, " + tok2.m_text +
                                       " at line " + tok2.m_line);
            return false;
          }
          tok3 = y.yylex();
          if (tok3.m_index == Sym.REG) {
            tok4 = y.yylex();
            if (tok4.m_index != Sym.NO) {
              AbstractParser.parse_error("Load/store instruction without offset, " +
                                         tok4.m_text + " at line " + tok3.m_line);
              return false;
            }
            instructionMemory.add(new Instruction(Inte.LDAR, tok3.m_subtype, tok.m_line));
            instructionMemory.add(new Instruction(Inte.LDBI,
                                                  Integer.parseInt(tok4.m_text), tok.m_line));
            instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
            instructionMemory.add(new Instruction(Inte.STCR, 1, tok.m_line));
            if (tok.m_subtype == Sym.LOAD) {
              instructionMemory.add(new Instruction(Inte.LDAP, 1, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAR, tok2.m_subtype, tok.m_line));
            }
            else if (tok.m_subtype == Sym.STORE) {
              instructionMemory.add(new Instruction(Inte.LDAR, tok2.m_subtype, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, 1, tok.m_line));
            }
            else {
              AbstractParser.parse_error("Erroneous instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
            }
          }
          else if (tok3.m_index == Sym.NO) {
            if (tok.m_subtype == Sym.STORE) {
              AbstractParser.parse_error("Store instruction followed by numeric constant, " +
                                         tok3.m_text + " at line " + tok3.m_line);
              return false;
            }
            instructionMemory.add(new Instruction(Inte.LDAI,
                                                  Integer.parseInt(tok3.m_text), tok.m_line));
            instructionMemory.add(new Instruction(Inte.STAR, tok2.m_subtype, tok.m_line));
          }
          else if (tok3.m_index == Sym.LABEL) {
            s = st.getSymbol(tok3.m_text);
            if (tok.m_subtype == Sym.LOAD) {
              instructionMemory.add(new Instruction(Inte.LDAM, s.where, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAR, tok2.m_subtype, tok.m_line));
            }
            else if (tok.m_subtype == Sym.STORE) {
              instructionMemory.add(new Instruction(Inte.LDAR, tok2.m_subtype, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAM, s.where, tok.m_line));
            }
            else {
              AbstractParser.parse_error("Erroneous instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
            }
          }
          else {
            System.out.println("Type of tok3 = " + tok3.m_index);
            System.out.println("Subtype of tok3 = " + tok3.m_subtype);
            AbstractParser.parse_error("Erroneous data given, " + tok3.m_text +
                                       " at line " + tok3.m_line);
            return false;
          }
          break;
        case Sym.JUMP0:
          tok2 = y.yylex();
          s = st.getSymbol(tok2.m_text);
          switch (tok2.m_index) {
            case Sym.LABELP:
              break;
            case Sym.SYMBOL:
              s.updates.add(instructionMemory.next());
              break;
            default:
              AbstractParser.parse_error("Unknown jump label, " + tok2.m_text +
                                         " at line " + tok2.m_line);
              return false;
          }
          switch (tok.m_subtype) {
            case Sym.JMP:
              instructionMemory.add(new Instruction(Inte.JMP, s.where, tok.m_line));
              break;
            case Sym.CALL:
              instructionMemory.add(new Instruction(Inte.CALL, s.where, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown jump instruction, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.JUMP1:
          tok2 = y.yylex();
          if (tok2.m_index != Sym.REG) {
            AbstractParser.parse_error("Instruction not followed by register, " +
                                       tok2.m_text + " at line " + tok2.m_line);
            return false;
          }
          instructionMemory.add(new Instruction(Inte.LDAR, tok2.m_subtype, tok.m_line));
          tok3 = y.yylex();
          if (tok3.m_index != Sym.REG) {
            AbstractParser.parse_error("Instruction not followed by register, " +
                                       tok3.m_text + " at line " + tok3.m_line);
            return false;
          }
          instructionMemory.add(new Instruction(Inte.LDBR, tok3.m_subtype, tok.m_line));
          tok4 = y.yylex();
          s = st.getSymbol(tok4.m_text);
          switch (tok4.m_index) {
            case Sym.LABELP:
              break;
            case Sym.SYMBOL:
              s.updates.add(instructionMemory.next());
              break;
            default:
              AbstractParser.parse_error("Unknown jump label, " +
                                         tok4.m_text + " at line " + tok4.m_line);
              return false;
          }
          switch (tok.m_subtype) {
            case Sym.JEQ:
              instructionMemory.add(new Instruction(Inte.JEQ, s.where, tok.m_line));
              break;
            case Sym.JNE:
              instructionMemory.add(new Instruction(Inte.JNE, s.where, tok.m_line));
              break;
            case Sym.JLT:
              instructionMemory.add(new Instruction(Inte.JLT, s.where, tok.m_line));
              break;
            case Sym.JLE:
              instructionMemory.add(new Instruction(Inte.JLE, s.where, tok.m_line));
              break;
            case Sym.JGT:
              instructionMemory.add(new Instruction(Inte.JGT, s.where, tok.m_line));
              break;
            case Sym.JGE:
              instructionMemory.add(new Instruction(Inte.JGE, s.where, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown jump instruction, " +
                                         tok.m_text + " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.SYMBOL:
          s = st.getSymbol(tok.m_text);
          s.kind = Sym.LABELP;
          s.where = instructionMemory.next();
          for (int i = 0; i < s.updates.size(); i++) {
            int next = s.updates.get(i);
            try {
              instructionMemory.getInstr(next).data = s.where;
            } catch (Exception e) {
              AbstractParser.parse_error("Failed to find instruction " + i +
                                         " to update at line " + tok.m_line);
              return false;
            }
          }
          s.updates = new ArrayList<Integer>();
          break;
        case Sym.NO:
          AbstractParser.parse_error("Unused number " + tok.m_text +
                                     " at line " + tok.m_line);
          return false;
        case Sym.STR:
          AbstractParser.parse_error("Unused string '" + tok.m_text +
                                     "' at line " + tok.m_line);
          return false;
        default:
          break;
      }
    }
    return OK;
  }
}

