package aoc3.dark.architecture.memory;

import aoc3.dark.architecture.*;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

public final class Parser extends AbstractParser
{
  private static boolean OK;

  private static int memPos;
  private static int regPos = 1;
  private final Yylex y;

  /**
   * constructor for string parsing
   *
   * @param mem
   * @param source
   */
  public Parser(String source, Mem mem)
  {
    this(new Yylex(new StringReader(source)), mem);
  }

  /**
   * main constructor
   *
   * @param mem
   * @param lex
   */
  private Parser(Yylex lex, Mem mem)
  {
    super(mem);
    memPos = 0;
    regPos = 1;
    OK = true;

    y = lex;

    st.empty();
    st.addSymbol(new Symbol("end", Sym.END, 0));
    st.addSymbol(new Symbol("equ", Sym.CONST, 0));
    st.addSymbol(new Symbol("const", Sym.CONST, 0));
    st.addSymbol(new Symbol("Mem", Sym.MEM, 0));
    st.addSymbol(new Symbol("stack", Sym.STACK, 0));
    st.addSymbol(new Symbol("stop", Sym.STOP, 0));
    st.addSymbol(new Symbol("ret", Sym.RET, 0));
    st.addSymbol(new Symbol("mv", Sym.CMD, Internals.MV));
    st.addSymbol(new Symbol("mov", Sym.CMD, Internals.MV));
    st.addSymbol(new Symbol("add", Sym.CMD, Internals.ADD));
    st.addSymbol(new Symbol("sub", Sym.CMD, Internals.SUB));
    st.addSymbol(new Symbol("mul", Sym.CMD, Internals.MUL));
    st.addSymbol(new Symbol("div", Sym.CMD, Internals.DIV));
    st.addSymbol(new Symbol("mod", Sym.CMD, Internals.MOD));
    st.addSymbol(new Symbol("or", Sym.CMD, Internals.OR));
    st.addSymbol(new Symbol("xor", Sym.CMD, Internals.XOR));
    st.addSymbol(new Symbol("and", Sym.CMD, Internals.AND));
    st.addSymbol(new Symbol("not", Sym.CMD, Internals.NOT));
    st.addSymbol(new Symbol("bor", Sym.CMD, Internals.BOR));
    st.addSymbol(new Symbol("bxor", Sym.CMD, Internals.BXOR));
    st.addSymbol(new Symbol("band", Sym.CMD, Internals.BAND));
    st.addSymbol(new Symbol("bnot", Sym.CMD, Internals.BNOT));
    st.addSymbol(new Symbol("call", Sym.JCMD, Internals.CALL));
    st.addSymbol(new Symbol("bra", Sym.JCMD, Internals.JMP));
    st.addSymbol(new Symbol("beq", Sym.JCMD, Internals.JEQ));
    st.addSymbol(new Symbol("bne", Sym.JCMD, Internals.JNE));
    st.addSymbol(new Symbol("bge", Sym.JCMD, Internals.JGE));
    st.addSymbol(new Symbol("bgt", Sym.JCMD, Internals.JGT));
    st.addSymbol(new Symbol("blt", Sym.JCMD, Internals.JLT));
    st.addSymbol(new Symbol("ble", Sym.JCMD, Internals.JLE));

    // System.out.println("Constructor for parsing");
  }

  private void getComma() throws ParseException, IOException
  {
    Yytoken tok = y.yylex();
    if (tok.m_index == Sym.COMMA)
      return;
    AbstractParser.parse_error("Expecting comma, got '" + tok.m_text +
                               "' at line " + tok.m_line);
  }

  private static int getCommand2Args(Symbol s) throws ParseException
  {
    switch (s.where) {
      case Internals.MV:
        return Inte.ADD;
      case Internals.NOT:
        return Inte.NOT;
      case Internals.BNOT:
        return Inte.BNOT;
      default:
        parse_error("Malformed instruction '" + s.identifier + '\'');
    }
    return Inte.STOP;
  }

  private static int getCommand3Args(Symbol s) throws ParseException
  {
    switch (s.where) {
      case Internals.ADD:
        return Inte.ADD;
      case Internals.SUB:
        return Inte.SUB;
      case Internals.MUL:
        return Inte.MUL;
      case Internals.DIV:
        return Inte.DIV;
      case Internals.MOD:
        return Inte.MOD;
      case Internals.OR:
        return Inte.OR;
      case Internals.XOR:
        return Inte.XOR;
      case Internals.AND:
        return Inte.BAND;
      case Internals.BOR:
        return Inte.BOR;
      case Internals.BXOR:
        return Inte.BXOR;
      case Internals.BAND:
        return Inte.BAND;
      default:
        parse_error("Malformed instruction '" + s.identifier + '\'');
    }
    return Inte.STOP;
  }

  private int getImmediate(Yytoken tok)
  {
    switch (tok.m_index) {
      case Sym.SYMBNO:
        Symbol s = st.getSymbol(tok.m_text);
        return s.where;
      case Sym.NO:
        return Integer.parseInt(tok.m_text);
      default:
        return 0;
    }
  }

  private static int getJump1Arg(Symbol s) throws ParseException
  {
    switch (s.where) {
      case Internals.JMP:
        return Inte.JMP;
      case Internals.CALL:
        return Inte.CALL;
      default:
        parse_error("Malformed instruction '" + s.identifier + '\'');
    }
    return Inte.STOP;
  }

  private static int getJump3Args(Symbol s) throws ParseException
  {
    switch (s.where) {
      case Internals.JEQ:
        return Inte.JEQ;
      case Internals.JNE:
        return Inte.JNE;
      case Internals.JGE:
        return Inte.JGE;
      case Internals.JGT:
        return Inte.JGT;
      case Internals.JLT:
        return Inte.JLT;
      case Internals.JLE:
        return Inte.JLE;
      default:
        parse_error("Malformed instruction '" + s.identifier + '\'');
    }
    return Inte.STOP;
  }

  public boolean parse() throws ParseException, IOException
  {
    boolean done;
    Yytoken tok;
    done = false;

    if (!OK)
      return false;

    while (!done && (tok = y.yylex()).m_index != 0) {
      try {
        done = !parseNextLine(tok);
      } catch (Error e) {
        done = true;
      }
    }

    return OK;
  }

  private int parseAddress() throws ParseException, IOException
  {
    return parseBaseAddress(y.yylex());
  }

  private int parseBaseAddress(Yytoken tok) throws ParseException, IOException
  {
    int i;

    switch (tok.m_index) {
      case Sym.LABEL:
        /* addr */
        return parseSimpleAddress(tok);
      case Sym.LEFT:
        /* (addr* */
        tok = y.yylex();
        i = parseBaseAddress(tok);
        tok = y.yylex();
        switch (tok.m_index) {
          case Sym.RIGHT:
            /* (addr) */
            instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
            instructionMemory.add(new Instruction(Inte.STAR, i, tok.m_line));
            return i;
          case Sym.COMMA:
            /* (addr,*) */
            tok = y.yylex();
            Yytoken tok2 = y.yylex();
            int j;
            switch (tok.m_index) {
              case Sym.LABEL:
                /* (addr,label) */
                j = parseSimpleAddress(tok);
                instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBP, j, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, i, tok.m_line));
                if (j == regPos - 1)
                  regPos--;
                return i;
              case Sym.LEFT:
                /* (addr,addr) */
                j = parseBaseAddress(tok);
                instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBP, j, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, i, tok.m_line));
                if (j == regPos - 1)
                  regPos--;
                return i;
              case Sym.SYMBNO:
              case Sym.NO:
                /* (addr,const|immediate) */
                j = getImmediate(tok);
                instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
                instructionMemory.add(new Instruction(Inte.LDBI, j, tok.m_line));
                instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STCR, i, tok.m_line));
                return i;
              default:
                AbstractParser.parse_error("Bad token '" + tok.m_text +
                                           "' at line " + tok.m_line);
            }
            break;
          default:
            AbstractParser.parse_error("Bad token '" + tok.m_text +
                                       "' at line " + tok.m_line);
        }
        break;
      default:
        AbstractParser.parse_error("Bad address '" + tok.m_text + "' at line " + tok.m_line);
    }
    return 0;
  }

  private boolean parseCommand(Yytoken tok) throws ParseException, IOException
  {
    Yytoken tok2;
    int cmdArgs;
    int i;
    int j;

    Symbol s = st.getSymbol(tok.m_text);
    switch (s.where) {
      case Internals.MV:
      case Internals.NOT:
      case Internals.BNOT:
        /* These InstructionMemory have two operands */
        cmdArgs = 2;
        break;
      default:
        /* The rest have three */
        cmdArgs = 3;
        break;
    }

    if (cmdArgs == 2) {
      i = parseAddress();
      getComma();
      tok2 = y.yylex();
      switch (tok2.m_index) {
        case Sym.LABEL:
        case Sym.LEFT:
          j = parseBaseAddress(tok2);
          instructionMemory.add(new Instruction(Inte.LDAP, j, tok.m_line));
          instructionMemory.add(new Instruction(Inte.LDBI, 0, tok.m_line));
          instructionMemory.add(new Instruction(Parser.getCommand2Args(s), 0, tok.m_line));
          instructionMemory.add(new Instruction(Inte.STCP, i, tok.m_line));
          if (j == regPos - 1)
            regPos--;
          else
            System.out.println("j != (Parser.regPos - 1)");
          if (i == regPos - 1)
            regPos--;
          else
            System.out.println("i != (Parser.regPos - 1)");
          break;
        case Sym.SYMBNO:
        case Sym.NO:
          j = getImmediate(tok2);
          instructionMemory.add(new Instruction(Inte.LDAI, j, tok.m_line));
          instructionMemory.add(new Instruction(Inte.LDBI, 0, tok.m_line));
          instructionMemory.add(new Instruction(Parser.getCommand2Args(s), 0, tok.m_line));
          instructionMemory.add(new Instruction(Inte.STCP, i, tok.m_line));
          if (i == regPos - 1)
            regPos--;
          else
            System.out.println("i != (Parser.regPos - 1)");
          break;
        default:
          AbstractParser.parse_error("Unknown operand type " + tok2.m_text +
                                     " at line " + tok.m_line);
      }
    }
    else {
      i = parseAddress();
      getComma();
      j = parseAddress();
      getComma();
      tok2 = y.yylex();
      int k;
      switch (tok2.m_index) {
        case Sym.LABEL:
        case Sym.LEFT:
          k = parseBaseAddress(tok2);
          instructionMemory.add(new Instruction(Inte.LDAP, j, tok.m_line));
          instructionMemory.add(new Instruction(Inte.LDBP, k, tok.m_line));
          instructionMemory.add(new Instruction(Parser.getCommand3Args(s), 0, tok.m_line));
          instructionMemory.add(new Instruction(Inte.STCP, i, tok.m_line));
          if (k == regPos - 1)
            regPos--;
          else
            System.out.println("k != (Parser.regPos - 1)");
          if (j == regPos - 1)
            regPos--;
          else
            System.out.println("j != (Parser.regPos - 1)");
          if (i == regPos - 1)
            regPos--;
          else
            System.out.println("i != (Parser.regPos - 1)");
          break;
        case Sym.SYMBNO:
        case Sym.NO:
          k = getImmediate(tok2);
          instructionMemory.add(new Instruction(Inte.LDAP, j, tok.m_line));
          instructionMemory.add(new Instruction(Inte.LDBI, k, tok.m_line));
          instructionMemory.add(new Instruction(Parser.getCommand3Args(s), 0, tok.m_line));
          instructionMemory.add(new Instruction(Inte.STCP, i, tok.m_line));
          if (j == regPos - 1)
            regPos--;
          else
            System.out.println("j != (Parser.regPos - 1)");
          if (i == regPos - 1)
            regPos--;
          else
            System.out.println("i != (Parser.regPos - 1)");
          break;
        default:
          AbstractParser.parse_error("Unknown operand type " + tok2.m_text +
                                     " at line " + tok.m_line);
      }
    }
    return true;
  }

  private void parseEnd() throws ParseException, IOException
  {
    Yytoken tok = y.yylex();
    switch (tok.m_index) {
      case Sym.LABELP:
        if (instructionMemory.next() > 0) {
          Symbol s = st.getSymbol(tok.m_text);
          run.pc = s.where;
        }
        else {
          AbstractParser.parse_error("How could one jump to a label (" + tok.m_text +
                                     " within an empty file? ");
        }
        break;
      case Sym.LABEL:
        AbstractParser.parse_error("How could one jump to a data label ("
                                   + tok.m_text + ")?");
        break;
      case Sym.NO:
        AbstractParser.parse_error("How could one jump to a numeric constant (" + tok.m_text +
                                   ")?");
        break;
      case Sym.SYMBU:
        AbstractParser.parse_error("How could one jump to a nonexisting label (" +
                                   tok.m_text + ")?");
        break;
      case Sym.SYMBNO:
        AbstractParser.parse_error("How could one jump to a symbolic constant (" +
                                   tok.m_text + ")?");
        break;
      case Sym.END:
      case Sym.RET:
      case Sym.CMD:
      case Sym.JCMD:
      case Sym.STOP:
        AbstractParser.parse_error("How could start execution at (" + tok.m_text + ")?");
        break;
    }
  }

  private boolean parseJump(Yytoken tok) throws ParseException, IOException
  {
    Yytoken tok2;
    int i;

    Symbol s = st.getSymbol(tok.m_text);
    switch (s.where) {
      case Internals.JMP:
      case Internals.CALL:
        /* These InstructionMemory have one operand */
        tok2 = y.yylex();
        Symbol t = st.getSymbol(tok2.m_text);
        switch (tok2.m_index) {
          case Sym.LABELP:
            instructionMemory.add(new Instruction(Parser.getJump1Arg(s), t.where, tok.m_line));
            break;
          case Sym.SYMBU:
            t.updates.add(instructionMemory.next());
            instructionMemory.add(new Instruction(Parser.getJump1Arg(s), t.where, tok.m_line));
            break;
          default:
            AbstractParser.parse_error("Unknown operand type " + tok2.m_text +
                                       " at line " + tok.m_line);
        }
        return true;
      default:
        /* First argument MUST be a real address */
        i = parseAddress();
        getComma();

        /* Second argument can be either ... */
        tok2 = y.yylex();
        int j;
        switch (tok2.m_index) {
          case Sym.LABEL:
          case Sym.LEFT:
            /* ... a real address */
            j = parseBaseAddress(tok2);
            instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
            instructionMemory.add(new Instruction(Inte.LDBP, j, tok.m_line));
            if (j == regPos - 1)
              regPos--;
            else
              System.out.println("j != (Parser.regPos - 1)");
            break;
          case Sym.SYMBNO:
          case Sym.NO:
            /* ... or an immediate number */
            j = getImmediate(tok2);
            instructionMemory.add(new Instruction(Inte.LDAP, i, tok.m_line));
            instructionMemory.add(new Instruction(Inte.LDBI, j, tok.m_line));
            break;
          default:
            AbstractParser.parse_error("Unknown second operand " + tok2.m_text +
                                       " at line " + tok.m_line);
        }
        if (i == regPos - 1)
          regPos--;
        else
          System.out.println("i != (Parser.regPos - 1)");
        getComma();

        /* Now we add the correct label to jump to */
        tok2 = y.yylex();
        switch (tok2.m_index) {
          case Sym.LABELP:
            t = st.getSymbol(tok2.m_text);
            instructionMemory.add(new Instruction(Parser.getJump3Args(s), t.where, tok.m_line));
            break;
          case Sym.SYMBU:
            t = st.getSymbol(tok2.m_text);
            t.updates.add(instructionMemory.next());
            instructionMemory.add(new Instruction(Parser.getJump3Args(s), 0, tok.m_line));
            break;
          case Sym.LABEL:
            AbstractParser.parse_error("Trying to branch to a variable (" + tok2.m_text +
                                       ") at line " + tok.m_line);
            break;
          case Sym.SYMBNO:
            AbstractParser.parse_error("Trying to branch to a symbolic number (" +
                                       tok2.m_text + ") at line " + tok.m_line);
            break;
          case Sym.NO:
            AbstractParser.parse_error("Trying to branch to a number (" +
                                       tok2.m_text + ") at line " + tok.m_line);
            break;
          default:
            AbstractParser.parse_error("Unknown third operand " + tok2.m_text +
                                       " at line " + tok.m_line);
        }
        break;
    }

    return true;
  }

  private boolean parseNextLine(Yytoken tok) throws ParseException, IOException
  {
    Yytoken tok2;
    Symbol s;
    // System.out.println("Parser.parseNextLine("+tok+")");
    switch (tok.m_index) {
      case Sym.SYMBU:
        tok2 = y.yylex();
        s = st.getSymbol(tok.m_text);
        Yytoken tok3;
        switch (tok2.m_index) {
          case Sym.CONST:
            tok3 = y.yylex();
            if (tok3.m_index != Sym.NO) {
              AbstractParser.parse_error("Bad const initialization at line " + tok3.m_line);
              return false;
            }
            s.kind = Sym.SYMBNO;
            s.where = Integer.parseInt(tok3.m_text);
            break;
          case Sym.STACK:
            tok3 = y.yylex();
            s.kind = Inte.LABEL;
            s.where = memPos++;
            if (tok3.m_index != Sym.NO) {
              AbstractParser.parse_error("Bad stack size at line " + tok3.m_line);
              return false;
            }
            memPos += Integer.parseInt(tok3.m_text);
            mem.setValue(s.where, s.where);
            break;
          case Sym.MEM:
            tok3 = y.yylex();
            s.kind = Inte.LABEL;
            s.where = memPos++;
            switch (tok3.m_index) {
              case Sym.NO:
                mem.setValue(s.where, Integer.parseInt(tok3.m_text));
                break;
              case Sym.LEFT:
                tok3 = y.yylex();
                if (tok3.m_index == Sym.LABEL) {
                  Symbol pos = st.getSymbol(tok3.m_text);
                  mem.setValue(s.where, pos.where);
                  Yytoken tok4 = y.yylex();
                  if (tok4.m_index != Sym.RIGHT) {
                    AbstractParser.parse_error("No closing parenthesis after " + tok3.m_text +
                                               " at line " + tok3.m_line);
                    return false;
                  }
                }
                else {
                  AbstractParser.parse_error("Bad Mem reference " + tok3.m_text +
                                             " at line " + tok3.m_line);
                  return false;
                }
                break;
              default:
                return parseNextLine(tok3);
            }
            break;
          case Sym.CMD:
          case Sym.JCMD:
          case Sym.END:
          case Sym.RET:
          case Sym.STOP:
            s.kind = Sym.LABELP;
            s.where = instructionMemory.next();
            for (int i = 0; i < s.updates.size(); i++) {
              int next = s.updates.get(i);
              try {
                instructionMemory.getInstr(next).data = s.where;
              } catch (Exception e) {
                AbstractParser.parse_error("Failed to find instruction " + i +
                                           " to update at line " + tok.m_line);
                return false;
              }
            }
            s.updates = new ArrayList<Integer>();
            switch (tok2.m_index) {
              case Sym.CMD:
                return parseCommand(tok2);
              case Sym.JCMD:
                return parseJump(tok2);
              case Sym.END:
                parseEnd();
                return false;
              case Sym.RET:
                instructionMemory.add(new Instruction(Inte.RET, 0, tok.m_line));
                return true;
              case Sym.STOP:
                instructionMemory.add(new Instruction(Inte.STOP, 0, tok.m_line));
                return true;
            }
            break;
          default:
            AbstractParser.parse_error("Unknown instruction " + tok2.m_text +
                                       " at line " + tok2.m_line);
            return false;
        }
        break;
      case Sym.RET:
        instructionMemory.add(new Instruction(Inte.RET, 0, tok.m_line));
        return true;
      case Sym.STOP:
        instructionMemory.add(new Instruction(Inte.STOP, 0, tok.m_line));
        return true;
      case Sym.END:
        parseEnd();
        return false;
      case Sym.CMD:
        return parseCommand(tok);
      case Sym.JCMD:
        return parseJump(tok);
      case Sym.LABEL:
      case Sym.LABELP:
        AbstractParser.parse_error("Double definition of symbol '" + tok.m_text +
                                   "' at line " + tok.m_line);
        return false;
      case Sym.SYMBNO:
        AbstractParser.parse_error("Cannot redefine the numeric '" + tok.m_text +
                                   "' at line " + tok.m_line);
        return false;
      default:
        AbstractParser.parse_error("Unknown instruction " + tok.m_text +
                                   " at line " + tok.m_line);
        return false;
    }
    return true;
  }

  private int parseSimpleAddress(Yytoken tok) throws ParseException
  {
    switch (tok.m_index) {
      case Sym.LABEL:
        Symbol s = st.getSymbol(tok.m_text);
        instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
        instructionMemory.add(new Instruction(Inte.STAR, regPos, tok.m_line));
        return regPos++;
      default:
        AbstractParser.parse_error("Bad address '" + tok.m_text + "' at line " + tok.m_line);
    }
    return 0;
  }
}

