package aoc3.dark.architecture.stack;

import aoc3.dark.architecture.*;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

public final class Parser extends AbstractParser
{
  private static boolean OK = true;

  private static int memPos;
  private final Yylex y;

  /**
   * constructor for string parsing
   *
   * @param mem
   * @param source
   */
  public Parser(String source, Mem mem)
  {
    this(new Yylex(new StringReader(source)), mem);
  }

  /**
   * main constructor
   *
   * @param mem
   * @param lex
   */
  private Parser(Yylex lex, Mem mem)
  {
    super(mem);
    memPos = 0;
    OK = true;

    y = lex;

    st.empty();
    st.addSymbol(new Symbol("pull", Sym.SPEC, Sym.PULL));
    st.addSymbol(new Symbol("popa", Sym.SPEC, Sym.POPA));
    st.addSymbol(new Symbol("drop", Sym.SPEC, Sym.DROP));
    st.addSymbol(new Symbol("dup", Sym.SPEC, Sym.DUP));
    st.addSymbol(new Symbol("swap", Sym.ATTR0, Sym.SWAP));
    st.addSymbol(new Symbol("rot", Sym.ATTR0, Sym.ROT));
    st.addSymbol(new Symbol("add", Sym.EXE, Sym.ADD));
    st.addSymbol(new Symbol("sub", Sym.EXE, Sym.SUB));
    st.addSymbol(new Symbol("mul", Sym.EXE, Sym.MUL));
    st.addSymbol(new Symbol("div", Sym.EXE, Sym.DIV));
    st.addSymbol(new Symbol("mod", Sym.EXE, Sym.MOD));
    st.addSymbol(new Symbol("and", Sym.EXE, Sym.AND));
    st.addSymbol(new Symbol("or", Sym.EXE, Sym.OR));
    st.addSymbol(new Symbol("xor", Sym.EXE, Sym.XOR));
    st.addSymbol(new Symbol("not", Sym.ATTR0, Sym.NOT));
    st.addSymbol(new Symbol("band", Sym.EXE, Sym.BAND));
    st.addSymbol(new Symbol("bor", Sym.EXE, Sym.BOR));
    st.addSymbol(new Symbol("bxor", Sym.EXE, Sym.BXOR));
    st.addSymbol(new Symbol("bnot", Sym.ATTR0, Sym.BNOT));
    st.addSymbol(new Symbol("eq", Sym.EXE, Sym.EQ));
    st.addSymbol(new Symbol("ne", Sym.EXE, Sym.NE));
    st.addSymbol(new Symbol("lt", Sym.EXE, Sym.LT));
    st.addSymbol(new Symbol("le", Sym.EXE, Sym.LE));
    st.addSymbol(new Symbol("gt", Sym.EXE, Sym.GT));
    st.addSymbol(new Symbol("ge", Sym.EXE, Sym.GE));
    st.addSymbol(new Symbol("ret", Sym.ATTR0, Sym.RET));
    st.addSymbol(new Symbol("push", Sym.ATTR1, Sym.PUSH));
    st.addSymbol(new Symbol("pusha", Sym.ATTR1, Sym.PUSHA));
    st.addSymbol(new Symbol("pop", Sym.ATTR1, Sym.POP));
    st.addSymbol(new Symbol("data", Sym.DATA, Sym.DATA));
    st.addSymbol(new Symbol("area", Sym.AREA, Sym.AREA));
    st.addSymbol(new Symbol("stop", Sym.ATTR0, Sym.STOP));
    st.addSymbol(new Symbol("end", Sym.ATTR1, Sym.END));
    st.addSymbol(new Symbol("jmp", Sym.JUMPS, Sym.JMP));
    st.addSymbol(new Symbol("jtrue", Sym.JUMPS, Sym.JTRUE));
    st.addSymbol(new Symbol("jfalse", Sym.JUMPS, Sym.JFALSE));
    st.addSymbol(new Symbol("call", Sym.JUMPS, Sym.CALL));
  }

  public boolean parse() throws ParseException, IOException
  {
    Yytoken tok;
    boolean done = false;

    if (!OK)
      return false;

    while (!done && (tok = y.yylex()).m_index != 0) {
      Symbol s;
      Yytoken tok2;
      switch (tok.m_index) {
        case Sym.DATA:
          tok2 = y.yylex();
          if (tok2.m_index == Sym.NO) {
            Yytoken tok3 = y.yylex();
            if (tok3.m_index == Sym.SYMBOL) {
              s = SymbolTable.getInstance().getSymbol(tok3.m_text);
              s.kind = Inte.LABEL;
              s.where = memPos++;
              mem.setValue(s.where, Integer.parseInt(tok2.m_text));
            }
            else {
              AbstractParser.parse_error("Bad data initialization at line " + tok3.m_line);
              return false;
            }
          }
          else if (tok2.m_index == Sym.SYMBOL) {
            s = SymbolTable.getInstance().getSymbol(tok2.m_text);
            s.kind = Inte.LABEL;
            s.where = memPos++;
          }
          else {
            AbstractParser.parse_error("Bad data initialization at line " + tok2.m_line);
            return false;
          }
          break;
        case Sym.AREA:
          tok2 = y.yylex();
          if (tok2.m_index == Sym.NO) {
            memPos += Integer.parseInt(tok2.m_text);
          }
          else {
            AbstractParser.parse_error("No length of area given at line " + tok2.m_line);
            return false;
          }
          break;
        case Sym.ATTR0:
          switch (tok.m_subtype) {
            case Sym.NOT:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.NOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCP, Sym.STACK, tok.m_line));
              break;
            case Sym.BNOT:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.BNOT, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCP, Sym.STACK, tok.m_line));
              break;
            case Sym.SWAP:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDBP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STBP, Sym.STACK, tok.m_line));
              break;
            case Sym.ROT:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDBP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STCP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STBP, Sym.STACK, tok.m_line));
              break;
            case Sym.STOP:
              instructionMemory.add(new Instruction(Inte.STOP, 0, tok.m_line));
              break;
            case Sym.RET:
              instructionMemory.add(new Instruction(Inte.RET, 0, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.ATTR1:
          tok2 = y.yylex();
          s = SymbolTable.getInstance().getSymbol(tok2.m_text);
          switch (tok.m_subtype) {
            case Sym.PUSH:
              if (tok2.m_index == Sym.NO) {
                instructionMemory.add(new Instruction(Inte.LDAI,
                                                      Integer.parseInt(tok2.m_text), tok.m_line));
              }
              else if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for load, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              break;
            case Sym.PUSHA:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAI, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for load, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              break;
            case Sym.POP:
              if (tok2.m_index == Sym.LABEL) {
                instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
                instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
                instructionMemory.add(new Instruction(Inte.STAM, s.where, tok.m_line));
              }
              else {
                AbstractParser.parse_error("Unknown data type for load, " + tok2.m_text +
                                           " at line " + tok2.m_line);
                return false;
              }
              break;
            case Sym.END:
              if (tok2.m_index == Sym.LABELP) {
                run.pc = s.where;
              }
              done = true;
              break;
            default:
              AbstractParser.parse_error("Unknown instruction given, " + tok.m_text +
                                         " at line " + tok.m_line);
              break;
          }
          break;
        case Sym.EXE:
          instructionMemory.add(new Instruction(Inte.LDBP, Sym.STACK, tok.m_line));
          instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
          instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
          switch (tok.m_subtype) {
            case Sym.ADD:
              instructionMemory.add(new Instruction(Inte.ADD, 0, tok.m_line));
              break;
            case Sym.SUB:
              instructionMemory.add(new Instruction(Inte.SUB, 0, tok.m_line));
              break;
            case Sym.MUL:
              instructionMemory.add(new Instruction(Inte.MUL, 0, tok.m_line));
              break;
            case Sym.DIV:
              instructionMemory.add(new Instruction(Inte.DIV, 0, tok.m_line));
              break;
            case Sym.MOD:
              instructionMemory.add(new Instruction(Inte.MOD, 0, tok.m_line));
              break;
            case Sym.AND:
              instructionMemory.add(new Instruction(Inte.AND, 0, tok.m_line));
              break;
            case Sym.OR:
              instructionMemory.add(new Instruction(Inte.OR, 0, tok.m_line));
              break;
            case Sym.XOR:
              instructionMemory.add(new Instruction(Inte.XOR, 0, tok.m_line));
              break;
            case Sym.BAND:
              instructionMemory.add(new Instruction(Inte.BAND, 0, tok.m_line));
              break;
            case Sym.BOR:
              instructionMemory.add(new Instruction(Inte.BOR, 0, tok.m_line));
              break;
            case Sym.BXOR:
              instructionMemory.add(new Instruction(Inte.BXOR, 0, tok.m_line));
              break;
            case Sym.EQ:
              instructionMemory.add(new Instruction(Inte.JEQ,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            case Sym.NE:
              instructionMemory.add(new Instruction(Inte.JNE,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            case Sym.LT:
              instructionMemory.add(new Instruction(Inte.JLT,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            case Sym.LE:
              instructionMemory.add(new Instruction(Inte.JLE,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            case Sym.GT:
              instructionMemory.add(new Instruction(Inte.JGT,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            case Sym.GE:
              instructionMemory.add(new Instruction(Inte.JGE,
                                                    instructionMemory.next() + 3,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.JMP,
                                                    instructionMemory.next() + 2,
                                                    tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDCI, 1, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Erroneous instruction " +
                                         tok.m_text + " at line " + tok.m_line);
              return false;
          }
          instructionMemory.add(new Instruction(Inte.STCP, Sym.STACK, tok.m_line));
          break;
        case Sym.SPEC:
          switch (tok.m_subtype) {
            case Sym.PULL:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAR, 1, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDAP, 1, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              break;
            case Sym.POPA:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAR, 1, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, 1, tok.m_line));
              break;
            case Sym.DROP:
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              break;
            case Sym.DUP:
              instructionMemory.add(new Instruction(Inte.LDAP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.DECR, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.STAP, Sym.STACK, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Erroneous instruction " +
                                         tok.m_text + " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.JUMPS:
          tok2 = y.yylex();
          s = SymbolTable.getInstance().getSymbol(tok2.m_text);
          switch (tok2.m_index) {
            case Sym.LABELP:
            case Sym.SYMBOL:
              break;
            default:
              AbstractParser.parse_error("Unknown jump label, " + tok2.m_text +
                                         " at line " + tok2.m_line);
              return false;
          }
          switch (tok.m_subtype) {
            case Sym.JMP:
              if (tok2.m_index == Sym.SYMBOL)
                s.updates.add(instructionMemory.next());
              instructionMemory.add(new Instruction(Inte.JMP, s.where, tok.m_line));
              break;
            case Sym.JTRUE:
              instructionMemory.add(new Instruction(Inte.LDAI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDBP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              if (tok2.m_index == Sym.SYMBOL)
                s.updates.add(instructionMemory.next());
              instructionMemory.add(new Instruction(Inte.JNE, s.where, tok.m_line));
              break;
            case Sym.JFALSE:
              instructionMemory.add(new Instruction(Inte.LDAI, 0, tok.m_line));
              instructionMemory.add(new Instruction(Inte.LDBP, Sym.STACK, tok.m_line));
              instructionMemory.add(new Instruction(Inte.INCR, Sym.STACK, tok.m_line));
              if (tok2.m_index == Sym.SYMBOL)
                s.updates.add(instructionMemory.next());
              instructionMemory.add(new Instruction(Inte.JEQ, s.where, tok.m_line));
              break;
            case Sym.CALL:
              if (tok2.m_index == Sym.SYMBOL)
                s.updates.add(instructionMemory.next());
              instructionMemory.add(new Instruction(Inte.CALL, s.where, tok.m_line));
              break;
            default:
              AbstractParser.parse_error("Unknown jump instruction, " + tok.m_text +
                                         " at line " + tok.m_line);
              return false;
          }
          break;
        case Sym.SYMBOL:
          s = SymbolTable.getInstance().getSymbol(tok.m_text);
          s.kind = Sym.LABELP;
          s.where = instructionMemory.next();
          for (int i = 0; i < s.updates.size(); i++) {
            int next = s.updates.get(i);
            try {
              instructionMemory.getInstr(next).data = s.where;
            } catch (Exception e) {
              AbstractParser.parse_error("Failed to find instruction " + i +
                                         " to update at line " + tok.m_line);
              return false;
            }
          }
          s.updates = new ArrayList<Integer>();
          break;
        case Sym.NO:
          AbstractParser.parse_error("Unused number " + tok.m_text +
                                     " at line " + tok.m_line);
          return false;
        case Sym.STR:
          AbstractParser.parse_error("Unused string '" + tok.m_text +
                                     "' at line " + tok.m_line);
          return false;
        default:
          break;
      }
    }
    return OK;
  }
}

