package aoc3.dark.validation;

import aoc3.dark.architecture.Architecture;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Erik Axel Nielsen <erikaxel@stud.ntnu.no
 * @version 26.aug.2004
 */
final class LoadTest extends Test
{
  LoadTest(Architecture arch, Element elem)
  {
    super(arch, elem);
  }

  private void checkMem(int addr, int val) throws ValidationException
  {
    if (getMemoryValue(addr) != val)
      throw new ValidationException(this, "Expected mem[" + addr + "] = " + val + " but got mem[" + addr + "] = " + getMemoryValue(addr));
  }

  private void checkReg(int reg, int val) throws ValidationException
  {
    if (regs[reg] != val)
      throw new ValidationException(this, "Expected reg[" + reg + "] = " + val + " but got reg[" + reg + "] = " + regs[reg]);
  }

  protected void checkValues() throws ValidationException
  {
    NodeList memTests = elem.getElementsByTagName("MemTest");
    for (int i = 0; i < memTests.getLength(); i++) {
      Element element = (Element)memTests.item(i);
      int val = Integer.parseInt(element.getAttribute("value"));
      int addr = Integer.parseInt(element.getAttribute("addr"));
      checkMem(addr, val);
    }
    NodeList regTests = elem.getElementsByTagName("RegTest");
    for (int i = 0; i < regTests.getLength(); i++) {
      Element element = (Element)regTests.item(i);
      int val = Integer.parseInt(element.getAttribute("value"));
      int reg = Integer.parseInt(element.getAttribute("reg"));
      checkReg(reg, val);
    }
  }

  private void setMem(int addr, int val)
  {
    setMemoryValue(addr, val);
  }

  private void setReg(int reg, int val)
  {
    regs[reg] = val;
  }

  protected void setUp()
  {
    regs = architecture.getRun().regs;
    NodeList memSets = elem.getElementsByTagName("MemSet");
    for (int i = 0; i < memSets.getLength(); i++) {
      Element element = (Element)memSets.item(i);
      int val = Integer.parseInt(element.getAttribute("value"));
      int addr = Integer.parseInt(element.getAttribute("addr"));
      setMem(addr, val);
    }
    NodeList regSets = elem.getElementsByTagName("RegSet");
    for (int i = 0; i < regSets.getLength(); i++) {
      Element element = (Element)regSets.item(i);
      int val = Integer.parseInt(element.getAttribute("value"));
      int reg = Integer.parseInt(element.getAttribute("reg"));
      setReg(reg, val);
    }
  }
}

