package aoc3.dark.validation;

import aoc3.dark.architecture.Architecture;
import org.w3c.dom.Element;

/**
 * @author Erik Axel Nielsen <erikaxel@stud.ntnu.no>
 * @version 09.jun.2004
 */
abstract class Test
{
  /* validation times out after
    VALIDATION_TIMEOUT_MS * VALIDATION_TIMOUT_COUNT
    milliseconds.
  */
  private static final int VALIDATION_TIMEOUT_MS = 100;
  private static final int VALIDATION_TIMEOUT_COUNT = 200;
  final Architecture architecture;
  final Element elem;
  int[] regs;

  protected Test(Architecture architecture, Element elem)
  {
    this.elem = elem;
    this.architecture = architecture;
  }

  public void validate() throws ValidationException
  {
    try {
      setUp();
      architecture.setShowErrors(false);
      architecture.execute(Architecture.STEPS_ALL);

      // Try to execute, and wait max 10 seconds
      for (int i = 0; i < VALIDATION_TIMEOUT_COUNT && !architecture.getRun().stop; i++) {
        try {
          Thread.sleep(VALIDATION_TIMEOUT_MS);
        } catch (InterruptedException e) {
          e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
      }

      // Do the actual validation if time limit is not exceeded.
      if (!architecture.getRun().stop)
        throw new ValidationException("Time limit exceeded ");

      checkValues();
    } catch (NumberFormatException e) {
      throw new ValidationException("NumberFormatException in xml");
    }
  }

  int getMemoryValue(int adress)
  {
    return architecture.getMem().getValue(adress);
  }

  void setMemoryValue(int address, int value)
  {
    architecture.getMem().setValue(address, value);
  }

  protected abstract void checkValues() throws ValidationException;

  protected abstract void setUp();
}

