package aoc3.db;

import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 08.mai.2005
 * Time: 13:31:41
 * To change this template use File | Settings | File Templates.
 */
public final class UserQuestion implements Serializable, Comparable
{
  private static transient PreparedStatement insertPost;
  private static transient PreparedStatement updatePost;
  private final Timestamp timestamp;
  private final int userID;
  private final int questionID;
  private Object params[];
  private Object answer;
  private boolean correct;
  private int timesAnswered;

  static void readPosts(Statement statement, Map<Integer, List<UserQuestion>> posts)
    throws SQLException, IOException, ClassNotFoundException
  {
    Connection connection = statement.getConnection();
    insertPost = connection.prepareStatement("INSERT INTO UserQuestion(userid, questionid, date, correct, " +
                                             "params, answer, timesanswered) VALUES (?,?,?,?,?,?,?)");
    updatePost = connection.prepareStatement("UPDATE UserQuestion SET correct = ?, params = ?, " +
                                             "answer = ?, timesanswered = ? WHERE userid = ? and " +
                                             "questionid = ? AND date = ?");

    ResultSet rs = statement.executeQuery("SELECT userid, questionid, date, correct, params, answer, " +
                                          "timesanswered FROM UserQuestion");
    while (rs.next()) {
      int userID = rs.getInt(1);
      int questionID = rs.getInt(2);
      Timestamp date = rs.getTimestamp(3);
      boolean correct = rs.getInt(4) == 1;
      ObjectInputStream ois = new ObjectInputStream(rs.getBlob(5).getBinaryStream());
      Object params[] = (Object[])ois.readObject();
      Object answer = rs.getObject(6);
      int timesAnswered = rs.getInt(7);

      UserQuestion post = new UserQuestion(userID, questionID, date, correct, params, answer, timesAnswered);
      List<UserQuestion> userQuestions = posts.get(userID);
      if (userQuestions == null) {
        userQuestions = new ArrayList<UserQuestion>();
        posts.put(userID, userQuestions);
      }
      userQuestions.add(post);
    }
  }

  UserQuestion(int userID, int questionID, Timestamp timestamp, boolean correct,
               Object[] params, Object answer, int timesAnswered)
  {
    this.userID = userID;
    this.questionID = questionID;
    this.timestamp = timestamp;
    this.correct = correct;
    this.params = params;
    this.answer = answer;
    this.timesAnswered = timesAnswered;
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(Object o)
  {
    UserQuestion q = (UserQuestion)o;
    return getQuestionID() - q.getQuestionID();
  }

// --------------------- End Interface Comparable ---------------------

  public String toString()
  {
    return String.valueOf(questionID);
  }

  public Object getAnswer()
  {
    return answer;
  }

  public Object[] getParams()
  {
    return params;
  }

  public int getQuestionID()
  {
    return questionID;
  }

  public int getTimesAnswered()
  {
    return timesAnswered;
  }

  public Timestamp getTimestamp()
  {
    return timestamp;
  }

  public boolean isCorrect()
  {
    return correct;
  }

  void makePostPersistent() throws SQLException, IOException
  {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    ObjectOutputStream oos = new ObjectOutputStream(baos);
    oos.writeObject(params);
    oos.flush();
    insertPost.setInt(1, userID);
    insertPost.setInt(2, questionID);
    insertPost.setTimestamp(3, timestamp);
    insertPost.setInt(4, correct ? 1 : 0);
    insertPost.setBytes(5, baos.toByteArray());
    insertPost.setObject(6, answer);
    insertPost.setInt(7, timesAnswered);
    insertPost.executeUpdate();
  }

  void makeUpdatesPersistent() throws SQLException, IOException
  {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    ObjectOutputStream oos = new ObjectOutputStream(baos);
    oos.writeObject(params);
    oos.flush();
    updatePost.setInt(1, correct ? 1 : 0);
    updatePost.setBytes(2, baos.toByteArray());
    updatePost.setObject(3, answer);
    updatePost.setInt(4, timesAnswered);
    updatePost.setInt(5, userID);
    updatePost.setInt(6, questionID);
    updatePost.setTimestamp(7, timestamp);
    updatePost.executeUpdate();
  }

  void update(boolean correct, Object[] params, Object answer, int timesAnswered)
  {
    this.correct = correct;
    this.params = params;
    this.answer = answer;
    this.timesAnswered = timesAnswered;
  }
}

