package aoc3.net;

import aoc3.client.Client;
import aoc3.db.*;
import aoc3.event.*;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.Socket;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 15:03:30
 * To change this template use File | Settings | File Templates.
 */
public final class ClientConnection extends SocketConnection implements ClientToServer
{
  private final List<AnswerEventListener> answerEventListeners = new ArrayList<AnswerEventListener>();
  private final List<HTMLEventListener> htmlEventListeners = new ArrayList<HTMLEventListener>();
  private final List<LevelEventListener> levelEventListeners = new ArrayList<LevelEventListener>();
  private final List<LoginEventListener> loginEventListeners = new ArrayList<LoginEventListener>();
  private final List<PositionChangeListener> positionChangeListeners = new ArrayList<PositionChangeListener>();
  private final List<QuestionBookListener> questionBookListeners = new ArrayList<QuestionBookListener>();
  private final List<QuestionEventListener> questionEventListeners = new ArrayList<QuestionEventListener>();
  private final List<TextBookListener> textBookListeners = new ArrayList<TextBookListener>();
  private final List<UserEventListener> userEventListeners = new ArrayList<UserEventListener>();
  private int seed;

  public ClientConnection() throws IOException
  {
    super(new Socket(Client.getServer(), Server.PORT));
  }

  public ClientConnection(String server) throws IOException
  {
    super(new Socket(server, Server.PORT));
  }

// --------------------- Begin Interface ClientToServer ---------------------

  public synchronized void requestLogin(String username, String password)
  {
    try {
      output.writeChar(C2S_LOGIN_REQUEST);
      output.writeObject(username);
      output.writeObject(SocketConnection.makePasskey(password, seed));
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public synchronized void logout()
  {
    try {
      output.writeChar(C2S_LOGOUT);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
    continueToListen = false;
  }

  public void addLoginListener(LoginEventListener listener)
  {
    loginEventListeners.add(listener);
  }

  public void removeLoginListener(LoginEventListener listener)
  {
    loginEventListeners.remove(listener);
  }

  public void addHTMLListener(HTMLEventListener listener)
  {
    htmlEventListeners.add(listener);
  }

  public void removeHTMLListener(HTMLEventListener listener)
  {
    htmlEventListeners.remove(listener);
  }

  public synchronized void notifyMove(int posx, int posy)
  {
    try {
      output.writeChar(C2S_MOVE_NOTIFY);
      output.writeInt(posx);
      output.writeInt(posy);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public void addUserListener(UserEventListener listener)
  {
    userEventListeners.add(listener);
  }

  public void removeUserListener(UserEventListener listener)
  {
    userEventListeners.remove(listener);
  }

  public void addPositionChangeListener(PositionChangeListener listener)
  {
    positionChangeListeners.add(listener);
  }

  public void removePositionChangeListener(PositionChangeListener listener)
  {
    positionChangeListeners.remove(listener);
  }

  public void addQuestionListener(QuestionEventListener listener)
  {
    questionEventListeners.add(listener);
  }

  public void removeQuestionListener(QuestionEventListener listener)
  {
    questionEventListeners.remove(listener);
  }

  public synchronized void submitStaticAnswer(int id, Object answer)
  {
    try {
      output.writeChar(C2S_ANSWER_STATIC_SUBMIT);
      output.writeInt(id);
      output.writeObject(answer);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public synchronized void submitAutogenAnswer(int id, String answer)
  {
    try {
      output.writeChar(C2S_ANSWER_AUTOGEN_SUBMIT);
      output.writeInt(id);
      output.writeObject(answer);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public synchronized void submitDarkAnswer(int id, String answer)
  {
    try {
      output.writeChar(C2S_ANSWER_DARK_SUBMIT);
      output.writeInt(id);
      output.writeObject(answer);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public void addAnswerEventListener(AnswerEventListener listener)
  {
    answerEventListeners.add(listener);
  }

  public void removeAnswerEventListener(AnswerEventListener listener)
  {
    answerEventListeners.remove(listener);
  }

  public synchronized void requestLevel(int id)
  {
    try {
      output.writeChar(C2S_LEVEL_REQUEST);
      output.writeInt(id);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public void addLevelListener(LevelEventListener listener)
  {
    levelEventListeners.add(listener);
  }

  public void removeLevelListener(LevelEventListener listener)
  {
    levelEventListeners.remove(listener);
  }

  public synchronized void requestBookText(int type)
  {
    try {
      output.writeChar(C2S_BOOK_TEXT_REQUEST);
      output.writeInt(type);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public synchronized void requestBookQuestion()
  {
    try {
      output.writeChar(C2S_BOOK_QUESTION_REQUEST);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  public void addTextBookListener(TextBookListener listener)
  {
    textBookListeners.add(listener);
  }

  public void removeTextBookListener(TextBookListener listener)
  {
    textBookListeners.remove(listener);
  }

  public void addQuestionBookListener(QuestionBookListener listener)
  {
    questionBookListeners.add(listener);
  }

  public void removeQuestionBookListener(QuestionBookListener listener)
  {
    questionBookListeners.remove(listener);
  }

// --------------------- End Interface ClientToServer ---------------------

  protected void notifyDisconnect()
  {
    for (LoginEventListener listener : loginEventListeners)
      listener.loggedOut();
  }

  protected void processCommand(char command, ObjectInputStream input)
    throws IOException, ClassNotFoundException
  {
    switch (command) {
      case P2P_CHAT:
        ChatEvent chatEvent = (ChatEvent)input.readObject();
        fireLocalChatEvent(chatEvent);
        break;
      case S2C_LOGIN_INIT:
        seed = input.readInt();
        List<Highscore> highScore = (List<Highscore>)input.readObject();
        List<Highscore> highScoreWeek = (List<Highscore>)input.readObject();
        for (LoginEventListener listener : loginEventListeners)
          listener.highScoreReceived(highScore, highScoreWeek);
        break;
      case S2C_LOGIN_RESPONSE:
        LoginEvent loginEvent = (LoginEvent)input.readObject();
        for (LoginEventListener listener : loginEventListeners)
          listener.loggedIn(loginEvent);
        break;
      case S2C_TEXT:
        Text text = (Text)input.readObject();
        for (HTMLEventListener listener : htmlEventListeners)
          listener.textReceived(text);
        break;
      case S2C_USERCHANGE:
        UserEvent userEvent = (UserEvent)input.readObject();
        for (UserEventListener listener1 : userEventListeners)
          listener1.userChanged(userEvent);
        break;
      case S2C_QUESTION_RESPONSE:
        QuestionEvent event = (QuestionEvent)input.readObject();
        for (QuestionEventListener listener : questionEventListeners)
          listener.questionReceived(event);
        break;
      case S2C_ANSWER_EVAL:
        boolean wasCorrect = input.readBoolean();
        boolean canRetry = input.readBoolean();
        int newScore = input.readInt();
        for (AnswerEventListener listener : answerEventListeners)
          listener.answerEvaluated(wasCorrect, canRetry, newScore);
        break;
      case S2C_LEVEL_RESPONSE:
        Level level = (Level)input.readObject();
        Map<Integer, List<LevelItem>> levelItems = (Map<Integer, List<LevelItem>>)input.readObject();
        for (LevelEventListener listener : levelEventListeners)
          listener.levelReceived(level, levelItems);
        break;
      case S2C_LEVEL_ITEM_REMOVED:
        LevelItem levelItem = (LevelItem)input.readObject();
        for (LevelEventListener listener : levelEventListeners)
          listener.levelItemRemoved(levelItem);
        break;
      case S2C_LEVEL_ITEM_CHANGED:
        levelItem = (LevelItem)input.readObject();
        int newAnimNr = input.readInt();
        for (LevelEventListener listener : levelEventListeners)
          listener.levelItemChanged(levelItem, newAnimNr);
        break;
      case S2C_MOVE_USER:
        int userID = input.readInt();
        int posx = input.readInt();
        int posy = input.readInt();
        for (PositionChangeListener listener : positionChangeListeners)
          listener.positionChanged(userID, posx, posy);
        break;
      case S2C_MOVE_LEVEL:
        userID = input.readInt();
        int levelID = input.readInt();
        posx = input.readInt();
        posy = input.readInt();
        int posz = input.readInt();
        for (PositionChangeListener listener : positionChangeListeners)
          listener.positionChanged(userID, levelID, posx, posy, posz);
        break;
      case S2C_BOOK_TEXT_RESPONSE:
        int type = input.readInt();
        List<Text> visitedTexts = (List<Text>)input.readObject();
        Map<Integer, TextGroup> textGroups = (Map<Integer, TextGroup>)input.readObject();
        for (TextBookListener listener : textBookListeners)
          listener.textsReceived(type, visitedTexts, textGroups);
        break;
      case S2C_BOOK_QUESTION_RESPONSE:
        Map<UserQuestion, Question> completedQuestions = (Map<UserQuestion, Question>)input.readObject();
        Map<Integer, QuestionCategory> questionCategories = (Map<Integer, QuestionCategory>)input.readObject();
        Map<Integer, QuestionGroup> questionGroups = (Map<Integer, QuestionGroup>)input.readObject();
        Map<UserQuestion, String> autogenQuestionTexts = (Map<UserQuestion, String>)input.readObject();
        Map<UserQuestion, String> autogenFormulas = (Map<UserQuestion, String>)input.readObject();
        Map<UserQuestion, String> darkSuggestedSolution = (Map<UserQuestion, String>)input.readObject();

        for (QuestionBookListener listener : questionBookListeners)
          listener.categoriesAndGroupsReceived(questionCategories, questionGroups);
        for (QuestionBookListener listener : questionBookListeners)
          listener.questionsReceived(completedQuestions, autogenQuestionTexts, autogenFormulas, darkSuggestedSolution);
        break;
      case S2C_KICK:
        continueToListen = false;
        break;
      default:
        System.out.println("Unknown command: " + command);
    }
  }
}

