package aoc3.net;

import aoc3.db.*;
import aoc3.event.*;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.Socket;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 15:03:20
 * To change this template use File | Settings | File Templates.
 */
final class ServerConnection extends SocketConnection implements ServerToClient
{
  private final Server server;
  private final int seed;
  private ServerUser serverUser;

  ServerConnection(Server server, Socket socket, List<Highscore> highScore, List<Highscore> highScoreWeek)
    throws IOException
  {
    super(socket);
    this.server = server;
    Random random = new Random(System.currentTimeMillis());
    seed = random.nextInt(Integer.MAX_VALUE);
    try {
      output.writeChar(S2C_LOGIN_INIT);
      output.writeInt(seed);
      output.writeObject(highScore);
      output.writeObject(highScoreWeek);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

// --------------------- Begin Interface ServerToClient ---------------------

  public synchronized void fireLoginEvent(LoginEvent event) throws IOException
  {
    output.writeChar(S2C_LOGIN_RESPONSE);
    output.writeObject(event);
    output.flush();
  }

  public boolean passwordMatches(String actualPassword, byte[] suppliedPasskey)
  {
    return Arrays.equals(SocketConnection.makePasskey(actualPassword, seed), suppliedPasskey);
  }

  // Kalles for å kaste denne brukeren ut - brukes hvis samme bruker logger inn igjen
  public synchronized void disconnect()
  {
    try {
      output.writeChar(S2C_KICK);
      output.flush();
    } catch (IOException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    continueToListen = false;
    listenThread.interrupt();
  }

  public synchronized void fireTextEvent(Text text) throws IOException
  {
    output.writeChar(S2C_TEXT);
    output.writeObject(text);
    output.flush();
  }

  public synchronized void fireUserEvent(UserEvent event)
  {
    try {
      output.writeChar(S2C_USERCHANGE);
      output.writeObject(event);
      output.flush();
    } catch (IOException e) {
      continueToListen = false;
      listenThread.interrupt();
    }
  }

  public synchronized void firePositionChange(int userID, int posx, int posy)
  {
    try {
      output.writeChar(S2C_MOVE_USER);
      output.writeInt(userID);
      output.writeInt(posx);
      output.writeInt(posy);
      output.flush();
    } catch (IOException e) {
      continueToListen = false;
      listenThread.interrupt();
    }
  }

  public synchronized void fireLevelChange(int userID, int levelID, int posx, int posy, int posz)
  {
    try {
      output.writeChar(S2C_MOVE_LEVEL);
      output.writeInt(userID);
      output.writeInt(levelID);
      output.writeInt(posx);
      output.writeInt(posy);
      output.writeInt(posz);
      output.flush();
    } catch (IOException e) {
      continueToListen = false;
      listenThread.interrupt();
    }
  }

  public synchronized void fireQuestionEvent(QuestionEvent event) throws IOException
  {
    output.writeChar(S2C_QUESTION_RESPONSE);
    output.writeObject(event);
    output.flush();
  }

  public synchronized void fireAnswerEvent(boolean wasCorrect, boolean canRetry, int newScore) throws IOException
  {
    output.writeChar(S2C_ANSWER_EVAL);
    output.writeBoolean(wasCorrect);
    output.writeBoolean(canRetry);
    output.writeInt(newScore);
    output.flush();
  }

  public synchronized void fireLevelEvent(Level level, Map<Integer, List<LevelItem>> levelItems) throws IOException
  {
    output.writeChar(S2C_LEVEL_RESPONSE);
    output.writeObject(level);
    output.writeObject(levelItems);
    output.flush();
  }

  public synchronized void fireLevelEvent(LevelItem levelItem) throws IOException
  {
    output.writeChar(S2C_LEVEL_ITEM_REMOVED);
    output.writeObject(levelItem);
    output.flush();
  }

  public synchronized void fireLevelEvent(LevelItem levelItem, Integer newAnimNr) throws IOException
  {
    output.writeChar(S2C_LEVEL_ITEM_CHANGED);
    output.writeObject(levelItem);
    output.writeInt(newAnimNr);
    output.flush();
  }

  public synchronized void fireBookEvent(int type, List<Text> visitedTexts, Map<Integer, TextGroup> textGroups)
    throws IOException
  {
    output.writeChar(S2C_BOOK_TEXT_RESPONSE);
    output.writeInt(type);
    output.writeObject(visitedTexts);
    output.writeObject(textGroups);
    output.flush();
  }


  public synchronized void fireBookEvent(Map<UserQuestion, Question> completedQuestions,
                                         Map<Integer, QuestionCategory> questionCategories,
                                         Map<Integer, QuestionGroup> questionGroups,
                                         Map<UserQuestion, String> autogenQuestionTexts,
                                         Map<UserQuestion, String> autogenFormulas,
                                         Map<UserQuestion, String> darkSuggestedSolutions) throws IOException
  {
    output.writeChar(S2C_BOOK_QUESTION_RESPONSE);
    output.writeObject(completedQuestions);
    output.writeObject(questionCategories);
    output.writeObject(questionGroups);
    output.writeObject(autogenQuestionTexts);
    output.writeObject(autogenFormulas);
    output.writeObject(darkSuggestedSolutions);
    output.flush();
  }

// --------------------- End Interface ServerToClient ---------------------

  protected void notifyDisconnect()
  {
    server.broadcastDisconnect(serverUser);
  }

  protected void processCommand(char command, ObjectInputStream input)
    throws IOException, ClassNotFoundException
  {
    if (serverUser.getUser() == null && command != C2S_LOGIN_REQUEST && command != C2S_LOGOUT)
      return; // Ignorerer alle andre kommandoer til bruker er innlogget

    switch (command) {
      case P2P_CHAT:
        ChatEvent event = (ChatEvent)input.readObject();
        if (event.getMessage().startsWith("/"))
          serverUser.submitCommand(event);
        else
          fireLocalChatEvent(event);
        break;
      case C2S_LOGIN_REQUEST:
        serverUser.requestLogin((String)input.readObject(), (byte[])input.readObject());
        break;
      case C2S_ANSWER_STATIC_SUBMIT:
        serverUser.submitAnswerStatic(input.readInt(), input.readObject());
        break;
      case C2S_ANSWER_AUTOGEN_SUBMIT:
        serverUser.submitAnswerAutoGen(input.readInt(), (String)input.readObject());
        break;
      case C2S_ANSWER_DARK_SUBMIT:
        serverUser.submitAnswerDark(input.readInt(), (String)input.readObject());
        break;
      case C2S_LEVEL_REQUEST:
        serverUser.requestLevel(input.readInt());
        break;
      case C2S_MOVE_NOTIFY:
        serverUser.notifyMove(input.readInt(), input.readInt());
        break;
      case C2S_BOOK_TEXT_REQUEST:
        serverUser.requestBookText(input.readInt());
        break;
      case C2S_BOOK_QUESTION_REQUEST:
        serverUser.requestBookQuestion();
        break;
      case C2S_LOGOUT:
        continueToListen = false;
        break;
      default:
        System.out.println("Unknown command: " + command);
    }
  }

  void start(ServerUser serverUser)
  {
    this.serverUser = serverUser;
    startListening();
  }
}

