package aoc3.net;

import aoc3.event.ChatEvent;
import aoc3.event.ChatEventListener;

import java.io.*;
import java.net.Socket;
import java.net.SocketException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: Jon Olav Hauglid
 * Date: 10.apr.2005
 * Time: 15:22:20
 * To change this template use File | Settings | File Templates.
 */
abstract class SocketConnection implements Runnable
{
  private static MessageDigest md5;
  private static final Object LOCK = new Object();
  static final char P2P_CHAT = 'a';
  static final char S2C_LOGIN_INIT = 'b';
  static final char C2S_LOGIN_REQUEST = 'c';
  static final char S2C_LOGIN_RESPONSE = 'd';
  static final char S2C_TEXT = 'e';
  static final char S2C_USERCHANGE = 'f';
  static final char S2C_QUESTION_RESPONSE = 'g';
  static final char C2S_ANSWER_STATIC_SUBMIT = 'h';
  static final char C2S_ANSWER_AUTOGEN_SUBMIT = 'i';
  static final char C2S_ANSWER_DARK_SUBMIT = 'j';
  static final char S2C_ANSWER_EVAL = 'k';
  static final char C2S_LEVEL_REQUEST = 'l';
  static final char S2C_LEVEL_RESPONSE = 'm';
  static final char S2C_LEVEL_ITEM_REMOVED = 'n';
  static final char S2C_LEVEL_ITEM_CHANGED = 'o';
  static final char C2S_MOVE_NOTIFY = 'p';
  static final char S2C_MOVE_USER = 'q'; // Bare andre brukere
  static final char S2C_MOVE_LEVEL = 'r';
  static final char C2S_BOOK_TEXT_REQUEST = 's';
  static final char S2C_BOOK_TEXT_RESPONSE = 't';
  static final char C2S_BOOK_QUESTION_REQUEST = 'u';
  static final char S2C_BOOK_QUESTION_RESPONSE = 'v';
  static final char C2S_LOGOUT = 'w';
  static final char S2C_KICK = 'x';
  final ObjectOutputStream output;
  private final List<ChatEventListener> chatListeners = new ArrayList<ChatEventListener>();
  private final ObjectInputStream input;
  private final Socket socket;
  boolean continueToListen = true;
  Thread listenThread;

  static {
    try {
      md5 = MessageDigest.getInstance("MD5");
    } catch (NoSuchAlgorithmException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }

  static byte[] makePasskey(String password, int seed)
  {
    synchronized (LOCK) {
      password += seed;
      return md5.digest(password.getBytes());
    }
  }

  protected SocketConnection(Socket socket) throws IOException
  {
    this.socket = socket;
    socket.setReceiveBufferSize(32 * 1024);
    output = new ObjectOutputStream(new BufferedOutputStream(socket.getOutputStream()));
    output.flush();
    input = new ObjectInputStream(new BufferedInputStream(socket.getInputStream()));
  }

// --------------------- Begin Interface Runnable ---------------------

  public void run()
  {
    try {
      while (continueToListen) {
        char command = input.readChar();
//        System.out.println("Command: "  + command);
        processCommand(command, input);
      }
    } catch (EOFException e) {
      System.out.println("Connection lost");
    } catch (SocketException e) {
      System.out.println("Connection lost");
    } catch (IOException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }
    System.out.println("Client from " + socket + " disconnected");
    try {
      input.close();
      output.close();
      socket.close();
    } catch (IOException e1) {
    }
    notifyDisconnect();
  }

// --------------------- End Interface Runnable ---------------------

  public void addChatListener(ChatEventListener listener)
  {
    chatListeners.add(listener);
  }

  public synchronized void fireRemoteChatEvent(ChatEvent event)
  {
    try {
      output.writeChar(P2P_CHAT);
      output.writeObject(event);
      output.flush();
    } catch (IOException e) {
      continueToListen = false;
      listenThread.interrupt();
    }
  }

  public void removeChatListener(ChatEventListener listener)
  {
    chatListeners.remove(listener);
  }

  public void startListening()
  {
    listenThread = new Thread(this);
    listenThread.start();
  }

  void fireLocalChatEvent(ChatEvent event)
  {
    for (ChatEventListener listener : chatListeners)
      listener.chatLine(event);
  }

  protected abstract void notifyDisconnect();

  protected abstract void processCommand(char command, ObjectInputStream input)
    throws IOException, ClassNotFoundException;
}

