package com;

import gui.SQLTrace;

import java.sql.*;

/**
 * Communicates directly with the database server.
 * Receives queries to execute, and executes them on the database.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class DatabaseCommunicator {

    private static Connection connection;

    public static boolean connect(String database, String username, String password) {
        try {
            Class.forName("com.mysql.jdbc.Driver");

            connection = DriverManager.getConnection("jdbc:mysql://" + database, username, password);

        }
        catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
            return false;
        }
        catch (ClassNotFoundException e) {
            SQLTrace.printException(e.getMessage());
            return false;
        }

        return true;
    }

    public static boolean disconnect() {
        try {
            connection.close();
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
            return false;
        }
        return true;
    }

    public static ResultSet executeQuery(String query) {
        SQLTrace.println(query);
        try {
            Statement st = connection.createStatement();
            return st.executeQuery(query);
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
            return null;
        }
    }

    public static int executeAutoIncrementInsert(String query) {
        SQLTrace.println(query);
        try {
            Statement st = connection.createStatement();
            st.executeUpdate(query, Statement.RETURN_GENERATED_KEYS);
            ResultSet rs = st.getGeneratedKeys();
            rs.next();

            return rs.getInt(1);
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
            return -1;
        }
    }

    public static void executeInsert(String query) {
        SQLTrace.println(query);
        try {
            Statement st = connection.createStatement();
            st.executeUpdate(query);
            st.close();
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
        }
    }

    public static void executeUpdate(String query) {
        SQLTrace.println(query);
        try {
            Statement st = connection.createStatement();
            st.executeUpdate(query);
            st.close();
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
        }
    }

    public static boolean isConnected() {
        try {
            if (connection != null) {
                return !connection.isClosed();
            }
            else return false;
        } catch (SQLException e) {
            SQLTrace.printException(e.getMessage());
            return false;
        }
    }

}
