package gui;

import gui.listeners.MenuBarListener;
import sys.AoCAdmin;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Dialog showing the "about.html" file with clickable hyperlinks.
 *
 * @author Eivind Sorteberg - 31.jul.2006
 */
public class AboutDialog extends JDialog implements HyperlinkListener {
    private JEditorPane aboutPane;

    /**
     * Creates a dialog displaying the "about.html" file.
     *
     * @param owner The frame on which this dialog is displayed
     * @param menuBarListener The listener of the menubar
     */
    public AboutDialog(Frame owner, MenuBarListener menuBarListener) {
        super(owner, true);

        JPanel contentPane;
        JPanel buttonPanel;
        JButton closeButton;

        setUndecorated(true);

        contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());
        contentPane.setBorder(BorderFactory.createRaisedBevelBorder());

        String aboutFileName = "aocAdmin/doc/about.html";


        URL url;

        url = AoCAdmin.class.getClassLoader().getResource(aboutFileName);

        if (url == null) {
            try {
                url = new File(aboutFileName).toURL();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }

        try {
            aboutPane = new JEditorPane(url);
        } catch (IOException e) {
            aboutPane = new JEditorPane("text/html", "<br><br><br><br><br><center>Could not find <b>" + aboutFileName + "</b>.</center><br>");
            e.printStackTrace();
        }

        aboutPane.setEditable(false);
        aboutPane.addHyperlinkListener(this);
        contentPane.add(aboutPane, BorderLayout.CENTER);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        closeButton = new JButton("Close");
        closeButton.setActionCommand("CLOSE_ABOUT");
        closeButton.addActionListener(menuBarListener);
        buttonPanel.add(closeButton);
        buttonPanel.add(Box.createHorizontalGlue());
        contentPane.add(buttonPanel, BorderLayout.SOUTH);
        this.setContentPane(contentPane);
        this.setSize(400, 300);
        this.setLocationRelativeTo(owner);
    }

    /**
     * Called whenever one of the links is entered, exited or clicked.
     *
     * @param e
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ENTERED) {
            aboutPane.setToolTipText("Adress: " + e.getURL().toString());
        }
        else if (e.getEventType() == HyperlinkEvent.EventType.EXITED) {
            aboutPane.setToolTipText(null);
        }
        else {
            URLHandler.openURL(e.getURL().toString());
        }
    }
}
