package gui;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;

/**
 * Panel showing either the AdministratorPanel, or the
 * editorPanel (TabPanel), depending on the view mode
 * selected by the user.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see AdminPanel
 * @see TabPanel
 */
public class Content extends JPanel {
    public static final int CONTENT_WIDTH = 800;
    public static final int CONTENT_HEIGHT = 600;

    private AdminPanel adminPanel;
    private TabPanel tabPanel;
    private JComponent activePanel;
    private MainFrame mainFrame;

    private ArrayList<String> status;

    public Content(MainFrame mainFrame) {
        this.mainFrame = mainFrame;
        this.setLayout(new BorderLayout());
        this.tabPanel = new TabPanel(this);
        this.adminPanel = new AdminPanel();
        reload();

        this.activePanel = tabPanel;
        add(activePanel);

    }

    public void showTabPanel() {
        if (activePanel != tabPanel) {
            switchPanel();
        }
    }
    public void showAdminPanel() {
        if (activePanel != adminPanel) {
            switchPanel();
        }
    }

    public void switchPanel() {
        this.remove(activePanel);

        if (this.activePanel.equals(tabPanel)) {
            activePanel = adminPanel;
        }
        else if (this.activePanel.equals(adminPanel)) {
            activePanel = tabPanel;
        }

        this.add(activePanel);
        validate();
        repaint();
    }

    public void setStatus(String status) {
        ArrayList<String> a = new ArrayList<String>();
        if (activePanel != null) a.add(activePanel.toString());
        if (status != null) a.add(status);
        this.status = a;
        if (mainFrame.getStatusBar() != null) mainFrame.getStatusBar().setStatus(a);
    }

    public ArrayList<String> getStatus() {
        return status;
    }

    public void setStatusBar(StatusBar statusbar) {
    }

    public void reload() {
        tabPanel.getModelManager().buildTreeModels();
        tabPanel.updateModels();
        adminPanel.setAdminModel(tabPanel.getModelManager().getAdminModel());
    }

    public TabPanel getTabPanel() {
        return tabPanel;
    }

    public AdminPanel getAdminPanel() {
        return adminPanel;
    }

    public JComponent getActivePanel() {
        return activePanel;
    }

}
