package gui;

import javax.swing.*;
import java.awt.*;

/**
 * The dialog diaplying the status when loading the database.
 */
public class DBProgressMonitor {
    private static boolean isInitialized = false;

    private static JDialog progressDialog;
    private static JLabel loadingLabel;
    private static JProgressBar progressBar;
    private static long start;


    private DBProgressMonitor(MainFrame mainFrame, String title, int maximum) {

        progressDialog = new JDialog(mainFrame, title);

        progressDialog.setUndecorated(true);

        JPanel progressContentPane;
        JLabel pleaseWaitLabel;
        Box[] boxArray;


        progressContentPane = new JPanel();
        progressContentPane.setBorder(BorderFactory.createRaisedBevelBorder());
        progressContentPane.setLayout(new BoxLayout(progressContentPane, BoxLayout.Y_AXIS));

        boxArray = new Box[3];
        boxArray[0] = new Box(BoxLayout.X_AXIS);
        pleaseWaitLabel = new JLabel("Please wait");
        pleaseWaitLabel.setFont(new Font("Monospaced", Font.BOLD, 17));
        boxArray[0].add(Box.createHorizontalGlue());
        boxArray[0].add(pleaseWaitLabel);
        boxArray[0].add(Box.createHorizontalGlue());
        boxArray[1] = new Box(BoxLayout.X_AXIS);
        loadingLabel = new JLabel("Reading data from database");
        loadingLabel.setFont(new Font("Monospaced", Font.BOLD, 15));
        boxArray[1].add(Box.createHorizontalGlue());
        boxArray[1].add(loadingLabel);
        boxArray[1].add(Box.createHorizontalGlue());
        boxArray[2] = new Box(BoxLayout.X_AXIS);
        progressBar = new JProgressBar();
        boxArray[2].add(Box.createHorizontalGlue());
        boxArray[2].add(progressBar);
        boxArray[2].add(Box.createHorizontalGlue());

        progressContentPane.add(Box.createVerticalGlue());
        progressContentPane.add(boxArray[0]);
        progressContentPane.add(boxArray[1]);
        progressContentPane.add(boxArray[2]);
        progressContentPane.add(Box.createVerticalGlue());
        progressDialog.setContentPane(progressContentPane);

        progressBar.setMaximum(maximum);

        progressDialog.pack();
        progressDialog.setSize(350, 100);
        progressDialog.setLocationRelativeTo(null);
    }


    public static void init(MainFrame mainFrame) {
        if (isInitialized) {
            return;
        }
        isInitialized = true;
        new DBProgressMonitor(mainFrame, "Reading DB...", 150);
    }


    public static void reset() {
        start = System.currentTimeMillis();
        update("Reading data from database", progressBar.getMinimum());
        progressDialog.getOwner().setEnabled(false);
        progressDialog.setVisible(true);
    }


    public static void update(String message, int value) {
        loadingLabel.setText(message);
        progressBar.setValue(value);
    }


    public static void complete() {
        double elapsed;

        elapsed = (double)(System.currentTimeMillis() - start);
        elapsed = elapsed / 1000;

        update("Done! (took " + elapsed + " secs)", progressBar.getMaximum());
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
        }
        progressDialog.getOwner().setEnabled(true);
        progressDialog.setVisible(false);
    }


}
