package gui;

import gui.listeners.MenuBarListener;
import sys.AoCAdmin;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Dialog showing the "help.html" file with clickable hyperlinks.
 *
 * @author Eivind Sorteberg - 31.jul.2006
 */
public class HelpDialog extends JDialog implements HyperlinkListener {
    private JEditorPane helpPane;

    /**
     * Creates a dialog displaying the "about.html" file.
     *
     * @param owner The frame on which this dialog is displayed
     * @param menuBarListener The listener of the menubar
     */
    public HelpDialog(Frame owner, MenuBarListener menuBarListener) {
        super(owner, false);

        JPanel contentPane;
        JPanel buttonPanel;
        JButton closeButton;

        setUndecorated(false);
        setResizable(true);

        contentPane = new JPanel();
        contentPane.setLayout(new BorderLayout());
        contentPane.setBorder(BorderFactory.createRaisedBevelBorder());

        String helpFileName = "aocAdmin/doc/manual.html";


        URL url;

        url = AoCAdmin.class.getClassLoader().getResource(helpFileName);

        if (url == null) {
            try {
                url = new File(helpFileName).toURL();
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }

        try {
            helpPane = new JEditorPane(url);
        } catch (IOException e) {
            helpPane = new JEditorPane("text/html", "<br><br><br><br><br><center>Could not find <b>" + helpFileName + "</b>.</center><br>");
            e.printStackTrace();
        }

        helpPane.setEditable(false);
        helpPane.addHyperlinkListener(this);

        JScrollPane scrollPane = new JScrollPane(helpPane);
        scrollPane.setPreferredSize(new Dimension(602, 600));

        contentPane.add(scrollPane, BorderLayout.CENTER);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
        buttonPanel.add(Box.createHorizontalGlue());
        closeButton = new JButton("Close");
        closeButton.setActionCommand("CLOSE_HELP");
        closeButton.addActionListener(menuBarListener);
        buttonPanel.add(closeButton);
        buttonPanel.add(Box.createHorizontalGlue());
        contentPane.add(buttonPanel, BorderLayout.SOUTH);
        this.setContentPane(contentPane);
        this.pack();
        this.setLocationRelativeTo(this);
    }

    /**
     * Called whenever one of the links is entered, exited or clicked.
     *
     * @param e
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            URL url = e.getURL();

            if (!url.getProtocol().equals("http")) {
                try {
                    helpPane.setPage(url);
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            else {
                URLHandler.openURL(url.toString());
            }
        }
    }
}
