package gui;

import gui.tabs.AbstractTab;
import gui.tabs.LevelTab;
import gui.tabs.QuestionTab;
import gui.tabs.TextTab;
import sys.datatypes.GroupElement;
import sys.datatypes.interfaces.AoCObject;
import sys.models.LevelModel;
import sys.models.ModelManager;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * The panel representing the AbstractEditor view of the application.
 * Contains the three different tabs (editors).
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see Content
 * @see AbstractTab
 */
public class TabPanel extends JTabbedPane implements ChangeListener {
    private TextTab textTab;
    private QuestionTab questionTab;
    private LevelTab levelTab;
    private AbstractTab activeTab;

    private ModelManager modelManager;
    private Content content;

    public TabPanel(Content content) {
        this.content = content;
        this.modelManager = new ModelManager(this);

        this.addChangeListener(this);

        textTab =        new TextTab();
        questionTab =    new QuestionTab();
        levelTab =       new LevelTab();

        this.addTab(levelTab.toString(), levelTab);
        this.addTab(questionTab.toString(), questionTab);
        this.addTab(textTab.toString(), textTab);
    }

    public void updateModels() {
        textTab.setModel(modelManager.getTextModel());
        questionTab.setModel(modelManager.getQuestionModel());
        levelTab.setModel(modelManager.getSuperLevelModel());
    }

    public void selectionChanged(AoCObject current, AoCObject selection) {
        if (selection instanceof GroupElement) {
            content.setStatus(((GroupElement)selection).getStatusName());
        }

        activeTab.updateObject(current);

        levelTab.showObject(selection);
        questionTab.showObject(selection);
        textTab.showObject(selection);
    }

    public void reloadModel(AbstractTreeModel model) {
        questionTab.setModel(model);
    }

    public ModelManager getModelManager() {
        return modelManager;
    }

    /**
     * Invoked when the target of the listener has changed its state.
     *
     * @param e a ChangeEvent object
     */
    public void stateChanged(ChangeEvent e) {
        if (activeTab != null) activeTab.saveButtonPressed();
        activeTab = (AbstractTab)getSelectedComponent();

        content.setStatus(null);
    }

    public void updateLevelModel(LevelModel levelModel) {
        levelTab.updateObject(levelModel);
        levelTab.updateLevelModel(levelModel);
    }
    
    public AbstractTab getActiveTab() {
        return activeTab;
    }

    public void undoChange(AoCObject original) {
        activeTab.showObject(original);
    }

    public LevelTab getLevelTab() {
        return levelTab;
    }

    public String toString() {
        return activeTab.toString();
    }

    public void superLevelModelChanged() {
        textTab.superLevelModelChanged();
        questionTab.superLevelModelChanged();
        levelTab.superLevelModelChanged();
    }
}