package gui.editors;

import sys.datatypes.interfaces.AoCObject;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;

/**
 * Superclass for the editor panels shown in the top right
 * corner in the editor view.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see gui.editors.level.LevelEditor
 * @see gui.editors.question.QuestionEditor
 * @see gui.editors.text.TextEditor
 */
public abstract class AbstractEditor extends JPanel {
    protected AbstractTreeModel model;

    public AbstractEditor() {
        super();
        EmptyBorder editorBorder = new EmptyBorder(5, 0, 0, 0);
        this.setBorder(editorBorder);
        this.setLayout(new GridLayout(1, 1));
    }

    /**
     * Shows the selected object's attributes in different
     * JComponents suitable for the attributes.
     *
     * @param selection The object to display
     */
    public abstract void showObject(AoCObject selection);

    /**
     * Updates the object with the information in the
     * JComponents. Used whenever selection changes, so that
     * the object's attributes needs to be updated, or when
     * the savbe button is clicked (or when the application is
     * closed).
     *
     * @param current The object to be updated
     */
    public abstract void updateObject(AoCObject current);

    /**
     * Updates the model representing the editor.
     *
     * @param model
     */
    public abstract void setModel(AbstractTreeModel model);

    /**
     * Called whenever the LevelModel has been changed (Level added or removed), and level lists
     * needs to be updated
     */
    public abstract void superLevelModelChanged();
}
