package gui.editors.common;

import gui.editors.common.selector.ParentSelector;
import sys.datatypes.Group;
import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Contains information for selected groups
 * in the text or question editor view.
 *
 * @author Eivind Sorteberg - 22.jun.2006
 * @see gui.editors.question.QuestionEditor
 * @see gui.editors.text.TextEditor
 */
public abstract class AbstractGroupPanel extends DPanel implements ActionListener {
    protected SelectorField parentGroup;
    protected JButton changeGroup;
    protected JTextField name;
    protected InputPane description;
    protected AbstractTreeModel model;

    /**
     * Constructor for AbstractGroupPanel; should be
     * called by all implementing subclasses
     */
    public AbstractGroupPanel() {
        super();

        JPanel info = new JPanel();

        createComponents();

        placeComponents(info);

        JPanel text = new JPanel(new GridLayout(1, 1));
        text.setBorder(BorderFactory.createTitledBorder("Description"));
        text.add(description);

        this.setLayout(new BorderLayout());
        this.add(info, BorderLayout.NORTH);
        this.add(text, BorderLayout.CENTER);
    }

    /**
     * Shows the information shown in the object.
     *
     * @param group The currently selected group.
     */
    public void showObject(Group group) {
        this.parentGroup.setText(group.getParent() != null ? group.getParent().toString() : "");
        this.name.setText(group.getName());
        this.description.setText(group.getDescription());
    }

    /**
     * Stores the information of the panel's components
     * into the object.
     *
     * @param group The object to be updated
     */
    public void saveObject(Group group) {
        group.setName(this.name.getText());
        group.setDescription(this.description.getText());
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(changeGroup)) {
            new ParentSelector(this, this.model);
        }
    }

    public void setModel(AbstractTreeModel model) {
        this.model = model;
    }

    public String getTitle() {
        return name.getText();
    }

    public void setEditable(boolean editable) {
        this.parentGroup.setEnabled(editable);
        this.changeGroup.setEnabled(editable);
        this.name.setEnabled(editable);
        this.description.setEnabled(editable);
    }

    /**
     * Creates the components common for all implementing subclasses.
     *
     * @see gui.editors.text.TextGroupPanel
     */
    protected void createComponents() {
        parentGroup = new SelectorField();
        name = new JTextField();
        description = new InputPane();

        changeGroup = new JButton("...");
        changeGroup.addActionListener(this);
    }

    /**
     * Places the components of the panel.
     *
     * @param parent The panel on which all components should be placed
     */
    protected abstract void placeComponents(JPanel parent);
}
