package gui.editors.common;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MultiFormat;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A panel containing radiobuttons for selecting whether input
 * is written in HTML or LaTeX, and a button for previewing the
 * typed in information.
 *
 * @author Eivind Sorteberg - 21.jun.2006
 * @see gui.editors.question.question.MultiChoicePanel
 * @see gui.editors.question.question.InputQuestionPanel
 */
public class FormatBar extends JPanel implements ActionListener {
    private JRadioButton html;
    private JRadioButton latex;
    private JRadioButton active;

    private JButton preview;
    private Previewable parent;

    public FormatBar(JRadioButton html, JRadioButton latex, Previewable parent) {
        this.html = html;
        this.latex = latex;

        this.html.setText("HTML");
        this.latex.setText("LaTeX");
        this.preview = new JButton("Preview");
        this.parent = parent;

        this.preview.addActionListener(this);
        this.html.addActionListener(this);
        this.latex.addActionListener(this);

        this.placeComponents(html, latex, preview);
    }

    /**
     * @deprecated
     */
    public FormatBar() {
        //Adding components to the north panel
        this.html = new JRadioButton("HTML");
        this.latex = new JRadioButton("LaTeX");
        this.preview = new JButton("Preview");
        this.placeComponents(html, latex, preview);
    }

    private void placeComponents(JRadioButton html, JRadioButton latex, JButton preview) {
        ButtonGroup group = new ButtonGroup();
        group.add(html);
        group.add(latex);

        this.html.setSelected(true);

        this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
        this.add(new TitleLabel("Format"));
        this.add(Box.createRigidArea(new Dimension(5, 0)));
        this.add(html);
        this.add(latex);
        this.add(Box.createHorizontalGlue());
        this.add(preview);
    }

    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source.equals(preview)) {
            new PreviewFrame(parent.getTitle(), parent.getText(), "html");
        }
        else if (source instanceof JRadioButton) {
            AoCObject currentObject = parent.getModel().getModelManager().getCurrentObject();

            if (currentObject instanceof MultiFormat) {
                MultiFormat object = (MultiFormat)currentObject;

                if (source.equals(latex) && !latex.equals(active)) {
                    object.setText(this.parent.getInputPane().getText());
                    this.parent.getInputPane().setText(object.getLatex());
                    this.preview.setEnabled(false);
                }
                else if (source.equals(html) && !html.equals(active)) {
                    object.setLatex(this.parent.getInputPane().getText());
                    this.parent.getInputPane().setText(object.getText());
                    this.preview.setEnabled(true);
                }
            }
            active = (JRadioButton)e.getSource();
        }
    }

    public void setEditable(boolean editable) {
        this.html.setEnabled(editable);
        this.latex.setEnabled(editable);
    }
}
