package gui.editors.common;

import com.nilo.plaf.nimrod.NimRODLookAndFeel;

import javax.swing.*;
import javax.swing.plaf.basic.BasicTextAreaUI;
import java.awt.*;

/**
 * TextArea used when information may have more than one, but still few
 * lines (e.g. TeleSrc texts). Shows two lines, and makes the area
 * look like the textfields defined by the <code>NimROD</code> L&F, if this
 * is the application's L&F.
 *
 * @author Eivind Sorteberg - 21.jul.2006
 */
public class InfoTextArea extends JScrollPane {
    public InfoTextArea(JTextArea textarea) {
        super(textarea);

        int height = (textarea.getFont().getSize() + 4) * 2;
        this.getViewport().setPreferredSize(new Dimension(this.getViewport().getPreferredSize().width, height));
        textarea.setWrapStyleWord(true);
        textarea.setLineWrap(true);

        if (UIManager.getLookAndFeel().getName().equals("NimROD")) {
            textarea.setUI(new TextAreaUI());
        }
    }

    /**
     * Gives the textarea the look defined in <code>NimROD</code> L&F.
     */
    private class TextAreaUI extends BasicTextAreaUI {
        public TextAreaUI() {
            super();
        }

        protected void paintBackground( Graphics g) {
            super.paintBackground(g);

            Component c = getComponent();

            GradientPaint grad = new GradientPaint( 0,0, NimRODLookAndFeel.sombra,
                                                    0,c.getHeight(), NimRODLookAndFeel.brillo);

            Graphics2D g2D = (Graphics2D)g;
            g2D.setPaint( grad);
            g2D.fillRect( 0,0, c.getWidth(),c.getHeight());
          }

    }
}
