package gui.editors.common;

import sys.AoCAdmin;
import sys.DefaultValues;

import javax.swing.*;
import java.awt.*;

/**
 * Opens a frame where the given text is displayed in the selected
 * format. (Only html is currently supported). Uses the stylesheet
 * defined in the database, if available.
 *
 * @author Eivind Sorteberg - 23.jun.2006
 */
public class PreviewFrame {

    public PreviewFrame(String header, String[] text, String format) {
        JFrame frame = new JFrame(header);

        if (format.equals("html")) {

            frame.setLayout(new BorderLayout());
            JPanel center = new JPanel(new GridLayout(1, 1));
            JPanel south = new JPanel(new GridLayout((text.length - 1), 1));

            for (int i = 0; i < text.length; i++) {

                JEditorPane pane = new JEditorPane("text/html", "");

                String s = parseText(text[i]);
                pane.setText(s);

                pane.setBackground(Color.WHITE);
                JScrollPane scrollPane = new JScrollPane(pane);
                pane.setEditable(false);

                if (i==0) {
                    if (header != null) pane.setText("<h1>" + header + "</h1>" + pane.getText());
                    center.add(scrollPane, BorderLayout.CENTER);
                }
                else {
                    JPanel p = new JPanel(new BorderLayout());
                    p.add(new JSeparator(), BorderLayout.NORTH);
                    p.add(scrollPane, BorderLayout.CENTER);
                    south.add(p);
                }
                pane.setCaretPosition(0);
            }

            frame.add(south, BorderLayout.SOUTH);
            frame.add(center, BorderLayout.CENTER);

            frame.setSize(new Dimension(700, 500));
            frame.setLocationRelativeTo(null);
            frame.setVisible(true);
        }
    }

    /**
     * Method used to make sure the text displayed in this frame is the same
     * as in the application. Alters the image addresses to absolute addresses,
     * and loads the stylesheet used in AoC3. This method does not alter the string
     * to be written to DB.
     *
     * @param s The string to display
     * @return Modified string.
     */
    private String parseText(String s) {
        s = s.replace("<img src=\"", "<img src=\"" + AoCAdmin.WEB_SERVER + AoCAdmin.DATA_FOLDER);
        s = s.replace("<img src='", "<img src=\"" + AoCAdmin.WEB_SERVER + AoCAdmin.DATA_FOLDER);

        // Adding style sheet, so that the text is displayed correctly
        s =  "<html>" +
             "  <head>" +
             "      <link rel='stylesheet' type='text/css' href='" + AoCAdmin.WEB_SERVER + AoCAdmin.SETTINGS_FOLDER + DefaultValues.STYLE_CSS_FILE + "' />" +
             "  </head>" +
             "  <body>" +
             "      " + s +
             "  </body>" +
             "</html>";

        return s;
    }
}
