package gui.editors.common.selector;

import sys.models.treemodels.AbstractTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * The popup used to add new objects to the tree. This popup uses a combobox
 * defined by the current model to display the different selections.
 *
 * @see gui.tree.AbstractTreePanel
 * @author Eivind Sorteberg - 27.jun.2006
 */
public class NewObjectSelector extends AbstractSelector implements ActionListener, ListCellRenderer {
    public NewObjectSelector(JPanel parent, AbstractTreeModel model) {
        super(parent, model);
    }

    public void actionPerformed(ActionEvent e) {
        JComboBox sel = (JComboBox)selector;
        if (e.getSource().equals(sel)) {
            if (sel.getSelectedItem() instanceof JSeparator) {
                sel.setSelectedIndex(sel.getSelectedIndex() + 1);
            }
        }
        else {
            if (e.getSource().equals(ok)) {
                if (sel.getSelectedItem() instanceof String) {
                    this.model.createObject((String)sel.getSelectedItem());
                }
                else System.err.println("Error! New class selector. Expected String, found: " + sel.getSelectedItem().getClass().getName());
            }
            else if (e.getSource().equals(cancel)) {

            }

            this.remove();
        }
    }

    /**
     * Returns the component representing each line (object) in the list. Each
     * line is represented by a <code>JLabel</code>, and the looks of this label
     * is defined by the object's current state.
     *
     * @param list The list in which the object belongs
     * @param value The value of the object
     * @param index The line number of the object
     * @param isSelected Whether the object is selected or not
     * @param cellHasFocus Whether the cell has focus or not
     * @return The component representing the object
     */
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        if (value instanceof JSeparator) {
            return (JSeparator)value;
        }
        else if (value instanceof String) {
            JLabel label = new JLabel((String)value);
            label.setOpaque(true);

            if (isSelected) {
                label.setBackground(Color.BLUE);
                label.setForeground(Color.WHITE);
            }
            return label;
        }
        else ((JComponent)value).setOpaque(true);
        return null;
    }


    /* Methods implemented from AbstractSelector */

    protected JComponent createSelector() {
        JComboBox selector = new JComboBox(this.model.getObjectNames());
        selector.setLightWeightPopupEnabled(false);
        selector.setRenderer(this);
        selector.addActionListener(this);
        return selector;
    }

    protected JComponent getSelectorComponent() {
        return selector;
    }

    protected String setTitle() {
        return "Select the kind of object you want to create";
    }
}
