package gui.editors.common.util;

import gui.editors.common.TitleLabel;

import javax.swing.*;
import java.awt.*;

/**
 * Places components in a grid of width 3, where the middle
 * column is resized when the window is resized.
 *
 * @author Eivind Sorteberg - 22.jun.2006
 */
public class InfoPanelGenerator {
    /**
     * Places the components given as parameter in a X by 3 grid.
     *
     * @param parent The component where the grid is applied
     * @param width The width of the grid (should be three)
     * @param height The height of the grid
     * @param components The components to be placed in the grid
     */
    public static void placeComponents(JPanel parent, int width, int height, Object[] components) {
        if (width * height != components.length) {
            return;
        }

        GridBagLayout l = new GridBagLayout();
        parent.setLayout(l);

        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;

        if (width == 3) {
            for (int i = 0; i < components.length; i+=3) {
                placeThreeRow(parent, new Object[]{components[i], components[i+1], components[i+2]}, l, c);
            }
        }

    }

    /**
     * Places one row, containing three components, where the middle
     * component is resizable.
     *
     * @param parent The parent panel
     * @param row The objects in the row
     * @param l The layout of the parent panel
     * @param c The constraints being used
     */
    private static void placeThreeRow(JPanel parent, Object[] row, GridBagLayout l, GridBagConstraints c) {
        JComponent comp;

        comp = getComponent(row[0]);
        c.gridwidth = 1;
        l.setConstraints(comp, c);
        parent.add(comp);

        comp = getComponent(row[1]);
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.weightx = 1.0;
        l.setConstraints(comp, c);
        parent.add(comp);

        comp = getComponent(row[2]);
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.weightx = 0;
        l.setConstraints(comp, c);
        parent.add(comp);
    }

    /**
     * Returns a legal JComponent where needed (if the object in question is
     * a String or null it is comverted to a JLabel or JPanel, respectively.
     *
     * @param component The object in question
     * @return A legal JComponent representing the object
     */
    private static JComponent getComponent(Object component) {
        if (component == null) {
            return new JPanel();
        }
        else if (component instanceof String) {
            JPanel panel = new JPanel(new BorderLayout());
            panel.add(new TitleLabel(component.toString()), BorderLayout.CENTER);
            return panel;
        }
        else if (component instanceof JComponent) {
            return (JComponent)component;
        }
        else {
            System.err.println("InfoPanelGenerator.getComponent: Illegal object type; found " + component.getClass().getName());
            return null;
        }
    }
}
