package gui.editors.level;

import gui.editors.common.DPanel;
import gui.editors.common.util.InfoPanelGenerator;
import sys.datatypes.interfaces.AoCObject;
import sys.models.LevelModel;
import sys.models.treemodels.LevelTreeModel;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

/**
 * Places the information textfiels for the
 * selected <code>Level</code>.
 *
 * @author Eivind Sorteberg - 23.jun.2006
 */
public class LevelInfo extends DPanel implements ActionListener {
    public static final String DEFAULT_BOX_VALUE = "<Choose a map>";

    private JTextField name;
    private JComboBox mapBox;
    private LevelTreeModel model;

    public LevelInfo() {
        this.name = new JTextField();
        this.mapBox = new JComboBox();
        this.mapBox.addActionListener(this);

        InfoPanelGenerator.placeComponents(this, 3, 2, new Object[]{"Name", name, null,
                                                                    "Map", mapBox, null});
    }

    public String getName() {
        return name.getText();
    }

    public String getMap() {
        return (String)mapBox.getSelectedItem();
    }

    public void showObject(LevelModel levelModel) {
        mapBox.removeActionListener(this);
        name.setText(levelModel.getLevelObject().getName());
        name.setCaretPosition(0);
        String mapFileName = levelModel.getLevelObject().getMap();
        if (mapFileName.equalsIgnoreCase("undefined")) {
            mapBox.setSelectedIndex(0);
        }
        else {
            mapBox.setSelectedItem(mapFileName);
        }
        mapBox.addActionListener(this);
    }

    public void modelChanged() {
        Object selection = mapBox.getSelectedItem();
        mapBox.removeActionListener(this);
        ArrayList<String> maps = model.getMapList();
        mapBox.removeAllItems();
        mapBox.addItem(DEFAULT_BOX_VALUE);
        for (String map : maps) {
            mapBox.addItem(map);
        }
        try {
            mapBox.setSelectedItem(selection);
        } catch (Exception e) {
        }
        mapBox.addActionListener(this);
    }

    /**
     * Invoked when an action occurs.
     */
    public void actionPerformed(ActionEvent e) {

        if (e.getSource().equals(mapBox)) {
            if (this.model != null) {
                AoCObject currentObject = model.getModelManager().getCurrentObject();

                if (currentObject instanceof LevelModel) {
                    LevelModel lm = (LevelModel)currentObject;
                    String selection = mapBox.getSelectedItem().toString();
                    String oldMap = lm.getMap();

                    if (selection.equals(DEFAULT_BOX_VALUE)) return;


                    if (!(selection.equals(oldMap))) {
                        lm.setMap(selection);
                        model.getModelManager().fireLevelMapChanged((LevelModel)currentObject);
                    }
                }
            }
        }
    }

    public void setModel(LevelTreeModel model) {
        this.model = model;
        this.modelChanged();
    }

    public void setEditable(boolean editable) {
        this.name.setEnabled(editable);
        this.mapBox.setEnabled(editable);
    }
}