package gui.editors.map;

import sys.AoCAdmin;

import javax.swing.*;
import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;


/**
 * User: Jon Olav Hauglid
 * Date: 14.feb.2004
 * Time: 20:32:01
 */
public final class Icons
{
  public static final int ICON_QUESTION = 8;
  public static final int ICON_FACT = 9;
  public static final int ICON_HISTORY = 10;
  public static final int ICON_MESSAGE = 11;
  public static final int ICON_TELEPORTDEST = 12;
  public static final int ICON_TELEPORTSOURCE = 13;
  public static final int ICON_QUESTION_GREEN = 14;
  public static final int ICON_TELEPORTSOURCE_STATIC = 15;
  private static final Map<String, ImageIcon> ICONMAP = new HashMap<String, ImageIcon>(30);
  private static final String animations[];
  private static final Rectangle frameSizes[];

  static {
    animations = new String[16];
    frameSizes = new Rectangle[16];
    animations[0] = "avatar-ball-yell.png";
    animations[1] = "avatar-ball-oran.png";
    animations[2] = "avatar-ball-red.png";
    animations[3] = "avatar-ball-purp.png";
    animations[4] = "avatar-ball-blue.png";
    animations[5] = "avatar-ball-turk.png";
    animations[6] = "avatar-ball-gree.png";
    animations[7] = "avatar-ball-grey.png";
    frameSizes[0] = new Rectangle(32, 32);
    frameSizes[1] = new Rectangle(32, 32);
    frameSizes[2] = new Rectangle(32, 32);
    frameSizes[3] = new Rectangle(32, 32);
    frameSizes[4] = new Rectangle(32, 32);
    frameSizes[5] = new Rectangle(32, 32);
    frameSizes[6] = new Rectangle(32, 32);
    frameSizes[7] = new Rectangle(32, 32);

    animations[ICON_QUESTION] = "question.png";
    frameSizes[ICON_QUESTION] = new Rectangle(32, 32);
    animations[ICON_FACT] = "fact.png";
    frameSizes[ICON_FACT] = new Rectangle(33, 40);
    animations[ICON_HISTORY] = "history.png";
    frameSizes[ICON_HISTORY] = new Rectangle(33, 40);
    animations[ICON_MESSAGE] = "message.png";
    frameSizes[ICON_MESSAGE] = new Rectangle(32, 32);
    animations[ICON_TELEPORTDEST] = "teleporteranim-target.png";
    frameSizes[ICON_TELEPORTDEST] = new Rectangle(32, 32);
    animations[ICON_TELEPORTSOURCE] = "teleporteranim.png";
    frameSizes[ICON_TELEPORTSOURCE] = new Rectangle(32, 32);
    animations[ICON_QUESTION_GREEN] = "question-green.png";
    frameSizes[ICON_QUESTION_GREEN] = new Rectangle(32, 32);
    animations[ICON_TELEPORTSOURCE_STATIC] = "teleporterstatic.png";
    frameSizes[ICON_TELEPORTSOURCE_STATIC] = new Rectangle(32, 32);
  }

  public static ImageIcon getAnimation(int animNr)
  {
    return getIcon(animations[animNr]);
  }

  public static Rectangle getFrameSize(int animNr)
  {
    return frameSizes[animNr];
  }

  public static ImageIcon getIcon(String name)
  {
    ImageIcon icon = ICONMAP.get(name);
    if (icon == null) {
      icon = getIconNoCache(name);
      ICONMAP.put(name, icon);
    }
    return icon;
  }

  public static ImageIcon getIconNoCache(String name)
  {
      try {
          ImageIcon ii = new ImageIcon(new URL(AoCAdmin.WEB_SERVER +
                                               AoCAdmin.ICON_FOLDER +
                                               name));
          return ii;
      } catch (MalformedURLException e) {
          System.err.println("Unable to load icon " + name);
          return null;
      }

  }

  private Icons()
  {
  }
}

