package gui.editors.question;

import gui.editors.AbstractEditor;
import gui.editors.common.DPanel;
import gui.editors.question.category.QuestionCategoryPanel;
import gui.editors.question.group.QuestionGroupPanel;
import gui.editors.question.question.QuestionObjectPanel;
import sys.datatypes.Group;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.question.Question;
import sys.datatypes.question.QuestionCategory;
import sys.datatypes.question.QuestionGroup;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.QuestionTreeModel;

/**
 * Specific panel for the question editor panel, that is
 * for editing <code>Question</code>, <code>QuestionGroup</code>
 * and <code>QuestionCategory</code> objects.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class QuestionEditor extends AbstractEditor {
    private QuestionObjectPanel     objectPanel;
    private QuestionCategoryPanel   categoryPanel;
    private QuestionGroupPanel      groupPanel;

    private DPanel activePanel;

    public QuestionEditor() {
        super();

        this.objectPanel =      new QuestionObjectPanel();
        this.categoryPanel =    new QuestionCategoryPanel();
        this.groupPanel =       new QuestionGroupPanel();
    }

    private void changePanel(DPanel panel) {
        this.activePanel = panel;

        this.removeAll();
        this.add(panel);
        this.validate();
        this.repaint();
    }

    public void showObject(AoCObject selection) {
        if (selection instanceof Question) {
            this.objectPanel.showObject((Question)selection);
            this.changePanel(objectPanel);
            activePanel.setEditable(true);
        }
        else if (selection instanceof QuestionCategory) {
            this.categoryPanel.showObject((QuestionCategory)selection);
            this.changePanel(categoryPanel);
            activePanel.setEditable(true);
        }
        else if (selection instanceof QuestionGroup) {
            this.groupPanel.showObject((Group)selection);
            this.changePanel(groupPanel);
            activePanel.setEditable(true);
        }
        else { //Do not show old info
            if (activePanel != null) {
                activePanel.setEditable(false);
            }
        }
    }

    public void updateObject(AoCObject current) {
        if (current instanceof Question) {
            this.objectPanel.saveObject((Question)current);
        }
        else if (current instanceof QuestionCategory) {
            this.categoryPanel.saveObject((QuestionCategory)current);
        }
        else if (current instanceof QuestionGroup) {
            this.groupPanel.saveObject((Group)current);
        }
    }

    public void setModel(AbstractTreeModel model) {
        this.objectPanel.setModel((QuestionTreeModel)model);
        this.categoryPanel.setModel(model);
        this.groupPanel.setModel(model);
    }

    public void superLevelModelChanged() {
        categoryPanel.modelChanged();
    }
}
