package gui.editors.question.category;

import gui.editors.common.*;
import gui.editors.common.selector.ParentSelector;
import gui.editors.common.util.InfoPanelGenerator;
import sys.datatypes.question.QuestionCategory;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Panel for displaying information about <code>QuestionCategory</code>
 * objects.
 *
 * @author Eivind Sorteberg - 20.jun.2006
 * @see sys.datatypes.question.QuestionCategory
 * @see gui.editors.question.QuestionEditor
 */
public class QuestionCategoryPanel extends InfoPanel implements Previewable, ActionListener {
    private JTextField name;
    private SelectorField group;
    private JButton changeGroup;
    private JTextField reqAnswers;
    private JTextField description;
    private int nofAnswersRequiredBackup;

    private JRadioButton html;
    private JRadioButton latex;
    private InputPane text;
    private FormatBar formatBar;


    public QuestionCategoryPanel() {
        super();

        this.name = new JTextField();
        this.group = new SelectorField();
        this.changeGroup = new JButton("...");
        changeGroup.addActionListener(this);
        this.reqAnswers = new JTextField();
        this.description = new JTextField();

        InfoPanelGenerator.placeComponents(infoPanel, 3, 4, new Object[]{
                "Group",            group,      changeGroup,
                "Name",             name,       null,
                "Req. #answers",    reqAnswers, null,
                "Comment",          description,    null});


        //Adding the input and format parts
        JPanel inputPanel = new JPanel(new BorderLayout());
        inputPanel.setBorder(BorderFactory.createTitledBorder("Text"));

        this.html = new JRadioButton("HTML");
        this.latex = new JRadioButton("LaTeX");

        formatBar = new FormatBar(html, latex, this);
        inputPanel.add(formatBar, BorderLayout.NORTH);

        this.text = new InputPane();
        inputPanel.add(new JScrollPane(text), BorderLayout.CENTER);

        PlacementPanel placement = new PlacementPanel(levelBox, xPos, yPos, zPos, changePositionButton);

        this.setLayout(new BorderLayout());

        this.add(infoPanel, BorderLayout.NORTH);
        this.add(inputPanel, BorderLayout.CENTER);
        this.add(placement, BorderLayout.SOUTH);
    }

    public String[] getText() {
        return new String[]{text.getText()};
    }

    public String getTitle() {
        return name.getText();
    }

    public QuestionTreeModel getModel() {
        return (QuestionTreeModel)model;
    }

    public void showObject(QuestionCategory source) {
        this.name.setText(source.getName());
        this.group.setText(source.getParent().getName());
        this.reqAnswers.setText(Integer.toString(source.getQuestionsRequired()));
        this.nofAnswersRequiredBackup = source.getQuestionsRequired();
        this.description.setText(source.getDescription());
        this.text.setText(this.html.isSelected() ? source.getText() : source.getLatex());

        super.showObject(source);
    }

    public void saveObject(QuestionCategory object) {
        object.setName(this.name.getText());
        object.setDescription(this.description.getText());

        if (html.isSelected())          object.setText(this.text.getText());
        else if (latex.isSelected())    object.setLatex(this.text.getText());


        int nofAnswersRequired;
        try {
            nofAnswersRequired = Integer.parseInt(this.reqAnswers.getText());
        } catch (NumberFormatException e) {

            JOptionPane.showMessageDialog(null,
                                          "The value of questions required will be\n" +
                                          "reset to the previous value with correct format",
                                          "Illegal number format",
                                          JOptionPane.ERROR_MESSAGE);

            reqAnswers.setText(Integer.toString(nofAnswersRequiredBackup));
            nofAnswersRequired = nofAnswersRequiredBackup;
        }

        object.setQuestionsRequired(nofAnswersRequired);

        object.setMapPosition(getMapPosition());
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(changeGroup)) {
            new ParentSelector(this, this.model);
        }
        else super.actionPerformed(e);
    }

    public InputPane getInputPane() {
        return text;
    }

    public void setEditable(boolean editable) {
        super.setMovable(editable);
        this.group.setEnabled(editable);
        this.name.setEnabled(editable);
        this.changeGroup.setEnabled(editable);
        this.reqAnswers.setEnabled(editable);
        this.description.setEnabled(editable);
        this.text.setEnabled(editable);
        this.formatBar.setEditable(editable);
    }
}
