package gui.editors.question.question;

import gui.editors.common.DPanel;
import gui.editors.common.InputPane;
import sys.datatypes.question.DarkQuestion;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import java.awt.*;

/**
 * The panel showing the input boxes for question, suggested solution
 * and test code for dark questions. Each of these text boxes have
 * their own tab, in order to make the room for typing as big as possible.
 *
 * @author Eivind Sorteberg - 21.jun.2006
 * @see gui.editors.question.category.QuestionCategoryPanel
 */
public class DarkPanel extends DPanel {
    private InputPane code;
    private InputPane testCode;
    private InputPane suggSol;

    private DarkQuestionPanel questionPanel;
    private QuestionTreeModel model;

    public DarkPanel() {
        super();

        InputPane question = new InputPane();
        suggSol = new InputPane();
        testCode = new InputPane();
        code = new InputPane();

        questionPanel = new DarkQuestionPanel(question, this);

        JPanel codePanel = new JPanel(new GridLayout(1, 1));
        codePanel.add(new JScrollPane(code));

        JPanel testcodePanel = new JPanel(new GridLayout(1, 1));
        testcodePanel.add(new JScrollPane(testCode));

        JPanel suggSolPanel = new JPanel(new GridLayout(1, 1));
        suggSolPanel.add(new JScrollPane(suggSol));

        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.add("Question", questionPanel);
        tabbedPane.add("Code", codePanel);
        tabbedPane.add("Test Code", testcodePanel);
        tabbedPane.add("Suggested solution", suggSolPanel);

        this.setLayout(new GridLayout(1, 1));
        this.add(tabbedPane);
    }

    public void showObject(DarkQuestion question) {
        this.code.setText(question.getCode());
        this.suggSol.setText(question.getSuggSol());
        this.testCode.setText(question.getTestCode());

        this.questionPanel.showObject(question);
    }

    public void saveObject(DarkQuestion question) {
        this.questionPanel.saveObject(question);

        question.setCode(this.code.getText());
        question.setSuggSol(this.suggSol.getText());
        question.setTestCode(this.testCode.getText());
    }

    public void setModel(QuestionTreeModel model) {
        this.model = model;
    }

    public AbstractTreeModel getModel() {
        return this.model;
    }

    public void setEditable(boolean editable) {
        this.questionPanel.setEditable(editable);
        this.code.setEnabled(editable);
        this.suggSol.setEnabled(editable);
        this.testCode.setEnabled(editable);
    }
}
