package gui.editors.question.question;

import gui.editors.common.DPanel;
import gui.editors.common.FormatBar;
import gui.editors.common.InputPane;
import gui.editors.common.Previewable;
import sys.datatypes.question.InputQuestion;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import java.awt.*;

/**
 * The panel showing the input box for the question, and
 * the input field for the answer. A <code>FormatBar</code> is
 * used on the top of the panel, allowing the user to write in
 * both HTML and LaTeX.
 *
 * @author Eivind Sorteberg - 21.jun.2006
 * @see gui.editors.question.category.QuestionCategoryPanel
 * @see FormatBar
 */
public class InputQuestionPanel extends DPanel implements Previewable {
    private JRadioButton html;
    private JRadioButton latex;

    private InputPane question;
    private JTextField answer;

    private QuestionTreeModel model;
    private FormatBar formatBar;

    public InputQuestionPanel() {
        super();

        this.html = new JRadioButton("HTML");
        this.latex = new JRadioButton("LaTeX");

        formatBar = new FormatBar(html, latex, this);

        this.question = new InputPane();
        this.answer = new JTextField();

        JPanel questionPanel = new JPanel(new GridLayout(1, 1));
        JPanel answerPanel = new JPanel(new GridLayout(1, 1));

        questionPanel.setBorder(BorderFactory.createTitledBorder("Question"));
        questionPanel.add(new JScrollPane(question));

        answerPanel.setBorder(BorderFactory.createTitledBorder("Answer"));
        answerPanel.add(answer);

        this.setLayout(new BorderLayout());
        this.add(formatBar, BorderLayout.NORTH);
        this.add(questionPanel, BorderLayout.CENTER);
        this.add(answerPanel,  BorderLayout.SOUTH);
    }

    public String[] getText() {
        return new String[]{question.getText(), answer.getText()};
    }

    public String getTitle() {
        return null;
    }

    public AbstractTreeModel getModel() {
        return this.model;
    }

    public void showObject(InputQuestion question) {
        this.question.setText(this.html.isSelected() ? question.getText() : question.getLatex());
        this.answer.setText(question.getAnswer());
    }

    public void saveObject(InputQuestion question) {
        if (html.isSelected()) {
            question.setText(this.question.getText());
        }
        else {
            question.setLatex(this.question.getText());
        }

        question.setAnswer(answer.getText());
    }

    public void setModel(QuestionTreeModel model) {
        this.model = model;
    }

    public InputPane getInputPane() {
        return question;
    }

    public void setEditable(boolean editable) {
        this.formatBar.setEditable(editable);
        this.question.setEnabled(editable);
        this.answer.setEnabled(editable);
    }
}
