package gui.editors.question.question;

import gui.editors.common.DPanel;
import sys.datatypes.question.*;
import sys.models.treemodels.QuestionTreeModel;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;

/**
 * The panel containing the necessary fields for generating or
 * editing a question of the type selected by the user.
 * The panel shown on this panel is dependent on the user's
 * type selection.
 *
 * @author Eivind Sorteberg - 21.jun.2006
 * @see QuestionObjectPanel
 * @see MultiChoicePanel
 * @see InputQuestionPanel
 * @see DarkPanel
 * @see AutoGenPanel
 */
public class QuestionTypeContent extends DPanel {
    private MultiChoicePanel multiChoicePanel;
    private InputQuestionPanel inputQuestionPanel;
    private DarkPanel darkPanel;
    private AutoGenPanel autoGenPanel;

    private JPanel top;

    public QuestionTypeContent() {
        super();
        this.setLayout(new GridLayout(1, 1));
        this.setBorder(BorderFactory.createLoweredBevelBorder());

        top = new JPanel(new GridLayout(1, 1));
        top.setBorder(new EmptyBorder(5, 5, 0, 5));

        this.setLayout(new GridLayout(1, 1));
        this.multiChoicePanel = new MultiChoicePanel();
        this.inputQuestionPanel = new InputQuestionPanel();
        this.darkPanel = new DarkPanel();
        this.autoGenPanel = new AutoGenPanel();

        this.add(top);
    }

    public void showMultChoice() {
        this.showPanel(multiChoicePanel);
    }

    public void showInput() {
        this.showPanel(inputQuestionPanel);
    }

    public void showDark() {
        this.showPanel(darkPanel);
    }

    public void showAutoGen() {
        this.showPanel(autoGenPanel);

    }

    /**
     * Updates the panel and shows the correct type corresponding
     * to the user's selection.
     *
     * @param panel The panel to be shown.
     */
    public void showPanel(JPanel panel) {
        top.removeAll();
        top.add(panel);
        top.validate();
        top.repaint();
    }

    public MultiChoicePanel getMultiChoicePanel() {
        return multiChoicePanel;
    }

    public InputQuestionPanel getInputQuestionPanel() {
        return inputQuestionPanel;
    }

    public DarkPanel getDarkPanel() {
        return darkPanel;
    }

    public AutoGenPanel getAutoGenPanel() {
        return autoGenPanel;
    }

    public void showQuestion(Question question) {
        if (question instanceof MultiChoiceQuestion) {
            this.multiChoicePanel.showObject((MultiChoiceQuestion)question);
            this.showPanel(multiChoicePanel);
        }
        else if (question instanceof InputQuestion) {
            this.inputQuestionPanel.showObject((InputQuestion)question);
            this.showPanel(inputQuestionPanel);
        }
        else if (question instanceof DarkQuestion) {
            this.darkPanel.showObject((DarkQuestion)question);
            this.showPanel(darkPanel);
        }
        else if (question instanceof AutoGenQuestion) {
            this.autoGenPanel.showObject((AutoGenQuestion)question);
            this.showPanel(autoGenPanel);
        }
        else System.err.println("ERROR! Illegal questiontype; found " + question.getClass());
    }

    public void saveObject(Question question) {
        if (question instanceof MultiChoiceQuestion) {
            multiChoicePanel.saveObject((MultiChoiceQuestion)question);
        }
        else if (question instanceof InputQuestion) {
            inputQuestionPanel.saveObject((InputQuestion)question);
        }
        else if (question instanceof DarkQuestion) {
            darkPanel.saveObject((DarkQuestion)question);
        }
        else if (question instanceof AutoGenQuestion) {
            autoGenPanel.saveObject((AutoGenQuestion)question);
        }
    }

    public void setModel(QuestionTreeModel model) {
        this.multiChoicePanel.setModel(model);
        this.inputQuestionPanel.setModel(model);
        this.darkPanel.setModel(model);
    }

    public void setEditable(boolean editable) {
        this.multiChoicePanel.setEditable(editable);
        this.inputQuestionPanel.setEditable(editable);
        this.darkPanel.setEditable(editable);
        this.autoGenPanel.setEditable(editable);
    }
}
