package gui.editors.text;

import gui.editors.AbstractEditor;
import gui.editors.common.DPanel;
import sys.datatypes.Group;
import sys.datatypes.Text;
import sys.datatypes.TextGroup;
import sys.datatypes.interfaces.AoCObject;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.TextTreeModel;

/**
 * Specific panel for the text editor panel, that is
 * for editing <code>Fact</code> and <code>History</code>
 * objects, and their corresponding groups.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class TextEditor extends AbstractEditor {
    private TextObjectPanel objectPanel;
    private TextGroupPanel groupPanel;

    private DPanel activePanel;

    public TextEditor() {
        super();
        this.objectPanel = new TextObjectPanel();
        this.groupPanel = new TextGroupPanel();
    }

    public void changePanel(DPanel panel) {
        this.activePanel = panel;

        this.removeAll();
        this.add(panel);
        this.validate();
        this.repaint();
    }

    public void showObject(AoCObject source) {
        if (source instanceof Text) {
            this.objectPanel.showObject((Text)source);
            this.changePanel(objectPanel);
            activePanel.setEditable(true);
        }
        else if (source instanceof TextGroup) {
            this.groupPanel.showObject((Group)source);
            this.changePanel(groupPanel);
            activePanel.setEditable(true);
        }
        else {
            if (activePanel != null) activePanel.setEditable(false);
        }
    }

    public void updateObject(AoCObject current) {
        if (current instanceof Text) objectPanel.saveObject((Text)current);
        else if (current instanceof TextGroup) groupPanel.saveObject((TextGroup)current);
    }

    public void setModel(AbstractTreeModel model) {
        if (model instanceof TextTreeModel) {
            objectPanel.setModel(model);
            groupPanel.setModel(model);
        }
    }

    public void superLevelModelChanged() {
        objectPanel.modelChanged();
    }
}
