package gui.editors.text;

import gui.editors.common.*;
import gui.editors.common.selector.ParentSelector;
import gui.editors.common.util.InfoPanelGenerator;
import sys.datatypes.Text;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Panel containing information about selected <code>Text</code>
 * objects.
 *
 * @author Eivind Sorteberg - 20.jun.2006
 * @see sys.datatypes.Text
 * @see TextEditor
 */
public class TextObjectPanel extends InfoPanel implements Previewable, ActionListener {

    //TextInfo variables
    public JTextField name;
    public SelectorField textgroup;
    public JButton changeGroup;
    public JTextField sequence;

    //InputPane variables
    private JRadioButton html;
    private JRadioButton latex;
    private InputPane input;
    private FormatBar formatBar;
    private int valueBackup;

    public TextObjectPanel() {
        super();

        this.name = new JTextField();
        this.textgroup = new SelectorField();
        this.changeGroup = new JButton("...");
        this.changeGroup.addActionListener(this);
        this.sequence = new JTextField();

        this.html  = new JRadioButton();
        this.latex = new JRadioButton();
        this.input = new InputPane();

        this.formatBar = new FormatBar(html, latex, this);

        JPanel inputPanel = new JPanel(new BorderLayout());
        inputPanel.add(formatBar, BorderLayout.NORTH);
        inputPanel.add(new JScrollPane(input), BorderLayout.CENTER);

        PlacementPanel placement = new PlacementPanel(levelBox, xPos, yPos, zPos, changePositionButton);

        this.setLayout(new BorderLayout());

        JPanel info = new JPanel();

        InfoPanelGenerator.placeComponents(info, 3, 3, new Object[]{
                "Group",        textgroup, changeGroup,
                "Name",         name,      null,
                "Sequence #",   sequence,  null
        });

        this.add(info, BorderLayout.NORTH);
        this.add(inputPanel, BorderLayout.CENTER);
        this.add(placement, BorderLayout.SOUTH);
    }

    public String[] getText() {
        return new String[]{input.getText()};
    }

    public String getTitle() {
        return name.getText();
    }

    public void showObject(Text source) {
        this.name.setText(source.getName());
        this.textgroup.setText(source.getParent().toString());
        this.input.setText(html.isSelected() ? source.getText() : source.getLatex());

        int seq = source.getSequence();
        this.sequence.setText(Integer.toString(seq));
        this.valueBackup = seq;

        super.showObject(source);
    }

    public void saveObject(Text object) {
        object.setName(this.name.getText());

        if (html.isSelected())  object.setText(this.input.getText());
        else                    object.setLatex(this.input.getText());

        object.setMapPosition(getMapPosition());

        int valueTemp;

        try {
            valueTemp = Integer.parseInt(sequence.getText());
        } catch (NumberFormatException e) {
            sequence.setText(Integer.toString(valueBackup));
            valueTemp = valueBackup;
        }

        object.setSequence(valueTemp);
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource().equals(changeGroup)) {
            new ParentSelector(this, this.model);
        }
        else super.actionPerformed(e);
    }

    public InputPane getInputPane() {
        return input;
    }

    public void setEditable(boolean editable) {
        super.setMovable(editable);
        this.formatBar.setEditable(editable);
        this.name.setEnabled(editable);
        this.textgroup.setEnabled(editable);
        this.changeGroup.setEnabled(editable);
        this.input.setEnabled(editable);
    }
}
