package gui.listeners;

import gui.MainFrame;
import gui.SQLTrace;
import gui.editors.common.selector.NewObjectSelector;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Listener for the MainFrame's JMenuBar.
 *
 * @author Tarje Killingberg - 20.jun.2006
 */
public class MenuBarListener implements ActionListener, Runnable {


    private MainFrame mainFrame;


    /**
     * Creates a new MenuBarListener
     *
     * @param mainFrame
     */
    public MenuBarListener(MainFrame mainFrame) {
        this.mainFrame = mainFrame;
    }


    public void run() {
        mainFrame.reload();
    }


    /**
     * Called when an item on the menubar has been chosen
     *
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        String actionCommand = e.getActionCommand();

        /* File menu */
        if (actionCommand.equalsIgnoreCase("New")) {
            if (mainFrame.getContent().getActivePanel() == mainFrame.getContent().getTabPanel()) {
                new NewObjectSelector(mainFrame.getContent(), mainFrame.getContent().getTabPanel().getActiveTab().getModel());
            }
            else {
                mainFrame.getContent().getAdminPanel().newUser();
            }
        }
        else if (actionCommand.equalsIgnoreCase("Delete")) {
            if (mainFrame.getContent().getActivePanel() == mainFrame.getContent().getTabPanel()) {
                mainFrame.getContent().getTabPanel().getModelManager().fireObjectDeleted();
            }
            else {
                mainFrame.getContent().getAdminPanel().deleteButtonPressed();
            }
        }
        else if (actionCommand.equalsIgnoreCase("Reload")) {
            new Thread(this).start();
        }
        else if (actionCommand.equalsIgnoreCase("Exit")) {
            mainFrame.exit();
        }

        /* View menu */
        else if (actionCommand.equalsIgnoreCase("Editor")) {
            mainFrame.setContent("editor");
        }
        else if (actionCommand.equalsIgnoreCase("Admin")) {
            mainFrame.setContent("admin");
        }
        else if (actionCommand.equalsIgnoreCase("SQL trace")) {
            if (SQLTrace.isVisible()) {
                SQLTrace.hide();
            }
            else {
                SQLTrace.show();
            }
            ((JMenuItem)e.getSource()).setSelected(SQLTrace.isVisible());
        }

        /* Help menu */
        else if (actionCommand.equalsIgnoreCase("Help")) {
            mainFrame.setDialogVisible("HELP", true);
        }
        else if (actionCommand.equalsIgnoreCase("CLOSE_HELP")) {
            mainFrame.setDialogVisible("HELP", false);
        }
        else if (actionCommand.equalsIgnoreCase("About")) {
            mainFrame.setDialogVisible("ABOUT", true);
        }
        else if (actionCommand.equalsIgnoreCase("CLOSE_ABOUT")) {
            mainFrame.setDialogVisible("ABOUT", false);
        }

    }


}
