package gui.tabs;

import gui.editors.level.LevelEditor;
import gui.editors.map.MapViewPanel;
import gui.tree.LevelTreePanel;
import sys.models.LevelModel;
import sys.models.treemodels.AbstractTreeModel;
import sys.models.treemodels.LevelTreeModel;

/**
 * The tab containing the tree and the editor panel
 * for the level-related editor view.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class LevelTab extends AbstractTab {

    public LevelTab() {
    }

    public void setEditor() {
        this.editor = new LevelEditor();
    }

    public void setTree() {
        this.tree = new LevelTreePanel();
    }

    /**
     * Sets the model for this tab. Should take a
     * <code>LevelTreeModel</code> as in parameter.
     *
     * @param model The model representing the current Level view
     * @see sys.models.treemodels.LevelTreeModel
     */
    public void setModel(AbstractTreeModel model) {
        if (model instanceof LevelTreeModel) {
            this.model = model;
            this.tree.setModel(model);
            this.treelistener.setModel(model);
            editor.setModel(model);
        }
        else {
            System.err.println("Illegal model: Expected LevelTreeModel, found " + model.getClass());
        }
    }


    public void updateLevelModel(LevelModel levelModel) {
        if (this.editor instanceof LevelEditor) {
            ((LevelEditor)this.editor).getMapPanel().levelModelChanged(MapViewPanel.MAP_CHANGED);
        }
        else System.err.println("LevelTab.editor illegal type; found: " + this.editor.getClass().getName());
    }

    public String toString() {
        return "Level Editor";
    }
}
