package gui.tree;

import sys.AoCAdmin;
import sys.datatypes.*;
import sys.datatypes.question.QuestionCategory;

import javax.swing.*;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import java.awt.*;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Renders the nodes of the trees. For those objects that have icons defined,
 * these icons are displayed, while other objects (such as TextGroups) use
 * icons defined by the L&F in use. The icons are loaded once, on demand, from the web
 * server given in the login screen.
 *
 * @author Eivind Sorteberg - 28.jun.2006
 */
public class TreeRenderer implements TreeCellRenderer {
    private static Icon teleDestIcon;
    private static Icon teleSrcIcon;
    private static Icon msgIcon;
    private static Icon qGroupIcon;
    private static Icon qCatIcon;
    private static Icon questionIcon;
    private static Icon textGroupIcon;
    private static Icon factIcon;
    private static Icon historyIcon;

    public TreeRenderer() {
        teleDestIcon =  loadIcon(teleDestIcon,  "teledest.png");
        teleSrcIcon =   loadIcon(teleSrcIcon,   "telesrc.png");
        msgIcon =       loadIcon(msgIcon,       "message.png");
        qGroupIcon =    loadIcon(qGroupIcon,    "q_group.png");
        qCatIcon =      loadIcon(qCatIcon,      "q_cat.png");
        questionIcon =  loadIcon(questionIcon,  "question.png");
        textGroupIcon = loadIcon(textGroupIcon, "t_group.png");
        factIcon =      loadIcon(factIcon,      "fact.png");
        historyIcon =   loadIcon(historyIcon,   "history.png");
    }

    /**
     * Loads the given icon, if not previously done, using the
     * name of the icon and the address of the web server.
     *
     * @param icon The icon - null if not previously loaded.
     * @param name The name of the icon (file name)
     * @return The icon; null if not found.
     */
    private Icon loadIcon(Icon icon, String name) {
        URL iconAddress = null;
        try {
            iconAddress = new URL(AoCAdmin.WEB_SERVER + AoCAdmin.TREE_ICON_FOLDER + name);
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        if (icon == null) return new ImageIcon(iconAddress);
        else return icon;
    }

    /**
     * Returns a Component representing the object to display.
     *
     * @param tree The tree to which the node belongs
     * @param value The object to display
     * @param selected Whether the object is currently selected
     * @param expanded Whether the object is an expanded nonleaf
     * @param leaf Whether the object is a leaf
     * @param row
     * @param hasFocus Whether the object has focus
     * @return A Component representing the node in question
     */
    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        JLabel label = new JLabel(value.toString());

        if (value instanceof GroupElement) {
            GroupElement node = (GroupElement)value;


            if (selected) {
                label.setOpaque(true);
                DefaultTreeCellRenderer dr = new DefaultTreeCellRenderer();
                label.setBackground(dr.getBackgroundSelectionColor());
                if (hasFocus) {
                    label.setForeground(Color.RED);
                }
                else {
                    label.setForeground(Color.BLUE);
                }
            }

            label.setToolTipText(node.getToolTipText());

            Icon icon;

            if      (value instanceof TeleportDestination)  icon = teleDestIcon;
            else if (value instanceof TeleportSource)       icon = teleSrcIcon;
            else if (value instanceof Message)              icon = msgIcon;
            else if (value instanceof Fact)                 icon = factIcon;
            else if (value instanceof History)              icon = historyIcon;
            else if (value instanceof QuestionCategory)     icon = qCatIcon;
            else {
                DefaultTreeCellRenderer dr = new DefaultTreeCellRenderer();
                if (leaf)                                   icon = dr.getDefaultLeafIcon();
                else {
                    if (expanded)                           icon = dr.getDefaultOpenIcon();
                    else                                    icon = dr.getDefaultClosedIcon();
                }
            }

            label.setIcon(icon);

        }
        return label;
    }
}
