package sys.datatypes;

import sys.Parser.Parser;
import sys.datatypes.interfaces.AoCObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

/**
 * A Group contains a list of elements, which are the group's children.
 * A child (<code>GroupElement</code>) can be both other groups, or basic elements.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 * @see GroupElement
 */
public abstract class Group extends GroupElement {
    private ArrayList<GroupElement> groupElements;

    private String name;
    private String description;

    public Group(int id, int parent_id, String name, String description) {
        super(id, parent_id);
        this.name = name;
        this.description = description;

        this.groupElements = new ArrayList<GroupElement>();
    }

    public boolean addElement(GroupElement element) {
        return groupElements.add(element);
    }

    public void addElement(int index, GroupElement element) {
        groupElements.add(index, element);
    }

    public boolean removeElement(GroupElement element) {
        groupElements.remove(element);
        return true;
    }

    public GroupElement removeElement(int index) {
        return groupElements.remove(index);
    }

    public GroupElement getElement(int index) {
        return groupElements.get(index);
    }

    public int indexOfElement(GroupElement element) {
        return groupElements.indexOf(element);
    }

    public int getNumberOfElements() {
        return groupElements.size();
    }

    public void delete() {
        GroupElement[] copy = new GroupElement[groupElements.size()];
        groupElements.toArray(copy);
        ArrayList<AoCObject> toStore = new ArrayList<AoCObject>();


        for (GroupElement child : copy) {
            child.setParent(this.getParent());
            toStore.add((AoCObject)child);
        }

        Parser.updateObjects(toStore);

        this.getParent().removeElement(this);
    }

    public String toString() {
        return this.name;
    }

    public ArrayList<GroupElement> getGroupElements() {
        return groupElements;
    }

    public void sortGroupElements() {
        Collections.sort(groupElements);
    }

    public void setGroupElements(ArrayList<GroupElement> groupElements) {
        this.groupElements = groupElements;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDescription() {
        return description != null ? description : "";
    }

    public void setDescription(String description) {
        this.description = description;
    }



    /* ----- Methods from interface AoCObject ----- */
    public Object[] getFieldValues() {

        return new Object[]{this.getParent_id(),
                            this.getName(),
                            this.getDescription()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setName((String)attributes.get("Name"));
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof Group)) return true;

        Group g2 = (Group)object;

        return  super.differsFrom(g2) ||
                !this.getName().equals(g2.getName()) ||
                !this.getDescription().equals(g2.getDescription());
    }

    public void sortSubTree() {
        for (GroupElement child : getGroupElements()) {
            if (child instanceof Group) {
                ((Group)child).sortGroupElements();
                ((Group)child).sortSubTree();
            }
        }
    }

}
