package sys.datatypes;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.models.LevelModel;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class Level extends GroupElement implements MapItem {

    public static final String DB_NAME = "Level";
    public static final String ROOTLEVEL = "Rootlevel";

    private String name;
    private String map;
    private LevelModel levelModel;


    public Level(int id, String name, String map) {
        super(id, 0);
        this.name = name;
        this.map = map;
        levelModel = null;
    }

    /**
     * Construktor ONLY for the rootlevel.
     * NOT TO BE USED IN ANY OTHER CASE
     *
     * @param id
     * @param parent_id
     */
    public Level(int id, int parent_id) {
        super(id, parent_id);
        this.name = "Root";
        this.map = "nomap";
        levelModel = null;
    }


    public String getName() {
        return name;
    }

    public void setName(String name) {
        if (!getName().equals(ROOTLEVEL)) {
            this.name = name;
        }
    }

    public String getMap() {
        return map;
    }

    public boolean setMap(String map) {
        if (!(getName().equals(ROOTLEVEL)) && !(map.equals(this.getMap()))) {
            this.map = map;
            return true;
        }
        return false;
    }


    public String toString() {
        return !getName().equals("Root") ? (getId() > 9 ? getId() > 99 ? "" + getId() : "0" + getId() : "00" + getId()) + ": " + getName() : getName();
    }

    public Object[] getFieldValues() {
        return new Object[]{getName(), getMap()};
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setName((String)attributes.get("Name"));
        this.setMap((String)attributes.get("Map"));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Name:</b> " + getName() + "<br><b>Map:</b> " + getMap() + "</html>";
    }

    public String getDBname() {
        return DB_NAME;
    }

    public AoCObject makeClone() {
        return new Level(this.getId(), this.getName(), this.getMap());
    }

    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof Level)) return true;

        Level l2 = (Level)object;

        return  this.getId() != l2.getId() ||
                !(this.getName().equals(l2.getName())) ||
                !(this.getMap().equals(l2.getMap()));
    }

    public String[] getFieldNames() {
        return new String[]{"ID",
                            "Name",
                            "Map"};
    }



    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
    }


    public void moveTo(int x, int y, int z) {
    }


    public MapPosition getMapPosition() {
        return null;
    }


    public void setMapPosition(MapPosition mapPosition) {
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }


    public void setLevelModel(LevelModel lm) {
        levelModel = lm;
    }

    public String getStatusName() {
        return "Level";
    }
}
