package sys.datatypes;

import sys.DefaultValues;

/**
 * A representation of a position on a map, given
 * by x-, y- and z-coordinates.
 *
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class MapPosition {
    private int x;
    private int y;
    private int z;

    public MapPosition(int x, int y, int z) {
        this.changePosition(x, y, z);
    }

    /**
     * The default constructor sets the values of the
     * x- and y-coordinates to negative (undefined) values.
     * Default z-value is 0.
     */
    public MapPosition() {
        this(DefaultValues.MAPITEM_MAP_POSITION.getX(), DefaultValues.MAPITEM_MAP_POSITION.getY(), DefaultValues.MAPITEM_MAP_POSITION.getZ());
    }

    /**
     * Makes it possible to define the x- and y-coordinates,
     * while default z-value 0 is used.
     *
     * @param x
     * @param y
     */
    public MapPosition(int x, int y) {
        this(x, y, 0);
    }

    /**
     * Moved the position in three dimensions.
     * Recommended when moving objects between levels.
     *
     * @param x
     * @param y
     * @param z
     */
    public void changePosition(int x, int y, int z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    /**
     * Moves the posision in two dimensions, while the z-value
     * stays the same (horizontal movement).
     * Recommended when moving objects within the same level.
     *
     * @param x
     * @param y
     */
    public void changePosition(int x, int y) {
        this.changePosition(x, y, this.z);
    }

    public int getX() {
        return x;
    }

    public void setX(int x) {
        this.x = x;
    }

    public int getY() {
        return y;
    }

    public void setY(int y) {
        this.y = y;
    }

    public int getZ() {
        return z;
    }

    public void setZ(int z) {
        this.z = z;
    }

    public boolean differsFrom(MapPosition p2) {
        return this.getX() != p2.getX() || this.getY() != p2.getY() || this.getZ() != p2.getZ();
    }

    public String toString() {
        return "MapPosition (" + getX() + ", " + getY() + ", " + getZ() + ")";
    }
}
