package sys.datatypes;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.models.LevelModel;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class Message extends GroupElement implements MapItem {
    public static final String DB_NAME = "Message";

    private String text;
    private MapPosition mapPosition;
    private LevelModel levelModel;

    public Message(int id, int levelId, String text, MapPosition mapPosition) {
        super(id, levelId);
        this.text = text != null ? text : "";
        this.mapPosition = mapPosition;
        levelModel = null;
    }

    public Message(int levelId, MapPosition mapPosition) {
        super(-1, levelId);
        this.text = "";
        this.mapPosition = mapPosition;
        levelModel = null;
    }


    public String getText() {
        return text != null ? text : "";
    }

    public void setText(String text) {
        this.text = text;
    }

    public String toString() {
        return "Msg " + this.getId();
    }


    @Override
    public void delete() {
        super.delete();
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
    }



    /* ----- Methods from interface AoCObject -----*/


    public String getDBname() {
        return DB_NAME;
    }


    public String[] getFieldNames() {

        return new String[]{"Text",
                            "LevelID",
                            "PosX",
                            "PosY",
                            "PosZ"};

    }


    public Object[] getFieldValues() {

        return new Object[]{this.getText(),
                            this.getParent_id(),
                            this.getMapPosition().getX(),
                            this.getMapPosition().getY(),
                            this.getMapPosition().getZ()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setText((String)attributes.get("Text"));

        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Level ID:</b> " + getLevelModel().getId() + "<br><b>Text:</b> " + getText() + "</html>";
    }

    public String getStatusName() {
        return "Message";
    }

    public AoCObject makeClone() {

        return new Message(this.getId(),
                           this.getParent_id(),
                           this.getText(),
                           new MapPosition(this.getMapPosition().getX(),
                                           this.getMapPosition().getY(),
                                           this.getMapPosition().getZ()));

    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof Message)) return true;

        Message m2 = (Message)object;

        return  super.differsFrom(m2) ||
                !this.getText().equals(m2.getText()) ||
                this.getMapPosition().differsFrom(m2.getMapPosition());

    }

    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
        this.mapPosition.changePosition(x, y);
    }


    public void moveTo(int x, int y, int z) {
        this.mapPosition.changePosition(x, y, z);
    }


    public MapPosition getMapPosition() {
        return mapPosition;
    }


    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }


    public void setLevelModel(LevelModel lm) {
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
        this.levelModel = lm;
        this.levelModel.addElement(this);
        setParent_id(levelModel.getId());
    }


}
