package sys.datatypes;

import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.models.LevelModel;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class TeleportDestination extends GroupElement implements MapItem {
    public static final String DB_NAME = "TeleportDest";

    private String text;
    private MapPosition mapPosition;
    private LevelModel levelModel;

    /**
     * Complete constructor. Used when creating objects from DB
     *
     * @param id
     * @param text
     * @param levelId
     * @param mapPosition
     */
    public TeleportDestination(int id, String text, int levelId, MapPosition mapPosition) {
        super(id, levelId);

        this.text = text != null ? text : "";
        this.mapPosition = mapPosition;
        levelModel = null;
    }

    public TeleportDestination(int levelId, MapPosition mapPosition) {
        super(-1, levelId);
        this.text = "";
        this.mapPosition = mapPosition;
        levelModel = null;
    }


    public String getText() {
        return text != null ? text : "";
    }

    public void setText(String text) {
        this.text = text;
    }

    public String toString() {
        return "TeleDest " + this.getId();
    }


    @Override
    public void delete() {
        super.delete();
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
    }



    /* ----- Methods from interface AoCObject ----- */


    public String getDBname() {
        return DB_NAME;
    }


    public String[] getFieldNames() {

        return new String[]{"Text",
                            "LevelID",
                            "PosX",
                            "PosY",
                            "PosZ"};

    }


    public Object[] getFieldValues() {

        return new Object[]{this.getText(),
                            this.getParent_id(),
                            this.getMapPosition().getX(),
                            this.getMapPosition().getY(),
                            this.getMapPosition().getZ()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);

        this.setText((String)attributes.get("Text"));
        this.setParent_id((Integer)attributes.get("LevelID"));

        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Level ID:</b> " + getLevelModel().getId() + "</html>";
    }


    public AoCObject makeClone() {

        return new TeleportDestination(this.getId(),
                                       this.getText(),
                                       this.getParent_id(),
                                       new MapPosition(this.getMapPosition().getX(),
                                                       this.getMapPosition().getY(),
                                                       this.getMapPosition().getZ()));

    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof TeleportDestination)) return true;

        TeleportDestination t2 = (TeleportDestination)object;

        return  super.differsFrom(t2) ||
                !this.getText().equals(t2.getText()) ||
                this.getMapPosition().differsFrom(t2.getMapPosition());

    }

    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
        this.mapPosition.changePosition(x, y);
    }


    public void moveTo(int x, int y, int z) {
        this.mapPosition.changePosition(x, y, z);
    }


    public MapPosition getMapPosition() {
        return mapPosition;
    }


    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }


    public void setLevelModel(LevelModel lm) {
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
        this.levelModel = lm;
        this.levelModel.addElement(this);
        setParent_id(levelModel.getId());
    }

    public String getStatusName() {
        return "Teleport Destination";
    }
}
