package sys.datatypes;

import sys.DefaultValues;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.interfaces.MultiFormat;
import sys.models.LevelModel;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public abstract class Text extends GroupElement implements MapItem, MultiFormat {
    public static final String DB_NAME = "Text";

    protected String name;
    protected String text;
    protected String latex;
    protected int level;
    protected MapPosition mapPosition;
    private int sequence;
    protected LevelModel levelModel;

    protected Text(int id, int parent_id, String name, String text, String latex, int level, MapPosition mapPosition, int sequence) {
        super(id, parent_id);
        this.name = name;
        this.text = text;
        this.latex = latex;
        this.level = level;
        this.mapPosition = mapPosition;
        this.sequence = sequence;
    }

    protected Text(String name, int levelId, MapPosition mapPosition) {
        super(-1, DefaultValues.TEXT_GROUP);
        this.name = name;
        this.level = levelId;
        this.mapPosition = mapPosition;
    }

    protected Text(String name) {
        super(0, DefaultValues.TEXT_GROUP);
        this.name = name;
        this.mapPosition = new MapPosition();
        this.level = DefaultValues.MAPITEM_LEVEL_ID;
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        this.setId((Integer)attributes.get("ID"));
        this.setName((String)attributes.get("Name"));
        this.setText((String)attributes.get("Text"));
        this.setLatex((String)attributes.get("LaTeX"));
        this.setLevel_id((Integer)attributes.get("LevelID"));
        this.setSequence((Integer)attributes.get("Sequence"));

        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))
        ));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Name:</b> " + getName() + "<br><b>Type:</b> " + getFieldValues()[4] + "<br><b>Group ID:</b> " + getParent_id() + "<br><b>Level ID:</b> " + getLevelModel().getId() + "</html>";
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getText() {
        return text != null ? text : "";
    }

    public void setText(String text) {
        this.text = text;
    }

    public String getLatex() {
        return latex != null ? latex : "";
    }

    public void setLatex(String latex) {
        this.latex = latex;
    }

    public int getLevel() {
        return level;
    }

    public void setSequence(int sequence) {
        this.sequence = sequence;
    }

    public int getSequence() {
        return sequence;
    }


    public boolean setParent(Group parent) {
        return super.setParent(parent);
    }


    /**
     * Overrides GroupElement.delete().
     * Makes sure the object is deleted from both its textgroup and
     * its LevelModel.
     */
    public void delete() {
        try {
            this.getParent().removeElement(this);
            this.getLevelModel().removeElement(this);
        } catch (NullPointerException e) {
            e.printStackTrace();
        }
    }

    public String toString() {
        return name;
    }

    public String[] getFieldNames() {
        return new String[]{"TextGroupID",
                            "Name",
                            "Text",
                            "LaTeX",
                            "Type",
                            "LevelID",
                            "PosX",
                            "PosY",
                            "PosZ",
                            "Sequence"};
    }

    public Object[] getAttributes(String type) {
        int x = getMapPosition().getX();
        int y = getMapPosition().getY();
        int z = getMapPosition().getZ();
        return new Object[]{getParent_id(), getName(), getText(), getLatex(), type, getLevel(), x, y, z, getSequence()};
    }

    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof Text)) return true;

        Text t2 = (Text)object;

        return super.differsFrom(t2) ||
                !this.getName().equals(t2.getName()) ||
                !this.getText().equals(t2.getText()) ||
                !this.getLatex().equals(t2.getLatex()) ||
                this.getLevel() != t2.getLevel() ||
                this.getMapPosition().differsFrom(t2.getMapPosition());
    }


    /**
     * Method returning the DB_NAME "Text" for all subclasses of this class
     *
     * @return The name of the corresponding DB table ("Text").
     */
    public String getDBname() {
        return DB_NAME;
    }



    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
        this.mapPosition.changePosition(x, y);
    }


    public void moveTo(int x, int y, int z) {
        this.mapPosition.changePosition(x, y, z);
    }


    public MapPosition getMapPosition() {
        return mapPosition;
    }


    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }


    public void setLevelModel(LevelModel lm) {
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
        this.levelModel = lm;
        this.levelModel.addElement(this);
        this.level = levelModel.getId();
    }

    public void setLevel_id(int level_id) {
        this.level = level_id;
    }
}
