package sys.datatypes;

import sys.DefaultValues;
import sys.datatypes.interfaces.AoCObject;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class TextGroup extends Group {

    public static final String DB_NAME = "TextGroup";
    private int sequence;

    public TextGroup(int id, int parent, String name, String text, int sequence) {
        super(id, parent, name, text);
        this.sequence = sequence;
    }

    public TextGroup() {
        this(-1, DefaultValues.TEXT_GROUP, "New Text Group", null, 0);
    }

    public int getSequence() {
        return sequence;
    }

    public void setSequence(int sequence) {
        this.sequence = sequence;
    }

    public AoCObject makeClone() {
        return new TextGroup(this.getId(), this.getParent_id(), this.getName(), this.getDescription(), this.getSequence());
    }

    public String getDBname() {
        return DB_NAME;
    }

    public String[] getFieldNames() {
        return new String[] {"ParentID",
                             "Name",
                             "Text",
                             "Sequence"};
    }

    public Object[] getFieldValues() {
        return new Object[]{this.getParent_id(), this.getName(), this.getDescription(), this.getSequence()};
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setDescription((String)attributes.get("Text"));
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Name:</b> " + getName() + "<br><b>Group ID:</b> " + getParent_id() + "</html>";
    }

    public String getStatusName() {
        return "Text Group";
    }

    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof TextGroup)) return true;

        TextGroup g2 = (TextGroup)object;

        return  super.differsFrom(object) ||
                this.getSequence() != g2.getSequence();
    }
}
