package sys.datatypes;

import sys.DefaultValues;
import sys.datatypes.interfaces.AoCObject;

import java.util.HashMap;

/**
 * This class represents a User in the database.
 *
 * @author Tarje Killingberg - 20.jun.2006
 */
public class User implements AoCObject {


    public static final String DB_NAME = "User";


    private int Id;
    private String username;
    private String password;
    private boolean admin;
    private int avatar;
    private int levelID;
    private MapPosition mapPosition;
    private int points;



    public User(int Id, String username, String password, boolean admin, int avatar, int levelID, MapPosition mapPosition) {
        this.Id = Id;
        this.username = username;
        this.password = password;
        this.admin = admin;
        this.avatar = avatar;
        this.levelID = levelID;
        this.mapPosition = mapPosition;
        this.points = 0;
    }


    public User() {
        this.Id = -1;
        this.username = "New user";
        this.password = "";
        this.admin = false;
        this.avatar = 0;
        this.levelID = DefaultValues.USER_LEVEL_ID;
        this.mapPosition = DefaultValues.USER_MAP_POSITION;
        this.points = 0;
    }


    public String toString() {
        return "User " + getId();
    }



    /* ----- Getters and setters ----- */


    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public boolean isAdmin() {
        return admin;
    }

    public void setAdmin(boolean admin) {
        this.admin = admin;
    }

    public int getAvatar() {
        return avatar;
    }

    public void setAvatar(int avatar) {
        this.avatar = avatar;
    }

    public int getLevelID() {
        return levelID;
    }

    public void setLevelID(int levelID) {
        this.levelID = levelID;
    }

    public MapPosition getMapPosition() {
        return mapPosition;
    }

    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }

    public int getPoints() {
        return points;
    }

    public void setPoints(int points) {
        this.points = points;
    }



    /* ----- Methods from interface AoCObject ----- */


    public String getDBname() {
        return DB_NAME;
    }


    public int getId() {
        return Id;
    }


    public void setId(int Id) {
        this.Id = Id;
    }


    public String[] getIdFields() {
        return new String[]{"ID"};
    }


    public Object[] getIdValues() {
        return new Object[]{getId()};
    }


    public String[] getFieldNames() {

        return new String[]{"Username",
                            "Password",
                            "Admin",
                            "Avatar",
                            "LevelID",
                            "PosX",
                            "PosY",
                            "PosZ"};

    }


    public Object[] getFieldValues() {

        return new Object[]{this.getUsername(),
                            this.getPassword(),
                            this.isAdmin(),
                            this.getAvatar(),
                            this.getLevelID(),
                            this.getMapPosition().getX(),
                            this.getMapPosition().getY(),
                            this.getMapPosition().getZ()};

    }


    public void setAllAttributes(HashMap<String, Object> attributes) {
        this.setId((Integer)attributes.get("ID"));
        this.setUsername((String)attributes.get("Username"));
        this.setPassword((String)attributes.get("Password"));
        this.setAdmin((Boolean)attributes.get("Admin"));
        this.setAvatar((Integer)attributes.get("Avatar"));
        this.setLevelID((Integer)attributes.get("LevelID"));
        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))));
    }

    public Object[][] getAttributeArray() {
        return new Object[0][];
    }

    public AoCObject makeClone() {

        User user = new User(this.getId(),
                             this.getUsername(),
                             this.getPassword(),
                             this.isAdmin(),
                             this.getAvatar(),
                             this.getLevelID(),
                             new MapPosition(this.getMapPosition().getX(),
                                             this.getMapPosition().getY(),
                                             this.getMapPosition().getZ()));

        user.setPoints(this.getPoints());

        return user;
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof User)) return true;

        User u2 = (User)object;

        return  this.getId() != u2.getId() ||
                !this.getUsername().equals(u2.getUsername()) ||
                !this.getPassword().equals(u2.getPassword()) ||
                this.isAdmin() != u2.isAdmin() ||
                this.getAvatar() != u2.getAvatar() ||
                this.getLevelID() != u2.getLevelID() ||
                this.getMapPosition().differsFrom(u2.getMapPosition());

    }

    public String getInfo() {
        return null;
    }

    public HashMap<String, Object> getAllAttributes() {
        HashMap<String, Object> attributes = new HashMap<String, Object>();

        String[] idFields = getIdFields();
        Object[] idValues = getIdValues();

        for (int i = 0; i < idFields.length; i++) {
            attributes.put(idFields[i], idValues[i]);
        }

        String[] otherFields = getFieldNames();
        Object[] otherValues = getFieldValues();

        for (int i = 0; i < otherFields.length; i++) {
            attributes.put(otherFields[i], otherValues[i]);
        }

        return attributes;
    }

    public String getInfo(int coloredAttributes) {
        return getInfo();
    }
}