package sys.datatypes.question;

import sys.datatypes.interfaces.AoCObject;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class AutoGenQuestion extends Question {
    public static final String DB_NAME = "AutoGenQuestion";

    private String code;
    private String oldCode;
    private String formula;

    public AutoGenQuestion(int id, int parent_id, String text, String latex, int value, String comment, String code, String oldCode, String formula) {
        super(id, parent_id, text, latex, value, comment);
        this.code = code;
        this.oldCode = oldCode;
        this.formula = formula;
    }

    public AutoGenQuestion() {
        super();
        this.code = "";
        this.oldCode = "";
        this.formula = "";
    }

    public String getCode() {
        return code;
    }

    public String getOldCode() {
        return oldCode;
    }

    public String getFormula() {
        return formula;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public void setOldCode(String oldCode) {
        this.oldCode = oldCode;
    }

    public void setFormula(String formula) {
        this.formula = formula;
    }

    public Object[] getSpecialAttributes() {
        return new Object[]{getCode(), getOldCode(), getFormula()};
    }

    public String[] getSpecialFieldNames() {
        return new String[]{"Code", "OldCode", "Formula"};
    }

    public String getSpecialDBname() {
        return DB_NAME;
    }



    /* ----- Methods from interface AoCObject ----- */


    public Object[] getFieldValues() {
        return super.getAttributes("autogen");
    }

    public AoCObject makeClone() {
        AutoGenQuestion q = new AutoGenQuestion(
                this.getId(),
                this.getParent_id(),
                this.getText(),
                this.getLatex(),
                this.getValue(),
                this.getComment(),
                this.getCode(),
                this.getOldCode(),
                this.getFormula());

        return q;
    }

    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof AutoGenQuestion)) return true;

        AutoGenQuestion q2 = (AutoGenQuestion)object;

        return  this.getId() != q2.getId() ||
                this.getParent_id() != q2.getParent_id() ||
                !this.getText().equals(q2.getText()) ||
                !this.getLatex().equals(q2.getLatex()) ||
                this.getValue() != q2.getValue() ||
                !this.getComment().equals(q2.getComment()) ||
                !this.getCode().equals(q2.getCode()) ||
                !this.getFormula().equals(q2.getFormula()) ||
                !this.getOldCode().equals(q2.getOldCode());
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setCode((String)attributes.get("Code"));
        this.setOldCode((String)attributes.get("OldCode"));
        this.setFormula((String)attributes.get("Formula"));
    }

}