package sys.datatypes.question;

import sys.datatypes.interfaces.AoCObject;
import sys.models.MCAlternativeModel;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class MultiChoiceQuestion extends Question {
    public static final String DB_NAME = "MultiChoiceQuestion";

    private int answerID;
    private MCAlternativeModel alternativeModel;

    public MultiChoiceQuestion(int id, int parent_id, String text, String latex, int value, String comment, int answer_id) {
        super(id, parent_id, text, latex, value, comment);
        this.answerID = answer_id;
        this.alternativeModel = new MCAlternativeModel(this);
    }

    public MultiChoiceQuestion(int id, int parent_id, String text, String latex, int value, String comment, int answerID, MCAlternativeModel alternativeModel) {
        super(id, parent_id, text, latex, value, comment);
        this.answerID = answerID;
        this.alternativeModel = alternativeModel;
    }

    public MultiChoiceQuestion() {
        super();
        this.answerID = 0;
        this.alternativeModel = new MCAlternativeModel(this);
    }

    public int getAnswerID() {
        return answerID;
    }

    public void setAnswerID(int answerID) {
        this.answerID = answerID;
    }

    public void addAlternative(MultiChoiceAlternative alternative) {
        this.alternativeModel.addAlternative(alternative);
    }

    public Object[] getSpecialAttributes() {
        return new Object[]{getAnswerID()};
    }

    public String[] getSpecialFieldNames() {
        return new String[]{"AnswerID"};
    }

    public String getSpecialDBname() {
        return DB_NAME;
    }

    public ArrayList<MultiChoiceAlternative> getAlternatives() {
        return this.alternativeModel.getAlternatives();
    }

    public MCAlternativeModel getAlternativeModel() {
        return this.alternativeModel;
    }

    /* ------ Methods from interface AoCObject ----- */


    public Object[] getFieldValues() {
        return super.getAttributes("multichoice");
    }


    public AoCObject makeClone() {
        return new MultiChoiceQuestion(
                this.getId(),
                this.getParent_id(),
                this.getText(),
                this.getLatex(),
                this.getValue(),
                this.getComment(),
                this.getAnswerID(),
                this.getAlternativeModel().makeClone());
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof MultiChoiceQuestion)) return true;

        MultiChoiceQuestion q2 = (MultiChoiceQuestion)object;

        return  this.getId() != q2.getId() ||
                this.getParent_id() != q2.getParent_id() ||
                !this.getText().equals(q2.getText()) ||
                !this.getLatex().equals(q2.getLatex()) ||
                this.getValue() != q2.getValue() ||
                !this.getComment().equals(q2.getComment()) ||
                this.getAnswerID() != q2.getAnswerID() ||
                this.getAlternativeModel().differsFrom(q2.getAlternativeModel());

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setAnswerID((Integer)attributes.get("AnswerID"));
    }

    public void resetAlternatives(MultiChoiceQuestion multiChoiceQuestion) {
        this.alternativeModel = multiChoiceQuestion.getAlternativeModel().makeClone();
    }
}
