package sys.datatypes.question;

import sys.DefaultValues;
import sys.datatypes.GroupElement;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MultiFormat;

import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public abstract class Question extends GroupElement implements MultiFormat {
    public static final String DB_NAME = "Question";

    private String text;
    private String latex;
    private int value;
    private String comment;

    protected Question(int id, int parent_id, String text, String latex, int value, String comment) {
        super(id, parent_id);
        this.text = text;
        this.latex = latex;
        this.value = value;
        this.comment = comment;
    }

    protected Question() {
        this(-1, DefaultValues.QUESTION_CATEGORY, null, null, 0, null);
    }

    public String getText() {
        return text != null ? text : "";
    }

    public String getLatex() {
        return latex != null ? latex : "";
    }

    public int getValue() {
        return value;
    }

    public String getComment() {
        return comment != null ? comment : "";
    }

    public void setText(String text) {
        this.text = text;
    }

    public void setLatex(String latex) {
        this.latex = latex;
    }

    public void setValue(int value) {
        this.value = value;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public Object[] getAttributes(String type) {
        return new Object[]{getParent_id(), getText(), getLatex(), getValue(), getComment(), type};
    }

    public String toString() {
        return Integer.toString(getId());
    }


    public abstract String getSpecialDBname();

    public abstract String[] getSpecialFieldNames();

    public abstract Object[] getSpecialAttributes();

/* ------ Methods from interface AoCObject ----- */


    public String getDBname() {
        return DB_NAME;
    }


    public String[] getFieldNames() {
        return new String[]{
                "QuestionCategoryID",
                "Text",
                "LaTeX",
                "Value",
                "Comment",
                "Type"};
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof Question)) return true;

        Question q2 = (Question)object;

        return  super.differsFrom(q2) ||
                !this.getText().equals(q2.getText()) ||
                !this.getLatex().equals(q2.getLatex()) ||
                this.getValue() != q2.getValue() ||
                !this.getComment().equals(q2.getComment());

    }


    public Object[][] getAttributeArray() {
        Object[][] common = super.getAttributeArray();

        String[] specialFields = getSpecialFieldNames();
        Object[] specialValues = getSpecialAttributes();

        Object[][] attributes = new Object[common.length + specialFields.length][2];

        for (int i = 0; i < common.length; i++) {
            attributes[i][0] = common[i][0];
            attributes[i][1] = common[i][1];
        }

        for (int i = 0; i < specialFields.length; i++) {
            attributes[i + common.length][0] = specialFields[i];
            attributes[i + common.length][1] = specialValues[i];
        }

        return attributes;
    }

    public String getToolTipText() {
        return "<html><b>ID:</b> " + getId() + "<br><b>Category ID:</b> " + getParent_id() + "<br><b>Type:</b> " + getSpecialDBname() + "<br><b>Value:</b> " + getValue();
    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);

        this.setParent_id((Integer)attributes.get("QuestionCategoryID"));
        this.setText((String)attributes.get("Text"));
        this.setLatex((String)attributes.get("LaTeX"));
        this.setValue((Integer)attributes.get("Value"));
        this.setComment((String)attributes.get("Comment"));
    }

    public String getStatusName() {
        return "Question";
    }
}
