package sys.datatypes.question;

import sys.DefaultValues;
import sys.Parser.Parser;
import sys.datatypes.Group;
import sys.datatypes.GroupElement;
import sys.datatypes.MapPosition;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.interfaces.MultiFormat;
import sys.models.LevelModel;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * @author Eivind Sorteberg - 19.jun.2006
 */
public class QuestionCategory extends Group implements MapItem, MultiFormat {
    public static final String DB_NAME = "QuestionCategory";
    public static QuestionCategory defaultCategory;

    private String text;
    private String latex;
    private int questionsRequired;
    private int level;
    private MapPosition mapPosition;
    private LevelModel levelModel;

    public QuestionCategory(int id, int parent, String name, String description, String text, String latex, int questionsRequired, int level, MapPosition mapPosition) {
        super(id, parent, name, description);

        this.text = text;
        this.latex = latex;
        this.questionsRequired = questionsRequired;
        this.level = level;
        this.mapPosition = mapPosition;
        levelModel = null;
    }

    public QuestionCategory(int levelId, MapPosition mapPosition) {
        super(-1, DefaultValues.QUESTION_GROUP, "New Category", "");
        this.text = "";
        this.latex = "";
        this.questionsRequired = 0;
        this.level = levelId;
        this.mapPosition = mapPosition;
    }

    public QuestionCategory() {
        super(-1, DefaultValues.QUESTION_GROUP, "New Category", "");
        this.text = "";
        this.latex = "";
        this.questionsRequired = 0;
        this.level = 100;
        this.mapPosition = new MapPosition();
    }

    public QuestionCategory(int parent_id) {
        this(-1, parent_id, "New Category", "", "", "", 0, 100, new MapPosition());
    }

    public String getText() {
        return text ==null ? "" : text;
    }

    public String getLatex() {
        return latex==null ? "" : latex;
    }

    public int getQuestionsRequired() {
        return questionsRequired;
    }

    public String getToolTipText() {
        return "<html>" +
                    "<b>ID:</b> " + getId() + "<br>" +
                    "<b>Name:</b> " + getName() + "<br>" +
                    "<b>Group ID:</b> " + getParent_id() + "<br>" +
                    "<b>Questions Required:</b> " + getQuestionsRequired() + "<br>" +
                    "<b>Level ID:</b> " + getLevelModel().getId() + "<br>" +
                "</html>";
    }


    public int getLevel() {
        return level;
    }

    public void setText(String html) {
        this.text = html;
    }

    public void setLatex(String latex) {
        this.latex = latex;
    }

    public void setQuestionsRequired(int questionsRequired) {
        this.questionsRequired = questionsRequired;
    }


    /**
     * Overrides GroupElement.delete().
     * Makes sure all the object's children are moved to another
     * parent, and that the object itself
     * is deleted from both its textgroup and
     * its LevelModel.
     */
    public void delete() {

        GroupElement[] copy = new GroupElement[getGroupElements().size()];
        ArrayList<AoCObject> toStore = new ArrayList<AoCObject>();

        getGroupElements().toArray(copy);

        for (GroupElement child : copy) {
            child.setParent(defaultCategory);
            toStore.add((AoCObject)child);
        }

        Parser.updateObjects(toStore);

        this.getParent().removeElement(this);
        this.getLevelModel().removeElement(this);
    }

    public static void setDefaultCategory(QuestionCategory category) {
        defaultCategory = category;
    }



    /* ----- Methods from interface MapItem ----- */


    public void moveTo(int x, int y) {
        this.mapPosition.changePosition(x, y);
    }


    public void moveTo(int x, int y, int z) {
        this.mapPosition.changePosition(x, y, z);
    }


    public MapPosition getMapPosition() {
        return mapPosition;
    }


    public void setMapPosition(MapPosition mapPosition) {
        this.mapPosition = mapPosition;
    }


    public void setLevelModel(LevelModel lm) {
        try {
            this.levelModel.removeElement(this);
        } catch (NullPointerException e) {
        }
        this.levelModel = lm;
        this.levelModel.addElement(this);
        this.level = levelModel.getId();
    }


    public LevelModel getLevelModel() {
        return levelModel;
    }



    /* ----- Methods from interface AoCObject ----- */


    public String getDBname() {
        return DB_NAME;
    }


    public String[] getFieldNames() {
        return new String[]{
                "QuestionGroupID",
                "Name",
                "Description",
                "Text",
                "LaTeX",
                "QuestionsRequired",
                "LevelID",
                "PosX",
                "PosY",
                "PosZ"};
    }


    public Object[] getFieldValues() {

        return new Object[]{this.getParent_id(),
                            this.getName(),
                            this.getDescription(),
                            this.getText(),
                            this.getLatex(),
                            this.getQuestionsRequired(),
                            this.getLevel(),
                            this.getMapPosition().getX(),
                            this.getMapPosition().getY(),
                            this.getMapPosition().getZ()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
        super.setAllAttributes(attributes);
        this.setParent_id((Integer)attributes.get("QuestionGroupID"));
        this.setDescription((String)attributes.get("Description"));
        this.setText((String)attributes.get("Text"));
        this.setLatex((String)attributes.get("LaTeX"));
        this.setQuestionsRequired((Integer)attributes.get("QuestionsRequired"));
        this.setLevel_id((Integer)attributes.get("LevelID"));

        this.setMapPosition(new MapPosition(
                ((Integer)attributes.get("PosX")),
                ((Integer)attributes.get("PosY")),
                ((Integer)attributes.get("PosZ"))
        ));
    }

    private void setLevel_id(int level_id) {
        this.level = level_id;
    }

    public AoCObject makeClone() {
        return new QuestionCategory(this.getId(),
                                    this.getParent_id(),
                                    this.getName(),
                                    this.getDescription(),
                                    this.getText(),
                                    this.getLatex(),
                                    this.getQuestionsRequired(),
                                    this.getLevel(),
                                    new MapPosition(
                                            this.getMapPosition().getX(),
                                            this.getMapPosition().getY(),
                                            this.getMapPosition().getZ()
                                    ));
    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof QuestionCategory)) return true;

        QuestionCategory qc2 = (QuestionCategory)object;

        return this.getId() != qc2.getId() ||
                this.getParent_id() != qc2.getParent_id() ||
                !this.getName().equals(qc2.getName()) ||
                !this.getDescription().equals(qc2.getDescription()) ||
                !this.getText().equals(qc2.getText()) ||
                !this.getLatex().equals(qc2.getLatex()) ||
                this.getQuestionsRequired() != qc2.getQuestionsRequired() ||
                this.getLevel() != qc2.getLevel() ||
                this.getMapPosition().differsFrom(qc2.getMapPosition());

    }

    public String getStatusName() {
        return "Question Category";
    }
}
