package sys.models;

import gui.editors.map.MapPanel;
import gui.editors.map.MapViewPanel;
import sys.DefaultValues;
import sys.datatypes.*;
import sys.datatypes.interfaces.AoCObject;
import sys.datatypes.interfaces.MapItem;
import sys.datatypes.question.QuestionCategory;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * Model for a level.
 * A level consists of a collection of question groups, texts, messages and a map,
 * and this model represents that collection.
 *
 * @author Tarje Killingberg - 19.jun.2006
 */
public class LevelModel extends Group implements AoCObject {

    private HashMap<Integer, LevelModel> elements;

    private Level level;
    private MapPanel listener;

    public LevelModel(int id, int parentId, Level level) {
        super(id, parentId, level.getName(), "");
        this.level = level;
        setName(level.getName());
        level.setLevelModel(this);
        this.elements = new HashMap<Integer, LevelModel>();
    }

    public String getToolTipText() {
        return getLevelObject().getToolTipText();
    }


    public String getName() {
        return level.getName();
    }


    public Level getLevelObject() {
        return level;
    }


    public MapPanel getListener() {
        return listener;
    }

    public void setListener(MapPanel listener) {
        this.listener = listener;
    }


    public LevelModel getLevelModel(int key) {
        return elements.get(key);
    }


    public HashMap<Integer, LevelModel> getElements() {
        return elements;
    }


    public String toString() {
        return getLevelObject().toString();
    }


    @Override
    public boolean addElement(GroupElement element) {
        boolean b = super.addElement(element);
        if (element instanceof LevelModel) {
            LevelModel lm = (LevelModel)element;
            elements.put(lm.getLevelObject().getId(), lm);
        }
        return b;
    }


    public Object[] fixDependencies() {
        GroupElement[] copy = new GroupElement[getGroupElements().size()];
        getGroupElements().toArray(copy);

        ArrayList<AoCObject> toStore = new ArrayList<AoCObject>();
        ArrayList<AoCObject> toDelete = new ArrayList<AoCObject>();

        for (GroupElement child : copy) {
            if (child instanceof Message || child instanceof TeleportSource ||
                                            child instanceof TeleportDestination) {

                child.delete();
                toDelete.add((AoCObject)child);

            }
            else if (child instanceof Text || child instanceof QuestionCategory) {
                MapItem mapItem = (MapItem)child;
                LevelModel defaultLevel;
                if (child instanceof Text) {
                    defaultLevel = ((LevelModel)getParent()).getElements().get(DefaultValues.MAPITEM_LEVEL_ID);
                }
                else {
                    defaultLevel = ((LevelModel)getParent()).getElements().get(DefaultValues.MAPITEM_LEVEL_ID);
                }
                mapItem.setLevelModel(defaultLevel);
                toStore.add((AoCObject)mapItem);
            }
            else {
            }
        }

        return new Object[]{toDelete, toStore};
    }



    @Override
    public void delete() {
        try {
            this.getParent().removeElement(this);
        } catch (NullPointerException npe) {
            npe.printStackTrace();
        }
    }


    @Override
    public boolean removeElement(GroupElement ge) {
        boolean b = super.removeElement(ge);
        if (elements.get(ge.getId()) != null) {
            elements.remove(ge.getId());
        }
        return b;
    }


    public void setName(String name) {
        super.setName(name);
        getLevelObject().setName(name);
    }


    public void setMap(String mapFile) {
        if (getLevelObject().setMap(mapFile)) {
            fireModelChanged(MapViewPanel.MAP_CHANGED);
        }
    }


    public String getMap() {
        return getLevelObject().getMap();
    }


    public void fireModelChanged(int event) {
        try {
            listener.levelModelChanged(event);
        } catch (NullPointerException npe) {
            npe.printStackTrace();
        }
    }





    /* ----- Methods from interface AoCObject ----- */


    public String getDBname() {
        return Level.DB_NAME;
    }


    public String[] getFieldNames() {

        return new String[]{"Name",
                            "Map"};

    }


    public Object[] getFieldValues() {

        return new Object[]{level.getName(),
                            level.getMap()};

    }

    public void setAllAttributes(HashMap<String, Object> attributes) {
    }


    public AoCObject makeClone() {

        return new LevelModel(this.getId(),
                              this.getParent_id(),
                              new Level(getLevelObject().getId(),
                                        getLevelObject().getName(),
                                        getLevelObject().getMap()));

    }


    public boolean differsFrom(AoCObject object) {
        if (!(object instanceof LevelModel)) return true;

        LevelModel lm2 = (LevelModel)object;

        return  this.getId() != lm2.getId() ||
                !this.getName().equals(lm2.getName()) ||
                this.getLevelObject().getId() != lm2.getLevelObject().getId() ||
                !this.getLevelObject().getName().equals(lm2.getLevelObject().getName()) ||
                !this.getLevelObject().getMap().equals(lm2.getLevelObject().getMap());

    }

    public String getStatusName() {
        return "Level";
    }
}
