package sys.models.treemodels;

import sys.datatypes.*;
import sys.models.ModelManager;

import javax.swing.*;
import javax.swing.tree.TreePath;

/**
 * Model for the TextEditor-view.
 *
 * @author Tarje Killingberg - 19.jun.2006
 * @see AbstractTreeModel
 */
public class TextTreeModel extends AbstractTreeModel {
    public static final String group = "Text Group";
    public static final String history = "History";
    public static final String fact = "Fact";

    public TextTreeModel(Group root, ModelManager modelManager) {
        super(root, modelManager);
    }

    public Object[] getObjectNames() {
        return new Object[]{group, new JSeparator(), history, fact};
    }

    public void createObject(String type) {
        GroupElement object;

        if (type.equals(group))             object = new TextGroup();
        else if (type.equals(history))      object = new History();
        else if (type.equals(fact))         object = new Fact();
        else {
            System.err.println("ERROR! Illegal text type; found: " + type);
            return;
        }

        Boolean visible = filters.get(type);

        if (visible != null) {
            object.setVisible(visible);
        }

        TreePath selectionPath = tree.getSelectionPath();
        if (selectionPath != null) setParent(selectionPath, object);

        modelManager.fireObjectCreated(object);
    }

    private void setParent(TreePath selectionPath, GroupElement newObject) {
        GroupElement node = (GroupElement)selectionPath.getLastPathComponent();

        while(node.getParent() != null) {
            if (newObject instanceof Text || newObject instanceof TextGroup) {
                if (node instanceof Group) {
                    newObject.setParent_id(node.getId());
                    return;
                }
            }
            node = node.getParent();
        }
    }

    public String[] getFilterNames() {
        return new String[0];

    }

    public void fireFilterSelected(String filter, boolean value) {
    }
}
